package com.mile.v4_navigation_engine

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.annotation.SuppressLint
import android.app.Activity
import android.app.Application
import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Color
import android.graphics.RectF
import android.graphics.drawable.GradientDrawable
import android.location.Location
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.View
import android.view.animation.AnimationUtils
import android.widget.*
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.coordinatorlayout.widget.CoordinatorLayout
import androidx.core.content.ContextCompat
import androidx.core.graphics.drawable.DrawableCompat
import com.bumptech.glide.Glide
import com.devzone.fillprogresslayout.FillProgressLayout
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.gson.Gson
import com.jaygoo.widget.RangeSeekBar
import com.mapbox.android.core.location.*
import com.mapbox.android.core.permissions.PermissionsManager
import com.mapbox.android.gestures.MoveGestureDetector
import com.mapbox.api.directions.v5.DirectionsCriteria
import com.mapbox.api.directions.v5.models.DirectionsResponse
import com.mapbox.api.directions.v5.models.DirectionsRoute
import com.mapbox.api.directions.v5.models.RouteLeg
import com.mapbox.api.directions.v5.models.RouteOptions
import com.mapbox.api.matching.v5.MapboxMapMatching
import com.mapbox.api.matching.v5.models.MapMatchingMatching
import com.mapbox.api.matching.v5.models.MapMatchingResponse
import com.mapbox.geojson.Feature
import com.mapbox.geojson.FeatureCollection
import com.mapbox.geojson.LineString
import com.mapbox.geojson.Point
import com.mapbox.mapboxsdk.Mapbox
import com.mapbox.mapboxsdk.camera.CameraPosition
import com.mapbox.mapboxsdk.camera.CameraUpdate
import com.mapbox.mapboxsdk.camera.CameraUpdateFactory
import com.mapbox.mapboxsdk.geometry.LatLng
import com.mapbox.mapboxsdk.geometry.LatLngBounds
import com.mapbox.mapboxsdk.location.LocationComponentActivationOptions
import com.mapbox.mapboxsdk.location.LocationComponentOptions
import com.mapbox.mapboxsdk.location.modes.CameraMode
import com.mapbox.mapboxsdk.location.modes.RenderMode
import com.mapbox.mapboxsdk.maps.MapView
import com.mapbox.mapboxsdk.maps.MapboxMap
import com.mapbox.mapboxsdk.maps.OnMapReadyCallback
import com.mapbox.mapboxsdk.maps.Style
import com.mapbox.mapboxsdk.plugins.annotation.CircleManager
import com.mapbox.mapboxsdk.plugins.annotation.Symbol
import com.mapbox.mapboxsdk.plugins.annotation.SymbolManager
import com.mapbox.mapboxsdk.plugins.localization.LocalizationPlugin
import com.mapbox.mapboxsdk.style.layers.*
import com.mapbox.mapboxsdk.style.sources.GeoJsonSource
import com.mapbox.services.android.navigation.ui.v5.camera.DynamicCamera
import com.mapbox.services.android.navigation.ui.v5.camera.NavigationCamera
import com.mapbox.services.android.navigation.ui.v5.camera.NavigationCameraUpdate
import com.mapbox.services.android.navigation.ui.v5.instruction.InstructionView
import com.mapbox.services.android.navigation.ui.v5.listeners.NavigationListener
import com.mapbox.services.android.navigation.ui.v5.map.NavigationMapboxMap
import com.mapbox.services.android.navigation.ui.v5.voice.NavigationSpeechPlayer
import com.mapbox.services.android.navigation.ui.v5.voice.SpeechAnnouncement
import com.mapbox.services.android.navigation.ui.v5.voice.SpeechPlayerProvider
import com.mapbox.services.android.navigation.ui.v5.voice.VoiceInstructionLoader
import com.mapbox.services.android.navigation.v5.location.replay.ReplayRouteLocationEngine
import com.mapbox.services.android.navigation.v5.milestone.BannerInstructionMilestone
import com.mapbox.services.android.navigation.v5.milestone.Milestone
import com.mapbox.services.android.navigation.v5.milestone.MilestoneEventListener
import com.mapbox.services.android.navigation.v5.milestone.VoiceInstructionMilestone
import com.mapbox.services.android.navigation.v5.navigation.MapboxNavigation
import com.mapbox.services.android.navigation.v5.navigation.NavigationConstants
import com.mapbox.services.android.navigation.v5.navigation.NavigationRoute
import com.mapbox.services.android.navigation.v5.offroute.OffRouteListener
import com.mapbox.services.android.navigation.v5.routeprogress.ProgressChangeListener
import com.mapbox.services.android.navigation.v5.routeprogress.RouteProgress
import com.mapbox.services.android.navigation.v5.utils.DistanceFormatter
import com.mapbox.services.android.navigation.v5.utils.RouteUtils
import com.mapbox.turf.TurfMeasurement
import com.mapbox.turf.TurfTransformation
import com.mile.v4_navigation_engine.MileNavigationEnginePlugin.Companion.CREATED
import com.mile.v4_navigation_engine.MileNavigationEnginePlugin.Companion.DESTROYED
import com.mile.v4_navigation_engine.MileNavigationEnginePlugin.Companion.PAUSED
import com.mile.v4_navigation_engine.MileNavigationEnginePlugin.Companion.RESUMED
import com.mile.v4_navigation_engine.MileNavigationEnginePlugin.Companion.STARTED
import com.mile.v4_navigation_engine.MileNavigationEnginePlugin.Companion.STOPPED
import com.mile.v4_navigation_engine.Navigation.ApplicationRunner
import com.mile.v4_navigation_engine.Navigation.model.OfflineDatas
import com.mile.v4_navigation_engine.Navigation.model.WhiteLabel
import com.mile.v4_navigation_engine.Navigation.service.AudioService
import com.mile.v4_navigation_engine.Navigation.utils.OfflineUtils
import com.mile.v4_navigation_engine.interfaces.MP3Listener
import com.mile.v4_navigation_engine.model.*
import com.mile.v4_navigation_engine.model.LanguageCode.Companion.getLanguageCodeFrom
import com.mile.v4_navigation_engine.utils.*
import com.ncorti.slidetoact.SlideToActView
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import me.grantland.widget.AutofitTextView
import okhttp3.Cache
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import timber.log.Timber
import java.io.File
import java.lang.ref.WeakReference
import java.util.*
import java.util.concurrent.atomic.AtomicInteger
import kotlin.collections.ArrayList
import kotlin.collections.HashMap
import kotlin.collections.LinkedHashMap

class
MileNavigationEngine2 : Application.ActivityLifecycleCallbacks,
        MethodChannel.MethodCallHandler,
        PlatformView,
        MapboxMap.OnMapLongClickListener,
        OnMapReadyCallback,
        MapboxMap.OnMapClickListener,
        ProgressChangeListener,
        MilestoneEventListener,
        OffRouteListener,
        NavigationListener,
        MP3Listener,
        SlideToActView.OnSlideCompleteListener {

    var _activity: Activity
    var _context: Context
    private var disposed = false
    private var activityState: AtomicInteger? = null
    private var registrarActivityHashCode = 0

    private var mainView: View

    private var mapReadyResult: MethodChannel.Result? = null

    var flutterDistanceRemaining: Double? = null
    var flutterDistanceTravelled: Double = 0.0

    var _durationRemaining: Double? = null

    var route = ""
    var gpsColor = ""
    var accessToken = ""
    var mode = ""
    var distanceUnit = ""
    var locomotion = ""
    var downloadForOffline = false
    var whiteLabel: WhiteLabel? = null
    var offlineDatas: OfflineDatas? = null
    var sponsorData: Map<*,*> = HashMap<String, Any>()

    private var methodChannel: MethodChannel? = null

    var id: Int = -1

    var PERMISSION_REQUEST_CODE: Int = 367

    constructor(id: Int, context: Context, messenger: BinaryMessenger, activity: Activity, route: String, gpsColor: String, accessToken: String, mode: String, languages: ArrayList<String>, sponsor: String, distanceUnit: String, speedUnit: String, locomotion: String, whiteLabel: String, downloadForOffline: Boolean, offlineDatas: String, state: AtomicInteger) {
        this.id = id
        this._context = context
        this._activity = activity;
        this.route = route
        this.gpsColor = gpsColor
        this.accessToken = accessToken
        this.mode = mode
        this.distanceUnit = distanceUnit
        this.locomotion = locomotion
        this.gpsColor = gpsColor
        this.downloadForOffline = downloadForOffline
        this.whiteLabel = WhiteLabel.fromFlutter(whiteLabel)
        this.offlineDatas = OfflineDatas.fromFlutter(offlineDatas)
        ApplicationRunner.appDistanceMetrics = MetricsCode.getDistanceMetricFromSettings(distanceUnit)
        ApplicationRunner.appSpeedMetrics = MetricsCode.getSpeedMetricFromSettings(speedUnit)
        ApplicationRunner.appLanguage = getLanguageCodeFrom(languages[0])
        this.activityState = state
        this.registrarActivityHashCode = _activity.hashCode()

        this.methodChannel = MethodChannel(messenger, "flutter_mapbox_navigation_"+id)
        this.methodChannel!!.setMethodCallHandler(this)

        if(this.gpsColor == "") { //RC4
            _activity.setTheme(R.style.CustomInstructionView)
        } else {
            _activity.setTheme(R.style.CustomInstructionWhiteLabelView)
        }

        Mapbox.getInstance(_activity, accessToken)
        flutterRouteToKtRoute(route, gpsColor, this.whiteLabel!!.isWhiteLabel!!)
        if(!this.whiteLabel!!.isWhiteLabel!!) {
            flutterSponsorToMap(sponsor) //display sponsor only if not white label
        }

        var inflater = _activity.layoutInflater;
        mainView = inflater.inflate(R.layout.activity_component_navigation2, null)

        //OfflineRouteManager.fetchTileVersions(_context)

        this.callback = MileNavigationEngine2.ComponentActivityLocationCallback(this)

        mapView = view.findViewById(R.id.mapView)

        navigationLayout = view.findViewById(R.id.componentNavigationLayout)
        instructionView = view.findViewById(R.id.instructionView)
        instructionView.findViewById<TextView>(R.id.stepPrimaryText).textSize = 23f
        instructionView.findViewById<TextView>(R.id.stepPrimaryText).maxLines = 3

        componentNavigationLayout = view.findViewById( R.id.componentNavigationLayout );
        //toolbar = view.findViewById( R.id.toolbar );
        instructionViewNoNavigation = view.findViewById( R.id.instructionViewNoNavigation );
        maneuverImage = view.findViewById( R.id.maneuver_image );
        maneuverInstruction = view.findViewById( R.id.maneuver_instruction );
        coordinatorBannerPoi = view.findViewById( R.id.coordinator_banner_poi );
        containerGooddeal = view.findViewById( R.id.container_gooddeal );
        imageBannerPoi = view.findViewById( R.id.image_banner_poi );
        titleBannerPoi = view.findViewById( R.id.title_banner_poi );
        descriptionBannerPoi = view.findViewById( R.id.description_banner_poi );
        linkGooddeal = view.findViewById( R.id.link_gooddeal );
        menuLayout = view.findViewById( R.id.menu_layout );
        statsLayout = view.findViewById( R.id.stats_layout );
        distanceContainer = view.findViewById( R.id.distance_container );
        distanceLeftTextview = view.findViewById( R.id.distance_left_textview );
        timerTextview = view.findViewById( R.id.timer_textview );
        speedTextview = view.findViewById( R.id.speed_textview );
        resumeButton = view.findViewById( R.id.resume_button );
        finishButton = view.findViewById( R.id.finish_button );
        mp3Fillprogresslayout = view.findViewById( R.id.mp3_fillprogresslayout );
        mp3TimeTextview = view.findViewById( R.id.mp3_time_textview );
        mp3FillprogresslayoutStatic = view.findViewById( R.id.mp3_fillprogresslayout_static );
        mp3TimeTextviewStatic = view.findViewById( R.id.mp3_time_textview_static );
        titleStatsResume = view.findViewById( R.id.title_stats_resume );
        icFlagVariantOutline = view.findViewById( R.id.ic_flag_variant_outline );
        lengthTextview = view.findViewById( R.id.length_textview );
        icTimer = view.findViewById( R.id.ic_timer );
        durationTextview = view.findViewById( R.id.duration_textview );
        icSpeed = view.findViewById( R.id.ic_speed );
        speedTextviewStatic = view.findViewById( R.id.speed_textview_static );
        titleExperienceResume = view.findViewById( R.id.title_experience_resume );
        recenterButton = view.findViewById( R.id.recenter_button );
        pauseSwiper = view.findViewById( R.id.pause_swiper);
        navigationPauseScreen = view.findViewById(R.id.navigation_pause_screen);
        rangeProgress = view.findViewById(R.id.range_progress);
        greyBackground = view.findViewById(R.id.greybackground);
        menufab = view.findViewById(R.id.menu_fab)
        reportFab = view.findViewById(R.id.report_fab)
        soundFab = view.findViewById(R.id.sound_fab)
        emergencyFab = view.findViewById(R.id.emergency_fab)
        musicFab = view.findViewById(R.id.music_fab)
        sponsorLogo = view.findViewById(R.id.sponsor_logo)
        sponsorView = view.findViewById(R.id.sponsor_view)
        audioSwitch = view.findViewById( R.id.audio_switch ) //In pause View

        mapView.getMapAsync(this)

        AudioService.getInstance().init(_context, _activity, this.methodChannel)

        if(this.whiteLabel!!.isWhiteLabel!!) {
            musicFab!!.hide()
        }

        if(this.gpsColor != "") {
            setFlutterCustomColor();
        } else {
            //Must set green RC color here because if set in xml cannot be change else and I need for white label
            recenterButton!!.backgroundTintList = ColorStateList.valueOf(_activity.resources.getColor(R.color.colorPrimary))
            recenterButton!!.setBackgroundColor(_activity.resources.getColor(R.color.colorPrimary))
        }
        if(!sponsor.equals("")) {
            sponsorView!!.visibility = View.VISIBLE
            Glide.with(_activity.applicationContext).load(this.sponsorData["LogoUrl"]).into(sponsorLogo!!)
        } else {
            sponsorView!!.visibility = View.GONE
        }
        initView()
        initListeners()
        initFabMenu()

        //offlineParsingTest();

        showNavigationHolderLayout(false)
        //manageVisibilityMenuLayout(false)
        showStatsLayout(true)
    }

    fun offlineParsingTest() {
        var json: String = "{\"routes\":[{\"geometry\":\"_z}ivA{`rcHnHqElHo@Z}BhA_C~j@q_@j_@kX_g@gcAjEcInC{JpDaVJyRAaBEmBmBsfBy@ov@hAk]~Ai]fEwPdF{Mlh@qnA|EaLvGmOtaA{rAtQeP~V}MnLaDdMiCvJe@jLIzI`@MvD_@pMqAQf@qMd@cMMqBf@oIb@gMTaJK{E_@sEuAiFwBiEcH}ImGaKeDuIyAyH{@mHyAcSi@qKS}JHsEdAeKz@eARoAv@qEzCiG@eEpAsJWmTx@iJbD}V~APbCs@v@mAlMuRzKyR|ByG|@mFxb@ev@nAwBh@kCvKiSfAoB|AqCbQiZtWue@j@gAh@oBpF|CnUvM~EvD~I~G~h@na@db@t[pb@`\\\\|RnPjErD`DuCnh@ra@pQnNlO~K~TzRzJjJfF`ErE`DbAt@xClApClDxJjF~F`Edn@v[xj@lXtCv@jjAzLjAmApBBlLk@xIy@rGgBdkAe]`LiEnCeA`B_Ar@}Aj@aCtAMfDRnCtAhA}E?kBo@iC~GafBx@_SlOgHhDwCvGkHfKqGlS}e@lCsGzCkHjJ}OpDqEhN}Oz@_A~QwPdDyCbE{B`UqJjGwAfFwBfFwBx@]bEeBjDuBpKsDn]wG`QwExB_@bCQhFi@bD_@zIuAnD{AhIaE|KsFvCwAlJuE`]uPj@_@`KsGdAyAtKgO`CmE`Oy]fBiFp@yChCmNbFwZvBsMj@sDb@_D|@eJh@yGrEck@RgHAkCAiCc@oKeAmMqCeZiF_n@]qDhBm@f@Q~q@oUtBeBg@wCoNc{@oGgZ{BwLw@iI]uMQcPE}Co@_AWmYh@iAGcEqAujAiBwhAFuFV{CvEw^fB_LvBeKjCaJlBsFd@sAtW{b@`@cCnMuR|DeE\\\\_A|@qAtAqAhBiBdEwAfh@cMdF]lb@_AfFoCbFgF~C_B|D[jw@g@pHE|EEbG^z@FrDj@`Fx@z[jI~H~C|ShLdJhCjG`B`e@zLrXnG|Aj@pA`@jSxIjM~GlCf@r@VhBTnBrAcAnC}A|GqP`z@qBjK_HzCUb~@qFFyc@wK`@_Ex@_I\",\"legs\":[{\"annotation\":{\"distance\":[18.76529857906409,16.89799292440997,5.128023751772538,6.447637112534622,88.11376213258856,65.65796641593562,110.6331321907509,16.92812110392269,16.77095516009727,30.28148904869304,24.593973002958275,3.8018197514401417,4.278553347796795,128.73146556565854,68.94419784076621,37.91566075341864,37.99113471769659,24.674703498204845,22.456910286819816,123.22878944943547,20.375637315471153,25.661813280510035,157.85311651386667,39.50836475615437,46.55989335625173,24.832680646181284,25.809296889152293,20.96240599133063,23.80558709555054,19.39823900150363,7.177904592446133,18.158974489757355,4.61339374745216,18.207947804967652,17.655565092815348,4.488974473940388,13.218660050142176,17.796734589700673,13.782600122690745,8.557703571386211,8.411804645175879,10.257783830606451,10.290855873525132,21.16443454711681,21.20235007467364,16.159380124894195,13.165409792338071,12.177580974476799,25.47080052346593,15.76347569561847,14.855562951956133,8.24009571281366,15.617089741501603,4.301526885692034,3.2957421901469584,8.718941363283204,13.478695951267277,7.679221751921583,15.129742148998595,26.63647586302943,14.404127739312395,31.074069190042874,5.384302315304419,7.612874606139116,4.34151813010296,35.454795529167804,33.60795508664763,12.988431459441237,9.852764848183977,93.55356876930303,6.438246292349465,5.910438946462933,33.915598670780135,5.907535932623962,7.706285333827086,46.789743307206194,65.07937846727947,3.7127474040846353,4.931739821216096,14.787639772925385,44.027179984495135,14.356401678516145,22.538076419035022,86.13862690686359,72.03374666291884,72.84413431456639,41.60071165526647,13.320887049667126,10.724364111487436,85.4464699086922,38.22723786118544,33.40698960047706,46.27436424280378,25.41481972131118,14.935963935188358,13.359722932832113,4.32292831296952,9.083032090905546,10.55796295100912,22.928112574401652,16.103023808689848,91.24259723954198,84.11928998867182,8.620109102110607,135.23988337790402,5.197815176787998,6.341797222278124,23.974476574905665,19.373213666189063,15.870456936387493,140.66702491628584,24.531257048928307,8.456073027970278,5.9888403454862456,4.653755495739969,5.604916483648956,4.813461610334628,9.37516157580547,8.675292608986293,9.544033379862313,4.189220036458855,5.981580580436068,128.92522916487036,25.033741848780476,31.42510826630341,11.142048100065658,19.439478911552598,24.253189851680897,60.48831902627795,13.303473552687935,14.515040520298388,29.185678790503566,12.819873661644204,34.4181450520825,4.159011607038049,40.35786965375555,10.995978463595623,11.914293066892121,41.80396305258977,15.290245122416389,13.716283988866149,13.716287273864506,3.4290723326845503,11.596094457997495,10.604264643839056,23.421510757867345,55.35453189651351,33.218535343021166,6.897427666039709,7.374069587429813,13.11507183475369,9.204645879775873,19.638813506260988,10.418268905506718,19.835452943593644,24.89359775139493,9.116486475214113,21.982259253321587,57.83025028759409,2.7438542515320723,23.988683607639302,5.229218855306511,30.27775061868255,10.776470406794333,47.751607124869295,10.763706152889956,6.589662787872921,20.6437501602522,36.70843629566669,19.343251367413938,7.399245278923549,6.521934935434149,14.309873574427048,11.186488153704174,56.03168026511428,11.536800008018586,5.4323154050579125,5.354743572542494,15.646264405903167,18.34077785169313,34.7137581492484,59.77980394342772,7.104013728966528,6.159177856643233,2.3315503827291315,94.96150411901071,7.663072464919481,6.3024204764196075,79.5745241656551,37.0567041281027,18.409853863017815,13.175470957617081,18.30946901465359,21.28279863414053,6.138565051062404,3.645587784588946,32.8470340803761,3.7009554427612144,7.616672776395618,94.07017122029542,91.74384004021132,9.553727815591511,6.197324610470742,41.2047819168205,17.143521410366475,16.536226691736786,15.78670488543554,11.27054143510395,3.8839879004931372,62.559566800251936,5.458839219014336,35.541819366676144,13.063485416632895,2.9913565874615404,4.691349098451394,5.744081748231925,7.187609533458265,11.528494053832194,75.4746795412264,12.843866106706633,63.1141707534352,14.059820970311995,15.549599038973568,9.64612300939493,10.622190426854672,100.33812416834573,17.01921592441426,12.348313628268423,14.512620155335425,3.351193105788657,10.154879088605252,12.768422211069913,52.97633595575692,18.847806995829625,40.76199300466078,20.616869326580495,15.381670812201683,69.89298560208516,46.80796483885752,5.499323514272481,4.747245395820503,38.66508148751226,27.916020491692272,8.04812444145178,3.038108301999047,5.956472739078937,7.0298292874772565,6.746650535817494,12.266381902035393,79.71366223094765,16.621154696377108,17.122090534667613,78.38386896143474,13.461961147538892,67.39768000506967,7.685574603246419,12.827740305183918],\"congestion\":[\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\",\"unknown\"]},\"summary\":\"Rue des Contrebandiers, Quai Saint-Vincent\",\"weight\":1954.7,\"duration\":1855.2,\"steps\":[{\"intersections\":[{\"out\":0,\"entry\":[true],\"bearings\":[154],\"location\":[4.793886,45.792688]}],\"driving_side\":\"right\",\"geometry\":\"_z}ivA{`rcHnHqElHo@\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":154,\"bearing_before\":0,\"location\":[4.793886,45.792688],\"modifier\":\"left\",\"type\":\"depart\",\"instruction\":\"Neem Rue Jean-Claude Bartet in zuidoostelijke richting\"},\"weight\":11.7,\"duration\":11.7,\"name\":\"Rue Jean-Claude Bartet\",\"distance\":35.7,\"voiceInstructions\":[{\"distanceAlongGeometry\":35.7,\"announcement\":\"Neem Rue Jean-Claude Bartet in zuidoostelijke richting, daarna sla links om op Rue Jean-Claude Bartet te blijven\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Neem Rue Jean-Claude Bartet in zuidoostelijke richting, daarna sla links om op Rue Jean-Claude Bartet te blijven</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":35.7,\"primary\":{\"text\":\"Rue Jean-Claude Bartet\",\"components\":[{\"text\":\"Rue Jean-Claude Bartet\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":0,\"entry\":[false,true,true],\"bearings\":[0,120,240],\"location\":[4.794015,45.792385]}],\"driving_side\":\"right\",\"geometry\":\"ag}ivA}hrcHZ}BhA_C~j@q_@j_@kX\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":118,\"bearing_before\":172,\"location\":[4.794015,45.792385],\"modifier\":\"left\",\"type\":\"continue\",\"instruction\":\"Sla links om op Rue Jean-Claude Bartet te blijven\"},\"weight\":50.2,\"duration\":50.2,\"name\":\"Rue Jean-Claude Bartet\",\"distance\":165.3,\"voiceInstructions\":[{\"distanceAlongGeometry\":145.3,\"announcement\":\"Over 500 voet, ga linksaf naar Rue Félix Faure\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 500 voet, ga linksaf naar Rue Félix Faure</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":49.4,\"announcement\":\"Ga linksaf naar Rue Félix Faure, daarna ga rechtsaf naar Avenue de Lanessan (D 306)\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga linksaf naar Rue Félix Faure, daarna ga rechtsaf naar Avenue de Lanessan (D <say-as interpret-as=\\\"address\\\">306</say-as>)</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":165.3,\"primary\":{\"text\":\"Rue Félix Faure\",\"components\":[{\"text\":\"Rue Félix Faure\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"},\"secondary\":null},{\"distanceAlongGeometry\":49.4,\"primary\":{\"text\":\"Rue Félix Faure\",\"components\":[{\"text\":\"Rue Félix Faure\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"},\"secondary\":null,\"sub\":{\"text\":\"Avenue de Lanessan\",\"components\":[{\"text\":\"Avenue de Lanessan\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"}}]},{\"intersections\":[{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[45,150,330],\"location\":[4.795069,45.791112]}],\"driving_side\":\"right\",\"geometry\":\"owzivAyjtcH_g@gcA\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":49,\"bearing_before\":150,\"location\":[4.795069,45.791112],\"modifier\":\"left\",\"type\":\"turn\",\"instruction\":\"Ga linksaf naar Rue Félix Faure\"},\"weight\":29.4,\"duration\":29.4,\"name\":\"Rue Félix Faure\",\"distance\":110.6,\"voiceInstructions\":[{\"distanceAlongGeometry\":56.4,\"announcement\":\"Ga rechtsaf naar Avenue de Lanessan (D 306)\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Avenue de Lanessan (D <say-as interpret-as=\\\"address\\\">306</say-as>)</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":110.6,\"primary\":{\"text\":\"Avenue de Lanessan\",\"components\":[{\"text\":\"Avenue de Lanessan\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":{\"text\":\"D 306\",\"components\":[{\"text\":\"D 306\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"}}]},{\"intersections\":[{\"out\":0,\"in\":1,\"entry\":[true,false,true],\"bearings\":[120,225,315],\"location\":[4.796161,45.791752]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[60,105,165,300],\"location\":[4.796513,45.791578]},{\"out\":0,\"in\":2,\"entry\":[true,true,false,true],\"bearings\":[90,210,285,345],\"location\":[4.796882,45.791489]}],\"driving_side\":\"right\",\"geometry\":\"o_|ivAaovcHjEcInC{JpDaVJyRAaBEmBmBsfBy@ov@hAk]~Ai]fEwPdF{M\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":122,\"bearing_before\":49,\"location\":[4.796161,45.791752],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Avenue de Lanessan (D 306)\"},\"ref\":\"D 306\",\"weight\":100.7,\"duration\":100.7,\"name\":\"Avenue de Lanessan (D 306)\",\"distance\":417.4,\"voiceInstructions\":[{\"distanceAlongGeometry\":397.4,\"announcement\":\"Over 0,25 mijl, ga rechtdoor naar Avenue du 25e Régiment de Tirailleurs Sénégalais (D 306)\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 0,25 mijl, ga rechtdoor naar Avenue du <say-as interpret-as=\\\"address\\\">25e</say-as> Régiment de Tirailleurs Sénégalais (D <say-as interpret-as=\\\"address\\\">306</say-as>)</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":62.2,\"announcement\":\"Ga rechtdoor naar Avenue du 25e Régiment de Tirailleurs Sénégalais (D 306)\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtdoor naar Avenue du <say-as interpret-as=\\\"address\\\">25e</say-as> Régiment de Tirailleurs Sénégalais (D <say-as interpret-as=\\\"address\\\">306</say-as>)</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":417.4,\"primary\":{\"text\":\"Avenue du 25e Régiment de Tirailleurs Sénégalais\",\"components\":[{\"text\":\"Avenue du 25e Régiment de Tirailleurs Sénégalais\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"},\"secondary\":{\"text\":\"D 306\",\"components\":[{\"text\":\"D 306\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"}}]},{\"intersections\":[{\"out\":0,\"in\":1,\"entry\":[true,false],\"bearings\":[120,300],\"location\":[4.801342,45.791271]}],\"driving_side\":\"right\",\"geometry\":\"ma{ivA{r`dHlh@qnA|EaLvGmOtaA{rAtQeP~V}MnLaDdMiCvJe@jLIzI`@\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":126,\"bearing_before\":123,\"location\":[4.801342,45.791271],\"modifier\":\"straight\",\"type\":\"new name\",\"instruction\":\"Ga rechtdoor naar Avenue du 25e Régiment de Tirailleurs Sénégalais (D 306)\"},\"ref\":\"D 306\",\"weight\":131.8,\"duration\":131.8,\"name\":\"Avenue du 25e Régiment de Tirailleurs Sénégalais (D 306)\",\"distance\":528,\"voiceInstructions\":[{\"distanceAlongGeometry\":508,\"announcement\":\"Ga verder op Avenue du 25e Régiment de Tirailleurs Sénégalais (D 306) voor 0,25 mijl\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga verder op Avenue du <say-as interpret-as=\\\"address\\\">25e</say-as> Régiment de Tirailleurs Sénégalais (D <say-as interpret-as=\\\"address\\\">306</say-as>) voor 0,25 mijl</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":280.4,\"announcement\":\"Over 1000 voet, ga rechtsaf naar Rue des Contrebandiers\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 1000 voet, ga rechtsaf naar Rue des Contrebandiers</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":60.1,\"announcement\":\"Ga rechtsaf naar Rue des Contrebandiers, daarna ga scherpe bocht naar rechts naar Rue des Contrebandiers\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Rue des Contrebandiers, daarna ga scherpe bocht naar rechts naar Rue des Contrebandiers</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":528,\"primary\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null},{\"distanceAlongGeometry\":60.1,\"primary\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null,\"sub\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"sharp right\"}}]},{\"intersections\":[{\"out\":2,\"in\":0,\"entry\":[false,true,true],\"bearings\":[0,195,270],\"location\":[4.8051,45.787588]}],\"driving_side\":\"right\",\"geometry\":\"g{sivAw}gdHMvD_@pM\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":275,\"bearing_before\":178,\"location\":[4.8051,45.787588],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Rue des Contrebandiers\"},\"weight\":46.6,\"duration\":16.6,\"name\":\"Rue des Contrebandiers\",\"distance\":25.3,\"voiceInstructions\":[{\"distanceAlongGeometry\":22.9,\"announcement\":\"Ga scherpe bocht naar rechts naar Rue des Contrebandiers, daarna ga rechtdoor naar Rue des Contrebandiers\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga scherpe bocht naar rechts naar Rue des Contrebandiers, daarna ga rechtdoor naar Rue des Contrebandiers</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":25.3,\"primary\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"sharp right\"},\"secondary\":null},{\"distanceAlongGeometry\":22.9,\"primary\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"sharp right\"},\"secondary\":null,\"sub\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"}}]},{\"intersections\":[{\"out\":0,\"in\":1,\"entry\":[true,false,true],\"bearings\":[60,90,270],\"location\":[4.804775,45.787611]}],\"driving_side\":\"right\",\"geometry\":\"u|sivAmigdHqAQf@qM\",\"mode\":\"pushing bike\",\"maneuver\":{\"bearing_after\":56,\"bearing_before\":274,\"location\":[4.804775,45.787611],\"modifier\":\"sharp right\",\"type\":\"turn\",\"instruction\":\"Ga scherpe bocht naar rechts naar Rue des Contrebandiers\"},\"weight\":41.1,\"duration\":41.1,\"name\":\"Rue des Contrebandiers\",\"distance\":22.8,\"voiceInstructions\":[{\"distanceAlongGeometry\":8.3,\"announcement\":\"Ga rechtdoor naar Rue des Contrebandiers\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtdoor naar Rue des Contrebandiers</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":22.8,\"primary\":{\"text\":\"Rue des Contrebandiers\",\"components\":[{\"text\":\"Rue des Contrebandiers\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"},\"secondary\":null}]},{\"intersections\":[{\"classes\":[\"tunnel\"],\"out\":0,\"in\":1,\"entry\":[true,false],\"bearings\":[90,270],\"location\":[4.805017,45.787632]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[0,90,105,270],\"location\":[4.805243,45.787613]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[45,135,225],\"location\":[4.806482,45.787842]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[105,285,300],\"location\":[4.808204,45.788141]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false,true],\"bearings\":[45,90,195,285,315],\"location\":[4.808767,45.787983]},{\"out\":0,\"in\":1,\"entry\":[true,false,true],\"bearings\":[105,270,330],\"location\":[4.809291,45.787966]}],\"driving_side\":\"right\",\"geometry\":\"_~sivAqxgdHd@cMMqBf@oIb@gMTaJK{E_@sEuAiFwBiEcH}ImGaKeDuIyAyH{@mHyAcSi@qKS}JHsEdAeKz@eARoAv@qEzCiG@eEpAsJWmTx@iJbD}V\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":95,\"bearing_before\":95,\"location\":[4.805017,45.787632],\"modifier\":\"straight\",\"type\":\"notification\",\"instruction\":\"Ga rechtdoor naar Rue des Contrebandiers\"},\"weight\":118,\"duration\":113.8,\"name\":\"Rue des Contrebandiers\",\"distance\":393,\"voiceInstructions\":[{\"distanceAlongGeometry\":373,\"announcement\":\"Over 0,25 mijl, ga rechtsaf naar Rue Antonin Laborde\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 0,25 mijl, ga rechtsaf naar Rue Antonin Laborde</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":51.8,\"announcement\":\"Ga rechtsaf naar Rue Antonin Laborde\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Rue Antonin Laborde</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":393,\"primary\":{\"text\":\"Rue Antonin Laborde\",\"components\":[{\"text\":\"Rue Antonin Laborde\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[90,165,285],\"location\":[4.809674,45.787884]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[135,225,345],\"location\":[4.80973,45.787742]},{\"classes\":[\"tunnel\"],\"out\":0,\"in\":1,\"entry\":[true,false],\"bearings\":[135,315],\"location\":[4.810045,45.787511]},{\"out\":0,\"in\":1,\"entry\":[true,false],\"bearings\":[120,315],\"location\":[4.810362,45.787305]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[30,105,210,300],\"location\":[4.810503,45.787242]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[30,135,210,315],\"location\":[4.811565,45.786598]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[135,210,315],\"location\":[4.812016,45.786337]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[135,210,315],\"location\":[4.812526,45.786]}],\"driving_side\":\"right\",\"geometry\":\"wmtivAs{pdH~APbCs@v@mAlMuRzKyR|ByG|@mFxb@ev@nAwBh@kCvKiSfAoB|AqCbQiZtWue@j@gAh@oB\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":165,\"bearing_before\":106,\"location\":[4.809674,45.787884],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Rue Antonin Laborde\"},\"weight\":97.7,\"duration\":97.7,\"name\":\"Rue Antonin Laborde\",\"distance\":383.2,\"voiceInstructions\":[{\"distanceAlongGeometry\":363.2,\"announcement\":\"Over 0,25 mijl, ga rechts naar Quai du Commerce\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 0,25 mijl, ga rechts naar Quai du Commerce</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":58.8,\"announcement\":\"Ga rechts naar Quai du Commerce\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechts naar Quai du Commerce</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":383.2,\"primary\":{\"text\":\"Quai du Commerce\",\"components\":[{\"text\":\"Quai du Commerce\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[30,210,315],\"location\":[4.813237,45.785562]},{\"out\":1,\"in\":0,\"entry\":[false,true,true],\"bearings\":[30,210,300],\"location\":[4.812922,45.785081]},{\"out\":1,\"in\":0,\"entry\":[false,true,true],\"bearings\":[30,210,300],\"location\":[4.812134,45.784121]},{\"out\":1,\"in\":0,\"entry\":[false,true,true],\"bearings\":[30,210,300],\"location\":[4.81121,45.782989]}],\"driving_side\":\"right\",\"geometry\":\"s|oivAizwdHpF|CnUvM~EvD~I~G~h@na@db@t[pb@`\\\\|RnPjErD\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":203,\"bearing_before\":130,\"location\":[4.813237,45.785562],\"modifier\":\"right\",\"type\":\"end of road\",\"instruction\":\"Ga rechts naar Quai du Commerce\"},\"weight\":97.80000000000001,\"duration\":97.80000000000001,\"name\":\"Quai du Commerce\",\"distance\":381.6,\"voiceInstructions\":[{\"distanceAlongGeometry\":361.6,\"announcement\":\"Over 0,25 mijl, ga linksaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 0,25 mijl, ga linksaf</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":58.5,\"announcement\":\"Ga linksaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga linksaf</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":381.6,\"primary\":{\"text\":\"Ga linksaf\",\"components\":[{\"text\":\"Ga linksaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[30,150,210,315],\"location\":[4.81084,45.782568]}],\"driving_side\":\"right\",\"geometry\":\"oajivAodsdH`DuCnh@ra@pQnNlO~K~TzRzJjJfF`ErE`D\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":147,\"bearing_before\":210,\"location\":[4.81084,45.782568],\"modifier\":\"left\",\"type\":\"turn\",\"instruction\":\"Ga linksaf\"},\"weight\":64.3,\"duration\":64.3,\"name\":\"\",\"distance\":267.8,\"voiceInstructions\":[{\"distanceAlongGeometry\":247.8,\"announcement\":\"Over 900 voet, ga rechtdoor naar Quai Hippolyte Jaÿr\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 900 voet, ga rechtdoor naar Quai Hippolyte Jaÿr</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":62.5,\"announcement\":\"Ga rechtdoor naar Quai Hippolyte Jaÿr\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtdoor naar Quai Hippolyte Jaÿr</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":267.8,\"primary\":{\"text\":\"Quai Hippolyte Jaÿr\",\"components\":[{\"text\":\"Quai Hippolyte Jaÿr\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"},\"secondary\":null}]},{\"intersections\":[{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[30,105,210,270],\"location\":[4.809227,45.780499]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[30,120,195,300],\"location\":[4.8092,45.780465]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[15,120,225,330],\"location\":[4.809161,45.780388]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[30,120,195,300],\"location\":[4.808399,45.779243]},{\"out\":3,\"in\":0,\"entry\":[false,true,true,true,true],\"bearings\":[22,103,111,195,285],\"location\":[4.807992,45.778542]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[15,180,285,345],\"location\":[4.807779,45.777166]},{\"out\":3,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[0,30,150,180],\"location\":[4.807801,45.776951]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[135,165,345],\"location\":[4.808466,45.775212]},{\"out\":0,\"in\":1,\"entry\":[true,false,true],\"bearings\":[120,315,345],\"location\":[4.80858,45.775065]}],\"driving_side\":\"right\",\"geometry\":\"e`fivAu_pdHbAt@xClApClDxJjF~F`Edn@v[xj@lXtCv@jjAzLjAmApBBlLk@xIy@rGgBdkAe]`LiEnCeA`B_Ar@}Aj@aC\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":208,\"bearing_before\":206,\"location\":[4.809227,45.780499],\"modifier\":\"straight\",\"type\":\"new name\",\"instruction\":\"Ga rechtdoor naar Quai Hippolyte Jaÿr\"},\"weight\":156.79999999999998,\"duration\":156.79999999999998,\"name\":\"Quai Hippolyte Jaÿr\",\"distance\":642.9,\"voiceInstructions\":[{\"distanceAlongGeometry\":622.9,\"announcement\":\"Ga verder op Quai Hippolyte Jaÿr voor 0,5 mijl\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga verder op Quai Hippolyte Jaÿr voor 0,5 mijl</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":287,\"announcement\":\"Over 1000 voet, ga rechtsaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 1000 voet, ga rechtsaf</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":61.5,\"announcement\":\"Ga rechtsaf, daarna ga linksaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf, daarna ga linksaf</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":642.9,\"primary\":{\"text\":\"Ga rechtsaf\",\"components\":[{\"text\":\"Ga rechtsaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null},{\"distanceAlongGeometry\":61.5,\"primary\":{\"text\":\"Ga rechtsaf\",\"components\":[{\"text\":\"Ga rechtsaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null,\"sub\":{\"text\":\"Ga linksaf\",\"components\":[{\"text\":\"Ga linksaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"}}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[90,180,300],\"location\":[4.808645,45.775043]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true,true],\"bearings\":[0,105,180,270,300],\"location\":[4.808652,45.775]},{\"out\":3,\"in\":0,\"entry\":[false,true,true,true,true],\"bearings\":[5,87,97,203,270],\"location\":[4.808642,45.774916]}],\"driving_side\":\"right\",\"geometry\":\"ek{hvAi{ndHtAMfDRnCtA\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":172,\"bearing_before\":115,\"location\":[4.808645,45.775043],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf\"},\"weight\":13.5,\"duration\":13.5,\"name\":\"\",\"distance\":22.9,\"voiceInstructions\":[{\"distanceAlongGeometry\":22.9,\"announcement\":\"Ga linksaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga linksaf</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":22.9,\"primary\":{\"text\":\"Ga linksaf\",\"components\":[{\"text\":\"Ga linksaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":0,\"entry\":[false,true,true,true,true],\"bearings\":[30,120,150,270,330],\"location\":[4.808599,45.774844]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[15,90,180,300],\"location\":[4.80871,45.774807]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[75,150,270],\"location\":[4.808764,45.774807]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[90,165,270],\"location\":[4.810482,45.774687]}],\"driving_side\":\"right\",\"geometry\":\"w~zhvAmxndHhA}E?kBo@iC~GafBx@_S\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":115,\"bearing_before\":202,\"location\":[4.808599,45.774844],\"modifier\":\"left\",\"type\":\"turn\",\"instruction\":\"Ga linksaf\"},\"weight\":44.199999999999996,\"duration\":44.199999999999996,\"name\":\"\",\"distance\":173.7,\"voiceInstructions\":[{\"distanceAlongGeometry\":153.7,\"announcement\":\"Over 600 voet, ga rechtsaf naar Quai Joseph Gillet\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 600 voet, ga rechtsaf naar Quai Joseph Gillet</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":58.9,\"announcement\":\"Ga rechtsaf naar Quai Joseph Gillet\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Quai Joseph Gillet</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":173.7,\"primary\":{\"text\":\"Quai Joseph Gillet\",\"components\":[{\"text\":\"Quai Joseph Gillet\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null}]},{\"intersections\":[{\"out\":2,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[45,90,165,270],\"location\":[4.810802,45.774658]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[120,315,330],\"location\":[4.811313,45.773974]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[30,135,300],\"location\":[4.812074,45.773576]},{\"out\":0,\"in\":1,\"entry\":[true,false,true],\"bearings\":[135,315,345],\"location\":[4.812495,45.773316]}],\"driving_side\":\"right\",\"geometry\":\"cszhvAcbsdHlOgHhDwCvGkHfKqGlS}e@lCsGzCkHjJ}OpDqEhN}Oz@_A~QwPdDyCbE{B`UqJ\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":157,\"bearing_before\":97,\"location\":[4.810802,45.774658],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Quai Joseph Gillet\"},\"weight\":112.1,\"duration\":89.30000000000001,\"name\":\"Quai Joseph Gillet\",\"distance\":360.2,\"voiceInstructions\":[{\"distanceAlongGeometry\":340.2,\"announcement\":\"Over 0,25 mijl, blijf rechtdoor gaan op Quai Joseph Gillet\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 0,25 mijl, blijf rechtdoor gaan op Quai Joseph Gillet</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":60.5,\"announcement\":\"Blijf rechtdoor gaan op Quai Joseph Gillet, daarna ga rechtdoor naar Quai Saint-Vincent\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Blijf rechtdoor gaan op Quai Joseph Gillet, daarna ga rechtdoor naar Quai Saint-Vincent</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":360.2,\"primary\":{\"text\":\"Quai Joseph Gillet\",\"components\":[{\"text\":\"Quai Joseph Gillet\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"},\"secondary\":null},{\"distanceAlongGeometry\":60.5,\"primary\":{\"text\":\"Quai Joseph Gillet\",\"components\":[{\"text\":\"Quai Joseph Gillet\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"},\"secondary\":null,\"sub\":{\"text\":\"Quai Saint-Vincent\",\"components\":[{\"text\":\"Quai Saint-Vincent\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"}}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[150,165,345],\"location\":[4.813511,45.772114]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[0,165,345],\"location\":[4.8138,45.771535]}],\"driving_side\":\"right\",\"geometry\":\"ctuhvAmkxdHjGwAfFwBfFwBx@]bEeBjDuBpKsD\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":161,\"bearing_before\":158,\"location\":[4.813511,45.772114],\"modifier\":\"straight\",\"type\":\"continue\",\"instruction\":\"Blijf rechtdoor gaan op Quai Joseph Gillet\"},\"weight\":41.1,\"duration\":41.1,\"name\":\"Quai Joseph Gillet\",\"distance\":91.8,\"voiceInstructions\":[{\"distanceAlongGeometry\":33.5,\"announcement\":\"Ga rechtdoor naar Quai Saint-Vincent\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtdoor naar Quai Saint-Vincent</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":91.8,\"primary\":{\"text\":\"Quai Saint-Vincent\",\"components\":[{\"text\":\"Quai Saint-Vincent\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"straight\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[75,165,345],\"location\":[4.81389,45.771334]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[180,210,345],\"location\":[4.814163,45.77043]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[0,150,165,225],\"location\":[4.814184,45.770313]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[165,315,345],\"location\":[4.814243,45.770057]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[15,120,315],\"location\":[4.815504,45.768339]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[0,75,255],\"location\":[4.81903,45.767609]}],\"driving_side\":\"right\",\"geometry\":\"kcthvAccydHn]wG`QwExB_@bCQhFi@bD_@zIuAnD{AhIaE|KsFvCwAlJuE`]uPj@_@`KsGdAyAtKgO`CmE`Oy]fBiFp@yChCmNbFwZvBsMj@sDb@_D|@eJh@yGrEck@RgHAkCAiCc@oKeAmMqCeZiF_n@]qD\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":167,\"bearing_before\":161,\"location\":[4.81389,45.771334],\"modifier\":\"straight\",\"type\":\"new name\",\"instruction\":\"Ga rechtdoor naar Quai Saint-Vincent\"},\"weight\":265.1,\"duration\":265.1,\"name\":\"Quai Saint-Vincent\",\"distance\":757.1,\"voiceInstructions\":[{\"distanceAlongGeometry\":737.1,\"announcement\":\"Ga verder op Quai Saint-Vincent voor 0,5 mijl\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga verder op Quai Saint-Vincent voor 0,5 mijl</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":199.9,\"announcement\":\"Over 700 voet, ga rechtsaf naar Passerelle de l'Homme de la Roche\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 700 voet, ga rechtsaf naar Passerelle de l'Homme de la Roche</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":42.8,\"announcement\":\"Ga rechtsaf naar Passerelle de l'Homme de la Roche, daarna ga linksaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Passerelle de l'Homme de la Roche, daarna ga linksaf</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":757.1,\"primary\":{\"text\":\"Passerelle de l'Homme de la Roche\",\"components\":[{\"text\":\"Passerelle de l'Homme de la Roche\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null},{\"distanceAlongGeometry\":42.8,\"primary\":{\"text\":\"Passerelle de l'Homme de la Roche\",\"components\":[{\"text\":\"Passerelle de l'Homme de la Roche\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null,\"sub\":{\"text\":\"Ga linksaf\",\"components\":[{\"text\":\"Ga linksaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"}}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[75,165,255],\"location\":[4.820306,45.767814]}],\"driving_side\":\"right\",\"geometry\":\"kgmhvActeeHhBm@f@Q~q@oUtBeB\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":163,\"bearing_before\":75,\"location\":[4.820306,45.767814],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Passerelle de l'Homme de la Roche\"},\"weight\":33.7,\"duration\":33.7,\"name\":\"Passerelle de l'Homme de la Roche\",\"distance\":111.1,\"voiceInstructions\":[{\"distanceAlongGeometry\":49.5,\"announcement\":\"Ga linksaf\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga linksaf</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":111.1,\"primary\":{\"text\":\"Ga linksaf\",\"components\":[{\"text\":\"Ga linksaf\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"left\"},\"secondary\":null}]},{\"intersections\":[{\"out\":0,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[75,165,255,330],\"location\":[4.820749,45.766866]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[45,75,180,270],\"location\":[4.823117,45.767384]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[90,180,270],\"location\":[4.823786,45.767406]},{\"out\":0,\"in\":2,\"entry\":[true,true,false,true],\"bearings\":[105,195,270,315],\"location\":[4.8263,45.767496]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[135,180,300],\"location\":[4.827588,45.767139]},{\"out\":0,\"in\":2,\"entry\":[true,true,false],\"bearings\":[120,225,330],\"location\":[4.828684,45.766381]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[90,150,300],\"location\":[4.828716,45.766366]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[60,150,240,315],\"location\":[4.828798,45.766292]},{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[30,165,330],\"location\":[4.828851,45.766239]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[0,105,180,270],\"location\":[4.829136,45.765365]},{\"out\":1,\"in\":3,\"entry\":[true,true,true,false],\"bearings\":[15,150,195,330],\"location\":[4.82924,45.764682]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[0,150,180,270],\"location\":[4.829438,45.763491]},{\"out\":2,\"in\":0,\"entry\":[false,false,true,true],\"bearings\":[0,105,180,285],\"location\":[4.829441,45.763338]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,false],\"bearings\":[0,75,180,270],\"location\":[4.829444,45.763227]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[0,60,195,285],\"location\":[4.829428,45.763097]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[15,135,195,315],\"location\":[4.829402,45.762977]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[15,105,195,285],\"location\":[4.828845,45.761728]},{\"out\":2,\"in\":0,\"entry\":[false,true,true],\"bearings\":[15,60,195],\"location\":[4.828438,45.760575]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[15,60,195,255],\"location\":[4.828416,45.760528]},{\"out\":2,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[15,165,195,300],\"location\":[4.82805,45.759834]}],\"driving_side\":\"right\",\"geometry\":\"clkhvAyofeHg@wCoNc{@oGgZ{BwLw@iI]uMQcPE}Co@_AWmYh@iAGcEqAujAiBwhAFuFV{CvEw^fB_LvBeKjCaJlBsFd@sAtW{b@`@cCnMuR|DeE\\\\_A|@qAtAqAhBiBdEwAfh@cMdF]lb@_AfFoCbFgF~C_B|D[jw@g@pHE|EEbG^z@FrDj@`Fx@z[jI~H~C|ShLdJhCjG`B`e@zLrXnG|Aj@pA`@jSxIjM~GlCf@r@VhBT\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":68,\"bearing_before\":151,\"location\":[4.820749,45.766866],\"modifier\":\"left\",\"type\":\"turn\",\"instruction\":\"Ga linksaf\"},\"weight\":345.2,\"duration\":345.2,\"name\":\"\",\"distance\":1435.2,\"voiceInstructions\":[{\"distanceAlongGeometry\":1415.2,\"announcement\":\"Ga rechtdoor voor 1 mijl\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtdoor voor 1 mijl</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":291,\"announcement\":\"Over 1000 voet, rechts aanhouden op de splitsing\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 1000 voet, rechts aanhouden op de splitsing</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":62.4,\"announcement\":\"Rechts aanhouden op de splitsing, daarna ga rechtsaf naar Avenue Adolphe Max\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Rechts aanhouden op de splitsing, daarna ga rechtsaf naar Avenue Adolphe Max</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":1435.2,\"primary\":{\"text\":\"Rechts aanhouden op de splitsing\",\"components\":[{\"text\":\"Rechts aanhouden op de splitsing\",\"type\":\"text\"}],\"type\":\"fork\",\"modifier\":\"right\"},\"secondary\":null},{\"distanceAlongGeometry\":62.4,\"primary\":{\"text\":\"Rechts aanhouden op de splitsing\",\"components\":[{\"text\":\"Rechts aanhouden op de splitsing\",\"type\":\"text\"}],\"type\":\"fork\",\"modifier\":\"right\"},\"secondary\":null,\"sub\":{\"text\":\"Avenue Adolphe Max\",\"components\":[{\"text\":\"Avenue Adolphe Max\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"}}]},{\"intersections\":[{\"out\":2,\"in\":0,\"entry\":[false,true,true],\"bearings\":[15,165,210],\"location\":[4.828039,45.759781]}],\"driving_side\":\"right\",\"geometry\":\"iq}gvAmwteHnBrA\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":206,\"bearing_before\":187,\"location\":[4.828039,45.759781],\"modifier\":\"slight right\",\"type\":\"fork\",\"instruction\":\"Rechts aanhouden op de splitsing\"},\"weight\":6.6,\"duration\":6.6,\"name\":\"\",\"distance\":7,\"voiceInstructions\":[{\"distanceAlongGeometry\":7,\"announcement\":\"Ga rechtsaf naar Avenue Adolphe Max, daarna ga rechtsaf naar Rue Jean Carries\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Avenue Adolphe Max, daarna ga rechtsaf naar Rue Jean Carries</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":7,\"primary\":{\"text\":\"Avenue Adolphe Max\",\"components\":[{\"text\":\"Avenue Adolphe Max\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null,\"sub\":{\"text\":\"Rue Jean Carries\",\"components\":[{\"text\":\"Rue Jean Carries\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"}}]},{\"intersections\":[{\"out\":3,\"in\":0,\"entry\":[false,true,true,true],\"bearings\":[30,120,210,300],\"location\":[4.827997,45.759725]},{\"out\":3,\"in\":1,\"entry\":[true,false,true,true],\"bearings\":[15,120,195,300],\"location\":[4.827925,45.759759]},{\"out\":2,\"in\":0,\"entry\":[false,true,true],\"bearings\":[105,195,345],\"location\":[4.826639,45.760144]}],\"driving_side\":\"right\",\"geometry\":\"ym}gvAytteHcAnC}A|GqP`z@qBjK_HzC\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":303,\"bearing_before\":206,\"location\":[4.827997,45.759725],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Avenue Adolphe Max\"},\"weight\":52.1,\"duration\":39.6,\"name\":\"Avenue Adolphe Max\",\"distance\":132.5,\"voiceInstructions\":[{\"distanceAlongGeometry\":50.2,\"announcement\":\"Ga rechtsaf naar Rue Jean Carries\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Rue Jean Carries</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":132.5,\"primary\":{\"text\":\"Rue Jean Carries\",\"components\":[{\"text\":\"Rue Jean Carries\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null}]},{\"intersections\":[{\"out\":3,\"in\":2,\"entry\":[true,true,false,true,true],\"bearings\":[15,105,165,270,330],\"location\":[4.826561,45.760288]},{\"out\":0,\"in\":1,\"entry\":[true,false,true],\"bearings\":[0,90,180],\"location\":[4.825551,45.760299]}],\"driving_side\":\"right\",\"geometry\":\"_q~gvAa{qeHUb~@qFFyc@wK\",\"mode\":\"cycling\",\"maneuver\":{\"bearing_after\":270,\"bearing_before\":338,\"location\":[4.826561,45.760288],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Rue Jean Carries\"},\"weight\":76.6,\"duration\":46.6,\"name\":\"Rue Jean Carries\",\"distance\":159.2,\"voiceInstructions\":[{\"distanceAlongGeometry\":139.2,\"announcement\":\"Over 500 voet, ga rechtsaf naar Rue de la Brèche\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Over 500 voet, ga rechtsaf naar Rue de la Brèche</prosody></amazon:effect></speak>\"},{\"distanceAlongGeometry\":51.2,\"announcement\":\"Ga rechtsaf naar Rue de la Brèche, daarna u arriveert op de bestemming\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Ga rechtsaf naar Rue de la Brèche, daarna u arriveert op de bestemming</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":159.2,\"primary\":{\"text\":\"Rue de la Brèche\",\"components\":[{\"text\":\"Rue de la Brèche\",\"type\":\"text\"}],\"type\":\"turn\",\"modifier\":\"right\"},\"secondary\":null}]},{\"intersections\":[{\"out\":1,\"in\":2,\"entry\":[true,true,false],\"bearings\":[15,105,195],\"location\":[4.825751,45.761009]}],\"driving_side\":\"right\",\"geometry\":\"a~_hvAmhpeH`@_Ex@_I\",\"mode\":\"pushing bike\",\"maneuver\":{\"bearing_after\":104,\"bearing_before\":12,\"location\":[4.825751,45.761009],\"modifier\":\"right\",\"type\":\"turn\",\"instruction\":\"Ga rechtsaf naar Rue de la Brèche\"},\"weight\":18.4,\"duration\":18.4,\"name\":\"Rue de la Brèche\",\"distance\":20.5,\"voiceInstructions\":[{\"distanceAlongGeometry\":5.6,\"announcement\":\"Je bent gearriveerd op de bestemming.\",\"ssmlAnnouncement\":\"<speak><amazon:effect name=\\\"drc\\\"><prosody rate=\\\"1.08\\\">Je bent gearriveerd op de bestemming.</prosody></amazon:effect></speak>\"}],\"bannerInstructions\":[{\"distanceAlongGeometry\":20.5,\"primary\":{\"text\":\"U zult aankomen\",\"components\":[{\"text\":\"U zult aankomen\",\"type\":\"text\"}],\"type\":\"arrive\",\"modifier\":\"straight\"},\"secondary\":null},{\"distanceAlongGeometry\":5.6,\"primary\":{\"text\":\"U bent gearriveerd\",\"components\":[{\"text\":\"U bent gearriveerd\",\"type\":\"text\"}],\"type\":\"arrive\",\"modifier\":\"straight\"},\"secondary\":null}]},{\"intersections\":[{\"in\":0,\"entry\":[true],\"bearings\":[285],\"location\":[4.826007,45.760963]}],\"driving_side\":\"right\",\"geometry\":\"e{_hvAmxpeH\",\"mode\":\"pushing bike\",\"maneuver\":{\"bearing_after\":0,\"bearing_before\":105,\"location\":[4.826007,45.760963],\"type\":\"arrive\",\"instruction\":\"Je bent gearriveerd op de bestemming.\"},\"weight\":0,\"duration\":0,\"name\":\"Rue de la Brèche\",\"distance\":0,\"voiceInstructions\":[],\"bannerInstructions\":[]}],\"distance\":6644.8}],\"weight_name\":\"cyclability\",\"weight\":1954.7,\"duration\":1855.2,\"distance\":6644.8,\"voiceLocale\":\"nl-NL\"}],\"waypoints\":[{\"distance\":16.72947365339195,\"name\":\"Rue Jean-Claude Bartet\",\"location\":[4.793886,45.792688]},{\"distance\":0.45133715869294344,\"name\":\"Rue de la Brèche\",\"location\":[4.826007,45.760963]}],\"code\":\"Ok\",\"uuid\":\"U3nx4z756VP98b-7QO5XZ2UDIdGXCg92_Ixywp7X6cof6lgTA0t_YA==\"}";
        var direction: DirectionsResponse = DirectionsResponse.fromJson(json);
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        var arguments = call.arguments as? Map<String, Any>

        when (call.method) {
            "map#waitForMap" -> {
                if (::mapboxMap.isInitialized) {
                    result.success(null);
                    return;
                }
                mapReadyResult = result;
            }
            "playTTS" -> {

                //Reset smart fact activator for each announcement
                if(NavigationUtils.currentStopWatchTimer != null) {
                    saveTimeOfLastVocalInstruction = NavigationUtils.currentStopWatchTimer!!.elapsedTime
                }

                val description = call.argument<String>("description")
                audioService.playTTS(description, false, true)
                result.success(null);
            }
            "playInstructionTTS" -> { //For directionnal

                //Reset for smart fact activator for each announcement
                if(NavigationUtils.currentStopWatchTimer != null) {
                    saveTimeOfLastVocalInstruction = NavigationUtils.currentStopWatchTimer!!.elapsedTime
                }

                val description = call.argument<String>("description")
                audioService.playTTS(SpeechAnnouncement.builder().announcement(description).build(), true)
                result.success(null);
            }
            "stopTTS" -> {
                audioService.stop()
                result.success(null);
            }
            else -> throw IllegalArgumentException(
                    "Cannot interpret " + activityState!!.get() + " as an activity state");
        }
    }

    fun flutterRouteToKtRoute(routeJson: String?, gpsColor: String?, isWhiteLabel: Boolean?) {
        var route: RouteDataClass
        if(!isWhiteLabel!!) {
            route = RouteDataClass.fromFlutter(routeJson!!)
        } else {
            route = RouteDataClass.fromFlutterWhiteLabel(routeJson!!)
        }
        AppDataHolder.currentRoute = route
    }

    fun flutterSponsorToMap(sponsorJson: String?) {
        System.out.println(sponsorJson);
        if(sponsorJson != "") {
            val data: Map<*, *> = Gson().fromJson(sponsorJson, MutableMap::class.java)
            this.sponsorData = data
        }
    }

    companion object {

        private val FIRST = 0
        private val ONE_HUNDRED_MILLISECONDS = 100
        private val BOTTOMSHEET_PADDING_MULTIPLIER = 4
        public val TWO_SECONDS_IN_MILLISECONDS = 2000
        private val BEARING_TOLERANCE = 90.0
        private val SEARCHING_FOR_GPS_MESSAGE = "Searching for GPS..."
        private val GPS_FOUND_MESSAGE = "GPS found, have a nice run!"
        private val COMPONENT_NAVIGATION_INSTRUCTION_CACHE =
                "component-navigation-instruction-cache"
        private val TEN_MEGABYTE_CACHE_SIZE = (10 * 1024 * 1024).toLong()
        private val ZERO_PADDING = 0
        public val DEFAULT_ZOOM = 12.0
        public val DEFAULT_TILT = 0.0
        private val DEFAULT_BEARING = 0.0
        private val UPDATE_INTERVAL_IN_MILLISECONDS: Long = 1000
        private val FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS: Long = 500
        private val INSTRUCTION_DURATION_DISPLAY: Long = 5000

        private val DEPARTURE_MARKER_SOURCE = "departure_marker"
        private val ARRIVAL_MARKER_SOURCE = "arrival_marker"
        private val SPONSOR_MARKER_SOURCE = "SPONSOR_MARKER_SOURCE"
        private val SS_MARKER_SOURCE = "SS_MARKER_SOURCE"
        private val LENGTH_ANIMATION_GOODDEAL_MS = 10000

        private val INTERVAL_BETWEEN_SAVES = 5000
    }

    lateinit var navigationLayout: CoordinatorLayout
    lateinit var mapView: MapView
    lateinit var mapboxMap : MapboxMap
    lateinit var instructionView: InstructionView

    private lateinit var callback: LocationEngineCallback<LocationEngineResult>
    private var locationEngine: LocationEngine? = null
    private var navigation: MapboxNavigation? = null
    private var speechPlayer: NavigationSpeechPlayer? = null
    private var navigationMap: NavigationMapboxMap? = null
    private var lastLocation: Location? = null
    private var previousLocation: Location? = null
    private var calculatedRouteRun: DirectionsRoute? = null
    private var calculatedRouteLeadUserToRun: DirectionsRoute? = null
    private var destination: Point? = null
    private var mapState: MapState? = null
    private var loadedStyle: Style? = null
    private lateinit var annotationManager : SymbolManager
    private lateinit var circleManager: CircleManager
    private lateinit var displayedAnnotations : List<Symbol>
    private var displayedFeatures = ArrayList<Feature>()
    private var hasRunStarted = false
    private var isArrivedOnSmartRun = false
    private var hasRouteFinished = false
    private var hasLeadStarted = false
    private var hasLeadBeenRequested = false
    private var isUserTooFarFromRoute = false
    private var isMapboxSpeakerMuted = false
    private var currentBlockerPOI : POI? = null
    private var hasUserRequestedNavigation = false
    private var hasUserRequestedMockNavigation = false
    private var hasUserChangedRating = false
    private var isMockNavigationPaused = false
    private var currentMockNavigationSpeed = 20
    private var closestRoutePoint: Point? = null
    private var currentDisplayedPOI: POI? = null
    private var startNavTimestamp = 0L
    private var currentDirectionRoute : DirectionsRoute? = null

    // STATS
    private var startingLocation : Location? = null
    private var initDistanceToTravel = 0.0
    private var totalDistanceTravelled = 0.0
    private var totalTimeTravelled = 0L
    private var averageSpeed = 0f
    private var listSpeeds = ArrayList<Float>()
    private var currentRunSession: RunSession? = null

    private var audioService = AudioService.getInstance()

    private var bottomSheetPoiBehavior : BottomSheetBehavior<LinearLayout>? = null
    private var bottomSheetEndBehavior : BottomSheetBehavior<LinearLayout>? = null
    private var navigationState = NavigationState.WAITING_TO_START
    private var timeStampStartGooddealAnimation: Long = 0
    private var isGoodDealDisplayed = false
    private var idGoodDealDisplayed: String? = null

    private var totalTimeInMS = 0L
    private var previousTime = 0L
    private var lastTimeUserHasBeenOffRoute = 0L
    private var previousSaveTimeStamp = 0L

    private var saveTimeOfLastVocalInstruction = 0L

    private var componentNavigationLayout: CoordinatorLayout? = null
    private var  toolbar:Toolbar?= null
    private var  instructionViewNoNavigation: LinearLayout?= null
    private var  maneuverImage: ImageView?= null
    private var  maneuverInstruction: AutofitTextView? = null
    private var  coordinatorBannerPoi: CoordinatorLayout?= null
    private var  containerGooddeal: RelativeLayout?= null
    private var  imageBannerPoi:ImageView? = null
    private var  titleBannerPoi: TextView?= null
    private var  descriptionBannerPoi:TextView? = null
    private var  linkGooddeal: com.google.android.material.floatingactionbutton.FloatingActionButton?= null
    private var  menuLayout: LinearLayout?= null
    private var  statsLayout: ConstraintLayout? = null
    private var  distanceContainer:LinearLayout? = null
    private var  distanceLeftTextview: TextView? = null
    private var  timerTextview: TextView? = null
    private var  speedTextview: TextView? = null
    private var  resumeButton:Button? = null
    private var  finishButton:Button? = null
    private var  mp3Fillprogresslayout: FillProgressLayout? = null
    private var  mp3TimeTextview:TextView? = null
    private var  mp3FillprogresslayoutStatic: FillProgressLayout? = null
    private var  mp3TimeTextviewStatic:TextView? = null
    private var  titleStatsResume:TextView? = null
    private var  icFlagVariantOutline:ImageView? = null
    private var  lengthTextview:TextView? = null
    private var  icTimer:ImageView? = null
    private var  durationTextview:TextView? = null
    private var  icSpeed:ImageView? = null
    private var  speedTextviewStatic:TextView? = null
    private var  titleExperienceResume:TextView? = null
    private var recenterButton: com.google.android.material.floatingactionbutton.FloatingActionButton? = null
    private var pauseSwiper: SlideToActView? = null
    private var navigationPauseScreen: ConstraintLayout? = null
    private var rangeProgress: RangeSeekBar? = null
    private var menufab: com.github.clans.fab.FloatingActionMenu? = null
    private var reportFab: com.github.clans.fab.FloatingActionButton? = null
    private var soundFab: com.github.clans.fab.FloatingActionButton? = null
    private var emergencyFab: com.github.clans.fab.FloatingActionButton? = null
    private var musicFab: com.google.android.material.floatingactionbutton.FloatingActionButton? = null
    private var greyBackground: LinearLayout? = null
    private var sponsorLogo: ImageView? = null
    private var sponsorView: LinearLayout? = null
    private var audioSwitch: Switch? = null

    private enum class MapState {
        INFO,
        NAVIGATION
    }

    private enum class NavigationState {
        WAITING_TO_START,
        IN_PROGRESS,
        PAUSED,
        FINISHED
    }

    private var MARKER_SOURCE = "markers-source-custom-"
    private var RADIUS_MARKER_SOURCE = "radius-markers-source-custom-"
    private var INVISIBLE_SOURCE = "invisible-source"
    private var MARKER_STYLE_LAYER = "markers-style-layer-"
    private var RADIUS_MARKER_STYLE_LAYER = "radius-markers-style-layer-"
    private var INVISIBLE_LAYER = "invisible-style"
    private var MARKER_IMAGE = "custom-marker"


    override fun getView(): View {
        return mainView;
    }

    private fun initView(){
        //initBottomSheets()
        //bottomSheetPoiBehavior?.state = BottomSheetBehavior.STATE_HIDDEN
        //bottomSheetEndBehavior?.state = BottomSheetBehavior.STATE_HIDDEN
        //bottomSheetEndBehavior?.isHideable = false
        recenterButton!!.hide()
        finishButton!!.visibility = View.GONE
        resumeButton!!.visibility = View.GONE
        /*if(this.whiteLabel!!.isWhiteLabel!!) {
            resumeButton!!.setText(_activity.getString(R.string.continue_white_label_button))
        }*/
        initInstructionView()
        showCustomInstructionView(false)
        // TODO Improve distance algorithm computing when navigation is deactivated
        if(!AppDataHolder.currentRoute?.isActivated!!){
            distanceContainer!!.visibility = View.GONE
        }
    }

    private fun triggerNavigation(location: Location){
        if(navigationState == NavigationState.WAITING_TO_START){
            val arguments: MutableMap<String, Any> = java.util.HashMap(1)
            arguments["isStarted"] = true
            methodChannel!!.invokeMethod("onNavigationStarted", arguments)


            if(lastLocation == null) {
                if(downloadForOffline) {
                    //put selected point for simulate closest point of route
                    lastLocation = AppDataHolder.currentRoute?.arrayCoordinates!![0]
                } else {
                    lastLocation = location
                }
            }
            hasUserRequestedNavigation = true
            manageNavigationState()
            //Not lead route when I want download map and if offlineDatas is set
            if(checkIfUserIsTooFarFromRoute(location) && !downloadForOffline && offlineDatas == null){
                leadUserToRunRoute()
            }else{
                startRunNavigation(MapUtils.computeRouteCoordinatesBasedOnClosestPoint(lastLocation!!, AppDataHolder.currentRoute?.arrayCoordinates!!))
            }
        }else if(navigationState == NavigationState.IN_PROGRESS){
            manageNavigationState()
            navigationState = NavigationState.PAUSED
            //moveCameraOverhead()
            navigationMap!!.showRouteOverview(intArrayOf(150,150,150,150))
        }
    }

    private fun initListeners() {

        resumeButton!!.setOnClickListener {
            if(navigationState == NavigationState.PAUSED){
                manageNavigationState()
                navigationMap!!.resetCameraPositionWith(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
                val cameraUpdate = cameraOverheadUpdate()
                if (cameraUpdate != null) {
                    val navUpdate = NavigationCameraUpdate(cameraUpdate)
                    navigationMap!!.retrieveCamera().update(navUpdate)
                }
                navigationState = NavigationState.IN_PROGRESS
                navigationPauseScreen!!.visibility = View.GONE
                pauseSwiper!!.visibility= View.VISIBLE
            }
        }

        finishButton!!.setOnClickListener {
            finishNavigation(false)
        }

        recenterButton!!.setOnClickListener {
            recenterButton!!.hide()
            mapboxMap.locationComponent.setCameraMode(CameraMode.TRACKING_GPS, 1000, 30.0, null, 0.0, null);
            //navigationMap!!.resetCameraPositionWith(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
        }

        sponsorView!!.setOnClickListener {
            val argument: MutableMap<String, Any> = java.util.HashMap(1)
            argument["sponsorId"] = this.sponsorData["id"]!!
            methodChannel!!.invokeMethod("onSponsorClick", argument)
        }

        /*closeNavigation!!.setOnClickListener {
            val argument: MutableMap<String, Any> = java.util.HashMap(1)
            val values: MutableMap<String, Any?> = java.util.HashMap(4)
            values["distanceRemaining"] = this.flutterDistanceRemaining
            values["distanceTraveled"] = this.flutterDistanceTravelled
            values["distanceInitial"] = this.initDistanceToTravel
            values["time"] = this.totalTimeTravelled
            argument["isFinished"] = values
            methodChannel!!.invokeMethod("onNavigationFinished", argument)
        }*/

        pauseSwiper!!.onSlideCompleteListener = this

    }

    override fun onSlideComplete(view: SlideToActView) {
        if(pauseSwiper!!.isCompleted()) {
            if (navigationState == NavigationState.IN_PROGRESS) {
                manageNavigationState()
                navigationState = NavigationState.PAUSED
                //moveCameraOverhead()
                navigationMap!!.showRouteOverview(intArrayOf(150, 150, 150, 150))
                // TODO show pause screen
                pauseSwiper!!.visibility = View.GONE
                navigationPauseScreen!!.visibility = View.VISIBLE
                initPauseScreen()
            }
            pauseSwiper!!.resetSlider()
        }
    }

    private fun initPauseScreen(){

        var timePausedText: TextView? = null
        timePausedText = view.findViewById( R.id.time_paused_text );


        var titlePausedText: TextView? = null
        titlePausedText = view.findViewById(R.id.titlePaused);
        titlePausedText.text = _activity.getString(R.string.title_visit_paused);

        var pauseSpeedTextview: TextView? = null
        pauseSpeedTextview = view.findViewById( R.id.pause_speed_textview );

        var pauseDistanceLeftTextview: TextView? = null
        pauseDistanceLeftTextview = view.findViewById( R.id.pause_distance_left_textview );

        var finishPauseButton: Button? = null
        finishPauseButton = view.findViewById( R.id.finish_button );

        var pauseEmergencySms: LinearLayout? = null
        pauseEmergencySms = view.findViewById( R.id.pause_emergency_sms );


        audioSwitch!!.isChecked = !isMapboxSpeakerMuted
        audioSwitch!!.setOnClickListener() {
            updateAudioGuide(!audioSwitch!!.isChecked, false)
        }


        timePausedText.text = timerTextview!!.text

        pauseSpeedTextview.text =  MetricsUtils.displaySpeedFormatted(averageSpeed)
        pauseDistanceLeftTextview.text = distanceLeftTextview!!.text

        pauseEmergencySms.setOnClickListener(){
            methodChannel!!.invokeMethod("onEmergencySmsClick", null)
        }


        finishPauseButton.setOnClickListener(){
            val argument: MutableMap<String, Any> = java.util.HashMap(1)
            val values: MutableMap<String, Any?> = java.util.HashMap(4)
            values["distanceRemaining"] = this.flutterDistanceRemaining
            values["distanceTraveled"] = this.flutterDistanceTravelled
            values["distanceInitial"] = currentRunSession?.totalDistance
            values["time"] = this.totalTimeTravelled
            argument["isFinished"] = values
            methodChannel!!.invokeMethod("onNavigationFinished", argument)
        }
    }

    private fun setFlutterCustomColor() {

        //MENU BUTTON COLOR
        menufab!!.menuButtonColorNormal = Color.parseColor(this.gpsColor)
        reportFab!!.colorNormal = Color.parseColor(this.gpsColor)
        reportFab!!.setLabelColors(Color.parseColor(this.gpsColor), Color.parseColor(this.gpsColor), Color.parseColor(this.gpsColor))
        emergencyFab!!.colorNormal = Color.parseColor(this.gpsColor)
        emergencyFab!!.setLabelColors(Color.parseColor(this.gpsColor), Color.parseColor(this.gpsColor), Color.parseColor(this.gpsColor))
        soundFab!!.colorNormal = Color.parseColor(this.gpsColor)
        soundFab!!.setLabelColors(Color.parseColor(this.gpsColor), Color.parseColor(this.gpsColor), Color.parseColor(this.gpsColor))

        //TIMER LABEL COLOR
        timerTextview!!.setTextColor(Color.parseColor(this.gpsColor))

        //CONTINUE RUNNING BUTTON
        val gdDefault = GradientDrawable()
        gdDefault.setColor(Color.parseColor(this.gpsColor))
        gdDefault.setCornerRadius(200f)
        resumeButton!!.setBackgroundDrawable(gdDefault)

        //SLIDER BUTTON
        pauseSwiper!!.innerColor = Color.parseColor(this.gpsColor)
        pauseSwiper!!.iconColor = Color.WHITE

        //PROGRESS BAR
        rangeProgress!!.progressColor = Color.parseColor(this.gpsColor)

        //AUDIO SWITCH
        val states = arrayOf(intArrayOf(-android.R.attr.state_checked), intArrayOf(android.R.attr.state_checked))
        val thumbColors = intArrayOf(
                Color.GRAY,
                Color.parseColor(this.gpsColor))
        val trackColors = intArrayOf(
                Color.GRAY,
                Color.parseColor(this.gpsColor))
        DrawableCompat.setTintList(DrawableCompat.wrap(audioSwitch!!.getThumbDrawable()), ColorStateList(states, thumbColors))
        DrawableCompat.setTintList(DrawableCompat.wrap(audioSwitch!!.getTrackDrawable()), ColorStateList(states, trackColors))

        //RECENTER BUTTON
        recenterButton!!.backgroundTintList = ColorStateList.valueOf(Color.parseColor(this.gpsColor))
        recenterButton!!.setBackgroundColor(Color.parseColor(this.gpsColor))
    }

    /*private fun initBottomSheets(){
        bottomSheetPoiBehavior = BottomSheetBehavior.from(bottomSheetPoiLayout)
        bottomSheetPoiBehavior?.setBottomSheetCallback(object : BottomSheetBehavior.BottomSheetCallback() {
            override fun onStateChanged(bottomSheet: View, newState: Int) {
                // React to state change
                when (newState) {
                    BottomSheetBehavior.STATE_HIDDEN -> {
                        recenterButton!!.show()
                    }
                    BottomSheetBehavior.STATE_EXPANDED -> {
                        recenterButton!!.hide()
                    }
                    BottomSheetBehavior.STATE_HALF_EXPANDED -> {
                        recenterButton!!.hide()
                    }
                    BottomSheetBehavior.STATE_COLLAPSED -> {
                        recenterButton!!.show()
                    }
                    BottomSheetBehavior.STATE_DRAGGING -> {
                    }
                    BottomSheetBehavior.STATE_SETTLING -> {
                    }
                }
            }

            override fun onSlide(bottomSheet: View, slideOffset: Float) {
                // React to dragging events
            }
        })

        //bottomSheetEndBehavior = BottomSheetBehavior.from(bottomSheetRate)
        bottomSheetEndBehavior?.setBottomSheetCallback(object : BottomSheetBehavior.BottomSheetCallback() {
            override fun onStateChanged(bottomSheet: View, newState: Int) {
                // React to state change
                when (newState) {
                    BottomSheetBehavior.STATE_HIDDEN -> {

                    }
                    BottomSheetBehavior.STATE_EXPANDED -> {
                        recenterButton!!.hide()
                    }
                    BottomSheetBehavior.STATE_HALF_EXPANDED -> {
                        recenterButton!!.hide()
                    }
                    BottomSheetBehavior.STATE_COLLAPSED -> {
                    }
                    BottomSheetBehavior.STATE_DRAGGING -> {
                    }
                    BottomSheetBehavior.STATE_SETTLING -> {
                    }
                }
            }

            override fun onSlide(bottomSheet: View, slideOffset: Float) {
                // React to dragging events
            }
        })

        bottomSheetPoiLayout!!.setOnClickListener {

        }

        playPoiButton!!.setOnClickListener {
            /*if(currentDisplayedPOI?.arrayMP3!!.isEmpty())
                audioService.playTTS(SpeechAnnouncement.builder()
                        .announcement(currentDisplayedPOI?.description)
                        .build(), false)
            else*/
                audioService.playDescriptionMP3(currentDisplayedPOI!!, this)

            playPoiButton!!.visibility = View.GONE
            stopPoiButton!!.visibility = View.VISIBLE
            mp3Fillprogresslayout!!.visibility = View.VISIBLE
            Handler().postDelayed({
                mp3FillprogresslayoutStatic!!.visibility = View.GONE
            }, 750)
        }

        stopPoiButton!!.setOnClickListener {
            audioService.stop()

            playPoiButton!!.visibility = View.VISIBLE
            stopPoiButton!!.visibility = View.GONE
            mp3Fillprogresslayout!!.visibility = View.GONE
            mp3FillprogresslayoutStatic!!.visibility = View.VISIBLE

            this.onListeningFinished()
        }
    }*/

    private fun initFabMenu(){
        musicFab!!.setOnClickListener(){
            methodChannel!!.invokeMethod("onMusicClick", null)
        }

        menufab!!.setOnMenuButtonClickListener {
            if(greyBackground!!.visibility != View.VISIBLE ){
                greyBackground!!.visibility = View.VISIBLE
                musicFab!!.hide()
                menufab!!.open(true)
            } else {
                greyBackground!!.visibility = View.INVISIBLE
                if(!this.whiteLabel!!.isWhiteLabel!!) { //not show music fab in white label
                    musicFab!!.show()
                }
                menufab!!.close(false)
            }
        }

        reportFab!!.setOnClickListener(){
            methodChannel!!.invokeMethod("onReportClick", null)
        }
        soundFab!!.setOnClickListener(){
            updateAudioGuide(!isMapboxSpeakerMuted, true)
        }
        emergencyFab!!.setOnClickListener(){
            methodChannel!!.invokeMethod("onEmergencySmsClick", null)
        }
    }

    private fun updateAudioGuide(isActivated: Boolean, updateSwitch: Boolean) {
        isMapboxSpeakerMuted = isActivated

        if(updateSwitch) audioSwitch!!.isChecked = !isMapboxSpeakerMuted
        if(isMapboxSpeakerMuted) {
            soundFab!!.labelText = _activity.getString(R.string.fab_sound_inctivated);
        } else {
            soundFab!!.labelText = _activity.getString(R.string.fab_sound_activated);
        }
    }

    override fun dispose() {

        if (disposed) {
            return
        }
        disposed = true

        /*if (circleManager != null) {
            circleManager.onDestroy()
        }*/
        mapboxMap.locationComponent.isLocationComponentEnabled = false

        mapView.onDestroy()
        audioService.stopAudioService(false)

        // Ensure proper shutdown of the SpeechPlayer
        if (speechPlayer != null) {
            speechPlayer!!.onDestroy()
        }


        NavigationUtils.stopTimer()
        NavigationUtils.currentStopWatchTimer = null

        // Prevent leaks
        removeLocationEngineListener()

        (navigation!!.cameraEngine as DynamicCamera).clearMap()

        // MapboxNavigation will shutdown the LocationEngine
        navigation!!.onDestroy()
        _activity.application.unregisterActivityLifecycleCallbacks(this)
    }


    private fun showNavigationHolderLayout(visible: Boolean){

    }

    fun displayAverageSpeed(): String{
        // s= d/t
        var timehour: Float =0F
        var speed: Float =0F
        if(hasRunStarted ){
            //time in sec then in hours
            var timesec =(NavigationUtils.currentStopWatchTimer!!.elapsedTime/1000).toFloat()
            timehour = timesec/3600
        }
        //distance traveled
        var distance : Float = 0F

        if(totalDistanceTravelled == null) distance= 0F
        else{
            val distanceInMeters = totalDistanceTravelled.toFloat()
            if(ApplicationRunner.appDistanceMetrics == MetricsCode.METRIC){
                var distanceKM = (distanceInMeters/1000.toFloat())
                distance = distanceKM
            }
            else if(ApplicationRunner.appDistanceMetrics == MetricsCode.IMPERIAL){
                var distanceInFeet = distanceInMeters * 3.281
                var distanceInMiles = (distanceInFeet/5280.toFloat())
                distance= distanceInMiles.toFloat()
            }
        }
        if(timehour!=0F){
            speed = distance/timehour
        }
        return speed.toString()
    }

    private fun showVisibilityToolbar(visible: Boolean){
        /*if(visible) toolbar!!.visibility = View.VISIBLE
        else toolbar!!.visibility = View.GONE*/
    }

    private fun initMap(){
        mapboxMap.uiSettings.setCompassFadeFacingNorth(false)
        mapboxMap.uiSettings.isLogoEnabled = false
        mapboxMap.uiSettings.isAttributionEnabled = false
        var arrayLatLng = MapUtils.convertPointsToLatLng(MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute?.arrayCoordinates!!))
        if(lastLocation != null)
            arrayLatLng.add(LatLng(lastLocation?.latitude!!, lastLocation?.longitude!!))


        val latLngBounds = LatLngBounds.Builder()
                .includes(arrayLatLng)
                .build()

        mapboxMap.moveCamera(CameraUpdateFactory.newLatLngBounds(latLngBounds, 50))
    }

    /**
     * Enabling location symbol on the map if possible
     */
    private fun enableLocationComponent(loadedMapStyle: Style) {

        // Check if permissions are enabled and if not request
        if (PermissionsManager.areLocationPermissionsGranted(_context)) {

            // Create and customize the LocationComponent's options
            val customLocationComponentOptions = LocationComponentOptions.builder(_context)
                    .trackingGesturesManagement(true)
                    .accuracyColor(ContextCompat.getColor(_context, R.color.colorPrimaryDark))
                    .compassAnimationEnabled(true)
                    .layerAbove(INVISIBLE_LAYER)
                    .elevation(4f)
                    .build()

            val locationComponentActivationOptions = LocationComponentActivationOptions.builder(_context, loadedMapStyle)
                    .locationComponentOptions(customLocationComponentOptions)
                    .locationEngine(locationEngine)
                    .build()

            // Get an instance of the LocationComponent and then adjust its settings
            mapboxMap.locationComponent.apply {

                // Activate the LocationComponent with options
                activateLocationComponent(locationComponentActivationOptions)

                // Enable to make the LocationComponent visible
                isLocationComponentEnabled = true

                // Set the LocationComponent's camera mode
                cameraMode = CameraMode.TRACKING_GPS_NORTH

                // Set the LocationComponent's render mode
                renderMode = RenderMode.GPS

                moveCameraOverhead()
            }
        } else {
            PermissionManager.checkIfLocationIsGranted(_activity)

            //permissionsManager = PermissionsManager(this)
            //permissionsManager.requestLocationPermissions(this)
        }
    }

    /**
     * Init MapBox instruction view with user settings like language (for metrics)
     */
    private fun initInstructionView(){
        if(AppDataHolder.currentRoute != null && AppDataHolder.currentRoute!!.isActivated!!){
            val roundingIncrement = NavigationConstants.ROUNDING_INCREMENT_TWENTY_FIVE
            val distanceFormatter = DistanceFormatter(_context, ApplicationRunner.appLanguage.getLocaleLanguage(), ApplicationRunner.appDistanceMetrics.toString(), roundingIncrement)
            instructionView.setDistanceFormatter(distanceFormatter)

            instructionView.setInstructionListListener { visible ->
                if(visible) showRecenterButton(true)
                else showRecenterButton(false)
            }
        }else{
            instructionView.visibility = View.GONE
        }
    }

    /**
     * Displays a custom instruction view in case MapBox navigation is disabled - some routes does not use MapBox navigation
     */
    private fun showCustomInstructionView(show: Boolean){
        if(show) instructionViewNoNavigation!!.visibility = View.VISIBLE
        else instructionViewNoNavigation!!.visibility = View.GONE
    }
    /**
     * Callback once map is ready and loaded - we start to do things
     */
    override fun onMapReady(mapboxMap: MapboxMap) {
        this.mapboxMap = mapboxMap
        if (mapReadyResult != null) {
            mapReadyResult!!.success(null)
            mapReadyResult = null
        }
        this.mapboxMap.addOnMapClickListener(this)
        mapboxMap.setStyle(Style.Builder().fromUri(_activity.getString(R.string.map_uri))) { style ->

            initMap()
            //circleManager = CircleManager(mapView, mapboxMap, style)
            loadedStyle = style
            mapState = MapState.INFO
            navigationMap = NavigationMapboxMap(mapView, mapboxMap)

            mapboxMap.addOnMoveListener(object: MapboxMap.OnMoveListener{
                override fun onMoveBegin(detector: MoveGestureDetector) {
                }

                override fun onMove(detector: MoveGestureDetector) {
                    if(hasLeadStarted || hasRunStarted){
                        //if(bottomSheetPoiBehavior?.state == BottomSheetBehavior.STATE_HIDDEN) // If POI bottom sheet is visible, we don't show it
                        recenterButton!!.show()
                    }
                }

                override fun onMoveEnd(detector: MoveGestureDetector) {
                }

            })

            displayRouteOnMap()

            // We add POI images on map
            displayPOIS(style)
            displaySponsorOnMap(style)
            displaySportsSesssion(style)

            // For Location updates
            initializeLocationEngine()

            initLocalizationPlugin(mapView, mapboxMap, style)

            style.addImage(DEPARTURE_MARKER_SOURCE, _activity.resources.getDrawable(R.drawable.start_flag))
            style.addImage(ARRIVAL_MARKER_SOURCE, _activity.resources.getDrawable(R.drawable.finish_flag))

            //setFlutterCustomColor()

            // For annotations
            //initAnnotationManager(mapView, mapboxMap, style)

            //style.addLayer(SymbolLayer())

            // For navigation logic / processing
            initializeNavigation(mapboxMap)
            //initObservers()


            navigationMap!!.updateCameraTrackingMode(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
            navigationMap!!.updateLocationLayerRenderMode(RenderMode.GPS)

            // For voice instructions
            initializeSpeechPlayer()
        }
    }

    /**
     * Internet method to display all route POIs on the map (markers)
     */
    private fun displayPOIS(style : Style){

        var arrayPOIS = AppDataHolder.currentRoute!!.arrayPois
        if(AppDataHolder.currentRoute!!.isStartingPoints!!){
            arrayPOIS = ArrayList<POI>(arrayPOIS.subList(AppDataHolder.currentStartingPoint?.posPOI!!, arrayPOIS.size))
        }

        POI.Type.values().forEach { type ->
            // Applying image for current type of POI
            if(!hasUserRequestedMockNavigation){
                if(MapUtils.shouldDisplayPOICateg(type))
                    style.addImage(type.id.toString(), _activity.resources.getDrawable(POI.Type.getDrawableID(type.id)))
            } else {
                style.addImage(type.id.toString(), _activity.resources.getDrawable(POI.Type.getDrawableID(type.id)))
            }


            // We get all pois matching current type from route
            var matchingPOIS = arrayPOIS.filter { poi ->
                poi.idType == type
            }

            // Adding locations for current type of POI
            var poiFeatures = ArrayList<Feature>()
            matchingPOIS.forEach { poi ->
                var feature = Feature.fromGeometry(Point.fromLngLat(poi.arrayCoordinates.first().longitude, poi.arrayCoordinates.first().latitude))
                poiFeatures.add(feature)
                displayedFeatures.add(feature)

                if(hasUserRequestedMockNavigation){

                    // Adding source for current type of POI
                    if(style.getSource(RADIUS_MARKER_SOURCE + poi.id) == null)
                        style.addSource(
                                GeoJsonSource(RADIUS_MARKER_SOURCE + poi.id, TurfTransformation.circle(
                                        Point.fromLngLat(poi.mainCoordinates.longitude, poi.mainCoordinates.latitude), poi.radius.toDouble(), 64, "meters")))

                    val layer = FillLayer(RADIUS_MARKER_STYLE_LAYER + poi.id, RADIUS_MARKER_SOURCE + poi.id)
                    //val layer = CircleLayer(poi.id, RADIUS_MARKER_SOURCE + poi.id)
                    layer.withProperties(
                            PropertyFactory.fillColor("#000000"),
                            PropertyFactory.fillOpacity(0.5f)
                    )

                    //style.addLayerAbove(layer, MARKER_STYLE_LAYER+type.toString())
                    //style.addLayerBelow(layer, INVISIBLE_LAYER)
                    style.addLayerAt(layer, 50)
                }
            }

            // Adding source for current type of POI
            if(style.getSource(MARKER_SOURCE+type.toString()) == null)
                style.addSource(GeoJsonSource(MARKER_SOURCE+type.toString(), FeatureCollection.fromFeatures(poiFeatures)))


            // Applying layer for current type of POI
            if(style.getLayer(MARKER_STYLE_LAYER+type.toString()) == null)
                style.addLayerAbove(SymbolLayer(MARKER_STYLE_LAYER+type.toString(), MARKER_SOURCE+type.toString())
                        .withProperties(
                                PropertyFactory.iconAllowOverlap(true),
                                PropertyFactory.iconIgnorePlacement(true),
                                PropertyFactory.iconImage(type.id.toString()),
                                PropertyFactory.iconSize(0.5f),
                                PropertyFactory.iconOffset(arrayOf(0f, -52f))
                        ), "route-line-layer")
            /**style.addLayerAt(SymbolLayer(MARKER_STYLE_LAYER+type.toString(), MARKER_SOURCE+type.toString())
            .withProperties(
            iconAllowOverlap(true),
            iconIgnorePlacement(true),
            iconImage(type.id.toString()),
            iconSize(0.5f),
            // Adjust the second number of the Float array based on the height of your marker image.
            // This is because the bottom of the marker should be anchored to the coordinate point, rather
            // than the middle of the marker being the anchor point on the map.
            iconOffset(arrayOf(0f, -52f))
            ), 60)**/
        }
    }

    private fun displaySponsorOnMap(style : Style) {

        if(this.sponsorData.size > 0 && this.sponsorData.containsKey("GPS")) {
            var data: Map<String, Double> = this.sponsorData["GPS"] as Map<String, Double>
            var sponsorPoint = Location("provider")
            sponsorPoint.longitude = data["Longitude"]!!
            sponsorPoint.latitude = data["Latitude"]!!

            var sponsorFeature = Feature.fromGeometry(Point.fromLngLat(sponsorPoint.longitude!!, sponsorPoint.latitude!!))
            style.addImage(SPONSOR_MARKER_SOURCE, _activity.resources.getDrawable(R.drawable.pin_sponsor_img))

            style?.addSource(GeoJsonSource(SPONSOR_MARKER_SOURCE, FeatureCollection.fromFeature(sponsorFeature)))

            style?.addLayerAbove(SymbolLayer(SPONSOR_MARKER_SOURCE, SPONSOR_MARKER_SOURCE)
                    .withProperties(
                            PropertyFactory.iconAllowOverlap(true),
                            PropertyFactory.iconIgnorePlacement(true),
                            PropertyFactory.iconImage(SPONSOR_MARKER_SOURCE),
                            PropertyFactory.iconSize(0.3f),
                            PropertyFactory.iconOffset(arrayOf(0f, -52f))
                    ), "route-line-layer")
        }
    }

    private fun displaySportsSesssion(style : Style){
        if(AppDataHolder.currentRoute!!.arraySportSession.size > 0) {
            AppDataHolder.currentRoute!!.arraySportSession.forEach { sportSession ->

                var index = 0;
                for(loc in sportSession.arrayCoordinates) {
                    var markerId: String = SS_MARKER_SOURCE+sportSession.id.toString()+"_"+index.toString()
                    var sponsorFeature = Feature.fromGeometry(Point.fromLngLat(loc.longitude, loc.latitude))
                    style.addImage(markerId, _activity.resources.getDrawable(R.drawable.pin_workout))

                    style?.addSource(GeoJsonSource(markerId, FeatureCollection.fromFeature(sponsorFeature)))

                    style?.addLayerAbove(SymbolLayer(markerId, markerId)
                            .withProperties(
                                    PropertyFactory.iconAllowOverlap(true),
                                    PropertyFactory.iconIgnorePlacement(true),
                                    PropertyFactory.iconImage(markerId),
                                    PropertyFactory.iconSize(0.5f),
                                    PropertyFactory.iconOffset(arrayOf(0f, -52f))
                            ), "route-line-layer")
                    index++;
                }
                /*var sponsorFeature = Feature.fromGeometry(Point.fromLngLat(sportSession.arrayCoordinates!!.first().longitude!!, sportSession.arrayCoordinates!!.first().latitude!!))
                style.addImage(SS_MARKER_SOURCE+sportSession.id.toString(), _activity.resources.getDrawable(R.drawable.pin_workout))

                style?.addSource(GeoJsonSource(SS_MARKER_SOURCE+sportSession.id.toString(), FeatureCollection.fromFeature(sponsorFeature)))

                style?.addLayerAbove(SymbolLayer(SS_MARKER_SOURCE+sportSession.id.toString(), SS_MARKER_SOURCE+sportSession.id.toString())
                        .withProperties(
                                PropertyFactory.iconAllowOverlap(true),
                                PropertyFactory.iconIgnorePlacement(true),
                                PropertyFactory.iconImage(SS_MARKER_SOURCE+sportSession.id.toString()),
                                PropertyFactory.iconSize(0.5f),
                                PropertyFactory.iconOffset(arrayOf(0f, -52f))
                        ), "route-line-layer")*/
            }
        }
    }

    /**
     * Add specific marker such as departure and arrival points
     */
    private fun displayMarkersOnMap(){

        /*if(AppDataHolder.currentRoute!!.isStartingPoints == null || !AppDataHolder.currentRoute!!.isStartingPoints!!) {

        }*/

        var closestRoutePoint: Point?
        if(AppDataHolder.currentRoute!!.isStartingPoints == null || !AppDataHolder.currentRoute!!.isStartingPoints!!) {
            closestRoutePoint = MapUtils.getClosestPointFromUser(lastLocation!!, MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.arrayCoordinates!!))
        } else {
            closestRoutePoint = MapUtils.getClosestPointFromUser(AppDataHolder.currentStartingPoint!!.getLocation(), MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.arrayCoordinates!!))

        }
        var indexOfClosestRoutePoint: Int = MapUtils.getIndexClosestPointFromUser(lastLocation!!, MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.arrayCoordinates!!))
        var finishRoutePoint: Location?

        //If with lucky near point is the first position of tab or if route has a starting point
        if (indexOfClosestRoutePoint == 0 || (AppDataHolder.currentRoute!!.isStartingPoints != null && AppDataHolder.currentRoute!!.isStartingPoints!!)) {
            finishRoutePoint = AppDataHolder.currentRoute!!.arrayCoordinates!!.get(AppDataHolder.currentRoute!!.arrayCoordinates!!.size - 1 - 2)
        } else if (indexOfClosestRoutePoint == 1) {
            finishRoutePoint = AppDataHolder.currentRoute!!.arrayCoordinates!!.last()
        } else {
            finishRoutePoint = AppDataHolder.currentRoute!!.arrayCoordinates!!.get(indexOfClosestRoutePoint - 2)
        }

        var departureFeature = Feature.fromGeometry(Point.fromLngLat(closestRoutePoint?.longitude()!!, closestRoutePoint?.latitude()!!))
        var arrivalFeature = Feature.fromGeometry(Point.fromLngLat(finishRoutePoint.longitude!!, finishRoutePoint.latitude!!))

        loadedStyle?.removeLayer(DEPARTURE_MARKER_SOURCE)
        loadedStyle?.removeLayer(ARRIVAL_MARKER_SOURCE)
        loadedStyle?.removeSource(DEPARTURE_MARKER_SOURCE)
        loadedStyle?.removeSource(ARRIVAL_MARKER_SOURCE)

        loadedStyle?.addSource(GeoJsonSource(DEPARTURE_MARKER_SOURCE, FeatureCollection.fromFeature(departureFeature)))
        loadedStyle?.addSource(GeoJsonSource(ARRIVAL_MARKER_SOURCE, FeatureCollection.fromFeature(arrivalFeature)))


        if(loadedStyle?.getLayer("route-line-layer") != null) {
            loadedStyle?.addLayerAbove(SymbolLayer(DEPARTURE_MARKER_SOURCE, DEPARTURE_MARKER_SOURCE)
                    .withProperties(
                            PropertyFactory.iconAllowOverlap(true),
                            PropertyFactory.iconIgnorePlacement(true),
                            PropertyFactory.iconImage(DEPARTURE_MARKER_SOURCE),
                            PropertyFactory.iconSize(0.5f),
                            PropertyFactory.iconOffset(arrayOf(0f, -52f))
                    ), "route-line-layer")

            loadedStyle?.addLayerAbove(SymbolLayer(ARRIVAL_MARKER_SOURCE, ARRIVAL_MARKER_SOURCE)
                    .withProperties(
                            PropertyFactory.iconAllowOverlap(true),
                            PropertyFactory.iconIgnorePlacement(true),
                            PropertyFactory.iconImage(ARRIVAL_MARKER_SOURCE),
                            PropertyFactory.iconSize(0.5f),
                            PropertyFactory.iconOffset(arrayOf(0f, -52f))
                    ), "route-line-layer")
        }
    }

    fun detectSportSessionClick(point: LatLng): Boolean {
        var clickedFeatures : List<Feature>? = null
        var minDistance = 20000.0
        var closestSS : SportSession? = null

        //Check click for sport session

        AppDataHolder.currentRoute!!.arraySportSession.forEach { sportSession ->
            var index = 0;
            for(loc in sportSession.arrayCoordinates) {
                var screenPoint = mapboxMap.projection.toScreenLocation(point)
                var rectF = RectF(screenPoint.x - 10, screenPoint.y - 10, screenPoint.x + 10, screenPoint.y + 10)
                var markerId: String = SS_MARKER_SOURCE+sportSession.id.toString()+"_"+index.toString()
                var features = mapboxMap.queryRenderedFeatures(rectF, markerId)
                if(features.size > 0)
                    clickedFeatures = features
                index++;
            }
        }

        if(clickedFeatures != null){ // In case we detected to have a click on an element of layer

            AppDataHolder.currentRoute!!.arraySportSession.forEach { sportSession ->
                var distanceBetweenClickAndPois = TurfMeasurement.distance(MapUtils.convertLocationToPoint(sportSession.arrayCoordinates[0]), Point.fromLngLat(point.longitude, point.latitude))
                if(distanceBetweenClickAndPois < minDistance){
                    minDistance = distanceBetweenClickAndPois
                    closestSS = sportSession
                }
            }
        }

        if(closestSS != null){

            val argument: MutableMap<String, Any?> = java.util.HashMap(1)
            argument["activeSportSession"] = closestSS!!.id
            methodChannel!!.invokeMethod("onActiveSportSession", argument)

            //StatisticManager.poiClicked(closestPOI!!)

            //moveCameraOnPOI(closestPOI!!.mainCoordinates)
            //moveCameraTo(closestPOI!!.mainCoordinates)
            //currentDisplayedPOI = closestPOI
            return true
        }else{
            return false
        }
    }

    fun detectPOIClick(point: LatLng): Boolean {
        var clickedFeatures : List<Feature>? = null
        var minDistance = 20000.0
        var closestPOI : POI? = null

        POI.Type.values().forEach { type ->
            var screenPoint = mapboxMap.projection.toScreenLocation(point)
            var rectF = RectF(screenPoint.x - 10, screenPoint.y - 10, screenPoint.x + 10, screenPoint.y + 10)
            var features = mapboxMap.queryRenderedFeatures(rectF, MARKER_STYLE_LAYER+type.toString())
            Log.d("onMapClick", "features for ${type.toString()} found ${features.size}")
            if(features.size > 0)
                clickedFeatures = features
        }

        if(clickedFeatures != null){ // In case we detected to have a click on an element of layer

            AppDataHolder.currentRoute!!.arrayPois.forEach { poi ->
                var distanceBetweenClickAndPois = TurfMeasurement.distance(MapUtils.convertLocationToPoint(poi.mainCoordinates), Point.fromLngLat(point.longitude, point.latitude))
                if(distanceBetweenClickAndPois < minDistance){
                    minDistance = distanceBetweenClickAndPois
                    closestPOI = poi
                }
            }
        }

        if(closestPOI != null){
            if(MapUtils.shouldDisplayPOI(closestPOI!!)){
                AppDataHolder.currentPOI = closestPOI!!

                val argument: MutableMap<String, Any> = java.util.HashMap(1)
                val values: MutableMap<String, Any?> = java.util.HashMap(3)
                values["poi"] = closestPOI!!.id
                values["play"] = false
                values["direction"] = ""
                argument["activePOI"] = values
                methodChannel!!.invokeMethod("onActivePOI", argument)

                //StatisticManager.poiClicked(closestPOI!!)
            }
            //moveCameraOnPOI(closestPOI!!.mainCoordinates)
            //moveCameraTo(closestPOI!!.mainCoordinates)
            currentDisplayedPOI = closestPOI
            return true
        }else{
            return false
        }
    }

    fun detectSponsorClick(point: LatLng): Boolean {

        if(this.sponsorData.size > 0 && this.sponsorData["GPS"] != null) {
            var sponsor: Map<String, Double> = this.sponsorData["GPS"] as Map<String, Double>
            var clickOnSponsor: Boolean = false;
            var clickedFeatures : List<Feature>? = null
            var minDistance = 20000.0

            var screenPoint = mapboxMap.projection.toScreenLocation(point)
            var rectF = RectF(screenPoint.x - 10, screenPoint.y - 10, screenPoint.x + 10, screenPoint.y + 10)
            var features = mapboxMap.queryRenderedFeatures(rectF, SPONSOR_MARKER_SOURCE)
            if(features.size > 0)
                clickedFeatures = features


            if(clickedFeatures != null){ // In case we detected to have a click on an element of layer
                var distanceBetweenClickAndPois = TurfMeasurement.distance(Point.fromLngLat(sponsor["Longitude"]!!, sponsor["Latitude"]!!), Point.fromLngLat(point.longitude, point.latitude))
                if(distanceBetweenClickAndPois < minDistance){
                    minDistance = distanceBetweenClickAndPois
                    clickOnSponsor = true
                }
            }

            if(clickOnSponsor){
                val argument: MutableMap<String, Any> = java.util.HashMap(1)
                argument["sponsorId"] = this.sponsorData["id"]!!
                methodChannel!!.invokeMethod("onSponsorClick", argument)
                return true
            }else{
                return false
            }
        } else {
            return false;
        }
    }

    /**
     * Callback when user clicks for a short time on the map
     */
    override fun onMapClick(point: LatLng): Boolean {

        if(detectPOIClick(point)) {
            return true;
        }
        if(detectSponsorClick(point)){
            return true;
        }
        if(detectSportSessionClick(point)){
            return true;
        }

        return false;
    }

    /**
     * Callback when user clicks for a long time on the map
     */
    override fun onMapLongClick(point: LatLng): Boolean {
        // Only reverse geocode while we are not in navigation
        if (mapState == MapState.NAVIGATION) {
            return false
        }

        // Fetch the route with this given point
        destination = Point.fromLngLat(point.longitude, point.latitude)
        //calculateRouteWith(destination, false)

        // Clear any existing markers and add new one
        navigationMap!!.clearMarkers()
        //navigationMap!!.addMarker(this, destination)

        // Update camera to new destination
        moveCameraToInclude(destination!!)
        //vibrate()
        return false
    }


    @SuppressLint("RestrictedApi")
    private fun showRecenterButton(show: Boolean){
        if(show) recenterButton!!.visibility = View.VISIBLE
        else recenterButton!!.visibility = View.GONE
    }

    /*
     * Navigation listeners
     */

    // Once navigation is finished
    // maybe problem, when lead user to near point, it's not called because checkIfUserIsTooFarFromRoute is called before
    override fun onNavigationFinished() {
        if(hasLeadStarted && !hasRunStarted){
            if(AppDataHolder.currentRoute?.isStartingPoints!!){
                startRunNavigation(MapUtils.computeRouteCoordinatesBasedOnClosestPoint(AppDataHolder.currentStartingPoint?.getLocation()!!, AppDataHolder.currentRoute!!.arrayCoordinates!!.subList(AppDataHolder.currentStartingPoint?.pos!!, AppDataHolder.currentRoute!!.arrayCoordinates!!.size)))
            }else{
                startRunNavigation(MapUtils.computeRouteCoordinatesBasedOnClosestPoint(lastLocation!!, AppDataHolder.currentRoute!!.arrayCoordinates!!))
            }
        }else if(hasRunStarted){
            finishNavigation(true)
        }
    }

    // While navigation is up and running
    override fun onNavigationRunning() {

    }

    // If we cancel navigation
    override fun onCancelNavigation() {

    }

    /**
     * Callback when route navigation is started, we can track progress of user on the route
     */
    override fun onProgressChange(location: Location, routeProgress: RouteProgress) {
        // Cache "snapped" Locations for re-route Directions API requests
        updateLocation(location)
        flutterDistanceRemaining = routeProgress.distanceRemaining()
        flutterDistanceTravelled = routeProgress.distanceTraveled()
        // Update InstructionView data from RouteProgress
        instructionView.updateDistanceWith(routeProgress)

        // Updating stats on the view
        updateStats(routeProgress.distanceRemaining())
        updateRangeProgress(routeProgress.distanceTraveled(),routeProgress.distanceRemaining())
        currentRunSession?.distance = routeProgress.distanceTraveled()

        // Detecting if user has arrived at the end of the route
        val ru = RouteUtils()
        if(ru.isArrivalEvent(routeProgress) && hasRunStarted && AppDataHolder.currentRoute!!.isActivated!!){
            finishNavigation(true)
        }
    }

    private fun updateRangeProgress(distanceTraveled: Double, distanceRemaining : Double) {
        var ratio = distanceTraveled /(distanceTraveled+distanceRemaining)
        var progress= (ratio*100).toFloat()
        rangeProgress!!.setProgress(progress)
    }

    /**
     * Method to update stats, not all the parameters are necessary if they didn't change (enough or not at all)
     */
    private fun updateStats(distanceLeft: Double = 0.0, timer: Long = 0, speed: Float = 0f){
        if(AppDataHolder.currentRoute!!.isActivated!!){
            if(distanceLeft != 0.0)
                distanceLeftTextview!!.text = MetricsUtils.displayDistanceFormatted(distanceLeft)

            currentRunSession?.distance = AppDataHolder.currentRoute?.length?.toDouble()?.minus(distanceLeft)!!

        }else{
            var distanceBetweenLocations = TurfMeasurement.distance(Point.fromLngLat(lastLocation!!.longitude, lastLocation!!.latitude), Point.fromLngLat(previousLocation?.longitude!!, previousLocation?.latitude!!))
            if(distanceBetweenLocations < 1){
                Timber.d("distanceBetweenLocations : $distanceBetweenLocations")
                totalDistanceTravelled += distanceBetweenLocations
                Timber.d("totalDistanceTravelled : $totalDistanceTravelled")
                distanceLeftTextview!!.text = MetricsUtils.displayDistanceFormatted(AppDataHolder.currentRoute!!.length!! - totalDistanceTravelled)
            }
            if(hasRunStarted)
                checkIfUserHasArrived()
        }

        if(/*hasRunStarted &&*/ navigationState == NavigationState.IN_PROGRESS && timerTextview != null && NavigationUtils.currentStopWatchTimer != null){
            if(timer != 0L) {
                timerTextview!!.text = MetricsUtils.displayMillisecondsToSecondsFormatted(NavigationUtils.currentStopWatchTimer!!.elapsedTime)
            }

            System.out.println("Smart fact calculator : " +NavigationUtils.currentStopWatchTimer!!.elapsedTime.toString()+" - "+saveTimeOfLastVocalInstruction.toString() + " = " + (NavigationUtils.currentStopWatchTimer!!.elapsedTime - saveTimeOfLastVocalInstruction).toString())
            //Active Smart fact if no instruction during 3 min
            if(NavigationUtils.currentStopWatchTimer!!.elapsedTime - saveTimeOfLastVocalInstruction > 60000*3) {
                methodChannel!!.invokeMethod("onActiveSmartFact", null)

                saveTimeOfLastVocalInstruction = NavigationUtils.currentStopWatchTimer!!.elapsedTime
            }

            totalTimeInMS += (timer-previousTime)
            previousTime = timer
            currentRunSession?.duration = NavigationUtils.currentStopWatchTimer!!.elapsedTime
        }
        if(speed != 0f){
            //speed is in m/s
            speedTextview!!.text = MetricsUtils.displaySpeedFormatted(speed)
            listSpeeds.add(speed)
            currentRunSession?.arraySpeed?.add(MetricsUtils.getHumanUnderstandableSpeed(speed).toDouble())
        }
        saveStatistics()
    }

    /**
     * Simple method to check if the user has arrived at the end of the route
     */
    private fun checkIfUserHasArrived(){
        Timber.d("distance left : ${AppDataHolder.currentRoute!!.length!! - totalDistanceTravelled}")
        // If user started manual navigation more than 5 minutes ago we check if he has closer than 20m from his starting point to check if he arrived to his point of departure
        if(NavigationUtils.currentStopWatchTimer!!.elapsedTime > 300000){
            if(TurfMeasurement.distance(Point.fromLngLat(lastLocation!!.longitude, lastLocation!!.latitude), Point.fromLngLat(startingLocation!!.longitude, startingLocation!!.latitude)) < 0.20){
                finishNavigation(true)
            }
        }
    }

    /**
     * Called every time mapbox detects a new instruction or event on the route
     */
    override fun onMilestoneEvent(routeProgress: RouteProgress, instruction: String, milestone: Milestone) {
        // Vocally playing the milestone
        playAnnouncement(milestone)

        // Update InstructionView banner instructions
        instructionView.updateBannerInstructionsWith(milestone)
    }

    /**
     * Callback method from mapbox navigation but also used manually in case of polyline navigation (mapbox off)
     */
    override fun userOffRoute(location: Location) {
        if(hasRunStarted && navigationState == NavigationState.IN_PROGRESS && AppDataHolder.currentRoute?.isActivated!!){
            // We check if user is further away than 20 meters from route
            if(MapUtils.getRouteDistanceFromUser(location, MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.arrayCoordinates!!)) > (20/1000)){
                if(lastTimeUserHasBeenOffRoute + 10000 < System.currentTimeMillis()){ // If we didn't warn user for 10 seconds
                    lastTimeUserHasBeenOffRoute = System.currentTimeMillis()
                    if(offlineDatas == null && savedCurrentBlockedPoi.size == 0) { //Not play off route TTS if i'm offline or if i'm in bloquant poi
                        audioService.playTTS(LanguageResource.languagesResources.offRoute, true, false)
                    }
                }
            }
        }else if(offlineDatas == null && hasLeadBeenRequested){ //Prevent crash when starting MatchingNavigation and paused when i'm off route
            leadUserToRunRoute()
        }
    }

    /**
     * Called as soon as we get the first GPS signal / user location
     * Designed to setup the view / logic
     */
    internal fun checkFirstUpdate(location: Location) {
        if (lastLocation == null) {
            moveCameraTo(location)
            // Allow navigationMap clicks now that we have the current Location
            navigationMap!!.retrieveMap().addOnMapLongClickListener(this)
            //showSnackbar(GPS_FOUND_MESSAGE, BaseTransientBottomBar.LENGTH_LONG)
            setGPSSignal(true)
            //showNavigationHolderLayout(true)
            checkIfUserIsTooFarFromRoute(location)
            triggerNavigation(location)
        }
    }

    /**
     * Method made to check if the user is considered as too far from any point of the route or specific location
     */
    private fun checkIfUserIsTooFarFromRoute(userLocation: Location?) : Boolean{
        var points = MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.arrayCoordinates!!)
        //If not add this lines, this function return true when user near to all points
        //But for static point i want only 1 point of route
        if(AppDataHolder.currentRoute!!.isStartingPoints!!) {
            points = ArrayList<Point>()
            points.add(Point.fromLngLat(AppDataHolder.currentStartingPoint!!.getLocation().longitude, AppDataHolder.currentStartingPoint!!.getLocation().latitude))
        }
        var distanceFromRoute = MapUtils.getRouteDistanceFromUser(userLocation!!, points)
        if(distanceFromRoute > 0.010) { // User has to be closer than 10 meters from the route to start it
            isUserTooFarFromRoute = true
            return true
        } else {
            isUserTooFarFromRoute = false
            return false
        }
    }

    /**
     * This methods manages the view (bottom banner with action buttons) based on the state of the navigation
     */
    private fun manageNavigationState(){
        when (navigationState) {
            NavigationState.WAITING_TO_START -> {
                showVisibilityToolbar(false)
                finishButton!!.visibility = View.VISIBLE
                resumeButton!!.visibility = View.GONE
                //navigationState = NavigationState.IN_PROGRESS
            }
            NavigationState.IN_PROGRESS -> {
                finishButton!!.visibility = View.VISIBLE
                resumeButton!!.visibility = View.VISIBLE
                navigationState = NavigationState.PAUSED
                audioService.playTTS(LanguageResource.languagesResources.pauseNavigation, true, true)
                showVisibilityToolbar(false)
                if(NavigationUtils.currentStopWatchTimer != null && !NavigationUtils.currentStopWatchTimer?.isPaused!!) {
                    NavigationUtils.currentStopWatchTimer?.pause() // crash sometimes : java.lang.IllegalStateException: Already Paused
                }
            }
            NavigationState.PAUSED -> {
                navigationState = NavigationState.IN_PROGRESS
                finishButton!!.visibility = View.VISIBLE
                resumeButton!!.visibility = View.GONE
                if(NavigationUtils.currentStopWatchTimer != null && NavigationUtils.currentStopWatchTimer!!.isPaused)
                    NavigationUtils.currentStopWatchTimer?.resume()
                audioService.playTTS(LanguageResource.languagesResources.resumeNavigation, true, true)
            }
            NavigationState.FINISHED -> {
                navigationState = NavigationState.WAITING_TO_START
                finishButton!!.visibility = View.GONE
                resumeButton!!.visibility = View.GONE
                statsLayout!!.visibility = View.GONE
            }
            else -> {
                navigationState = NavigationState.IN_PROGRESS
            }
        }
    }

    /**
     * Methods designed to show / hide components
     */

    private fun showStatsLayout(visible: Boolean){
        if(visible) statsLayout!!.visibility = View.VISIBLE
        else statsLayout!!.visibility = View.GONE
    }

    private fun manageVisibilityMenuLayout(visible: Boolean){
        if(visible) menuLayout!!.visibility = View.VISIBLE
        else menuLayout!!.visibility = View.GONE
    }

    /**
     * Method called every time user changes his location - called by MapBox
     */
    internal fun updateLocation(location: Location) {

        previousLocation = Location("provider")
        if(lastLocation != null){
            previousLocation!!.longitude = lastLocation!!.longitude
            previousLocation!!.latitude = lastLocation!!.latitude
        }

        lastLocation = location
        navigationMap!!.updateLocation(location)
        updateStats(0.0, System.currentTimeMillis(), location.speed)
        checkIfBannerIsDisplayed()

        if(hasUserRequestedNavigation || hasUserRequestedMockNavigation){
            if(hasRunStarted){
                checkIfUserIsNearPOI(location)
                checkIfUserIsNearSportSession(location)
                if(!AppDataHolder.currentRoute!!.isActivated!!){
                    userOffRoute(location)
                } else{
                    saveStatistics()
                }

                //onProgressListener is stopped when start manual navigation for polyline
                //So use this listener for detect some things
                if(AppDataHolder.currentRoute!!.isStartingPoints!!) {

                    listenerForPolyline(location)
                }
            } else{
                if(!checkIfUserIsTooFarFromRoute(location)){
                    navigation!!.stopNavigation()
                    //audioService.playTTS(LanguageResource.getLanguagesResources().routeReached, true, true)
                    if(AppDataHolder.currentRoute?.isStartingPoints!!) {
                        startRunNavigation(MapUtils.computeRouteCoordinatesBasedOnClosestPoint(AppDataHolder.currentStartingPoint?.getLocation()!!, AppDataHolder.currentRoute!!.arrayCoordinates!!.subList(AppDataHolder.currentStartingPoint?.pos!!, AppDataHolder.currentRoute!!.arrayCoordinates!!.size)))
                    } else {
                        startRunNavigation(MapUtils.computeRouteCoordinatesBasedOnClosestPoint(location!!, AppDataHolder.currentRoute!!.arrayCoordinates!!))
                    }
                }
            }
        }

    }

    private fun listenerForPolyline(userLocation: Location) {
        this.flutterDistanceTravelled += userLocation.distanceTo(previousLocation)
        //Check end of route
        val indexLastCoordinate = AppDataHolder.currentRoute!!.arrayCoordinates!!.size - 3
        val lastCoordinateOfRoute = AppDataHolder.currentRoute!!.arrayCoordinates!!.get(indexLastCoordinate)
        val distance = TurfMeasurement.distance(Point.fromLngLat(userLocation.longitude, userLocation.latitude), Point.fromLngLat(lastCoordinateOfRoute.longitude, lastCoordinateOfRoute.latitude))
        if(distance < 0.010) { // User has to be closer than 10 meters from the route to start it
            finishNavigation(true)
        }
    }
    /**
     * Method made to save current run session's statistics in Firebase, we trigger it quite often so we don't lose any user data in case of ANR / crash
     */
    private fun saveStatistics(){

        try{
            averageSpeed = listSpeeds.average().toFloat()
            totalTimeTravelled = NavigationUtils.currentStopWatchTimer!!.elapsedTime
        }catch(exception: Exception){
            Timber.e(exception.localizedMessage)
        }

        // We check first if we need to save stats regarding interval
        if(System.currentTimeMillis() - previousSaveTimeStamp > INTERVAL_BETWEEN_SAVES){
            if(currentRunSession != null){
                // NavigationStatisticManager.saveRunSession(currentRunSession!!)
                previousSaveTimeStamp = System.currentTimeMillis()
            }
        }
    }

    private fun checkIfUserIsNearSportSession(userLocation: Location) {
        AppDataHolder.currentRoute!!.arraySportSession.forEach { sportSession ->
            MapUtils.getDistancesFromSportSession(userLocation, sportSession).forEach { distance -> // Checking distance from every POI
                if(distance <= ((sportSession.radius).toFloat()/1000).toDouble()) { // If user is the SS radius
                    if (!sportSession.isSeen) { // Checking that POI has never been played

                        val argument: MutableMap<String, Any?> = java.util.HashMap(1)
                        argument["activeSportSession"] = sportSession.id
                        methodChannel!!.invokeMethod("onActiveSportSession", argument)

                        sportSession.isSeen = true
                    }
                }
            }
        }
    }

    var savedCurrentBlockedPoi: ArrayList<String> = ArrayList();

    /**
     * Method to check if user is near POI
     */
    private fun checkIfUserIsNearPOI(userLocation: Location){
        //isMapboxSpeakerMuted = checkIfMapboxSpeakIsMuted(userLocation, currentBlockerPOI) // Should we read POI ?
        var allBlockedPoiInCurrentArea: ArrayList<String> = ArrayList();
        AppDataHolder.currentRoute!!.arrayPois.forEach { poi ->
            MapUtils.getDistancesFromPOI(userLocation, poi).forEach { distance -> // Checking distance from every POI
                //Log.d("checkIfUserIsNearPOI", "distance from POI ${poi.name}: $distance <? ${((poi.radius).toFloat()/1000).toDouble()}")
                if(distance <= ((poi.radius).toFloat()/1000).toDouble()){ // If user is the POI radius
                    // If the poi is made to block mapbox intructions
                    /*if(poi.doesBlockMapboxSpeaker != null && poi.doesBlockMapboxSpeaker){
                        currentBlockerPOI = poi
                    }*/

                    if(poi.isBloquant!! || (poi.id!!.contains("POD") && (poi.arrayDescription.values.toTypedArray()[0] == "" ||  poi.arrayDescription.values.toTypedArray()[0] == " "))) {
                        allBlockedPoiInCurrentArea.add(poi.id!!)
                    }

                    if(!poi.isSeen){ // Checking that POI has never been played
                        /*if(poi.isMP3Activated != null && poi.isMP3Activated){ // Checking if we need to play vocal MP3 instead of text
                            audioService.playDescriptionMP3(poi, this)
                            poi.isSeen = true
                        }else{*/ // We read text
                        // Checking if this is a handmade navigation instruction
                        // Directionnal
                        if(poi.isDirectionnal!!) {//if(poi.idType == POI.Type.DIRECTIONNAL_CLOCKWISE || poi.idType == POI.Type.DIRECTIONNAL_COUNTERCLOCKWISE){

                            val argument: MutableMap<String, Any> = java.util.HashMap(1)
                            val values: MutableMap<String, Any?> = java.util.HashMap(3)
                            values["poi"] = poi.id
                            values["play"] = !isMapboxSpeakerMuted
                            values["direction"] = ""
                            argument["activePOI"] = values
                            methodChannel!!.invokeMethod("onActivePOI", argument)

                            //audioService.playTTS(SpeechAnnouncement.builder().announcement(poi.description).build(), true)

                            // We need to display custom instruction
                            if(AppDataHolder.currentRoute!!.isActivated!!){
                                // TODO override mapbox instructions
                            }else{
                                maneuverInstruction!!.text = poi.description
                                //maneuverImage!!.setImageDrawable(_activity.getDrawable(POI.Direction.getDrawableID(poi.idDirection)))
                                Handler().postDelayed({
                                    _activity.runOnUiThread {
                                        maneuverInstruction!!.text = ""
                                        maneuverImage!!.setImageDrawable(null)
                                    }
                                }, INSTRUCTION_DURATION_DISPLAY)
                            }
                        } else{ // This is a "normal" POI
                            //if(AppDataHolder.currentRoute!!.isSynthActivated!!){ // We check that route has synthetizer activated
                            /*if(poi.isAutoDirectionAcivated!!)
                                audioService.playTTS(audioService.generateAudioDescriptionForPOI(previousLocation, userLocation, poi), false, true)
                            else{
                                audioService.playTTS(poi.description, false, true)
                            }*/

                            var direction = ""
                            if(poi.isAutoDirectionAcivated!!)
                                direction = audioService.generateAudioDescriptionForPOI(previousLocation, userLocation, poi)

                            val argument: MutableMap<String, Any> = java.util.HashMap(1)
                            val values: MutableMap<String, Any?> = java.util.HashMap(3)
                            values["poi"] = poi.id
                            values["play"] = !isMapboxSpeakerMuted
                            values["direction"] = direction
                            argument["activePOI"] = values
                            methodChannel!!.invokeMethod("onActivePOI", argument)

                            //}
                            //else{ // We don't read any description, simply a generic message
                            //audioService.playTTS(LanguageResource.languagesResources.infoPOINearby, true, true)
                            //}
                        }
                        if(poi.isBloquant) {
                            poi.isSeen = false
                        } else {
                            poi.isSeen = true
                        }
                    }
                    // As we enter the radius, we check if we need to display the nearby POI
                    if(MapUtils.shouldDisplayPOI(poi)){
                        AppDataHolder.currentPOI = poi
                        //if(poi.id != idGoodDealDisplayed && !isGoodDealDisplayed) displayBottomSheetFor(poi) //Or Banner
                        PhoneUtils.triggerVibration(500)
                    }
                    //StatisticManager.poiListened(poi)
                    //}
                }else{
                    // In case we are no longer in the area of displayed poi -- we hide bottom sheet
                }
            }
        }
        //Save blocker POI for disable mapbox instruction. If List is empty mapbox can speak
        //Check condition on playAnnouncement() methods
        this.savedCurrentBlockedPoi = allBlockedPoiInCurrentArea
    }

    /**
     * Method made to check if MapBox navigation instructions should be read
     */
    private fun checkIfMapboxSpeakIsMuted(userLocation: Location, poi: POI?) : Boolean{
        if(poi == null) return false
        var distanceFromPOI = MapUtils.getDistanceFromPOI(userLocation, poi)
        return distanceFromPOI < ((poi.radius).toFloat()/1000).toDouble() /*&& (poi.doesBlockMapboxSpeaker != null && poi.doesBlockMapboxSpeaker)*/
    }

    // TODO SINGLETON ?
    /**
     * Initialize Android TTS
     */
    private fun initializeSpeechPlayer() {
        val appLanguage = ApplicationRunner.appLanguage.getLocaleLanguage()
        val cache = Cache(
                File(_activity.application.cacheDir, COMPONENT_NAVIGATION_INSTRUCTION_CACHE),
                TEN_MEGABYTE_CACHE_SIZE
        )
        val voiceInstructionLoader = VoiceInstructionLoader(
                _activity.application,
                Mapbox.getAccessToken(), cache
        )
        val speechPlayerProvider = SpeechPlayerProvider(
                _activity.application, appLanguage, true,
                voiceInstructionLoader
        )
        speechPlayer = NavigationSpeechPlayer(speechPlayerProvider)
    }

    /**
     * Init mapview with correct language labels
     */
    private fun initLocalizationPlugin(mapView : MapView, mapboxMap: MapboxMap, style: Style){
        val localizationPlugin = LocalizationPlugin(mapView, mapboxMap, style)

        try {
            localizationPlugin.matchMapLanguageWithDeviceDefault()
        } catch (exception: RuntimeException) {
            Timber.d(exception.toString())
        }
    }

    /**
     * Managing our location engine for the navigation
     */
    @SuppressLint("MissingPermission")
    private fun initializeLocationEngine(route : DirectionsRoute? = null) {

        if(route != null){
            // Fake location engine to do a mock navigation for tests
            locationEngine = ReplayRouteLocationEngine()
            (locationEngine as ReplayRouteLocationEngine).assign(route)
            (locationEngine as ReplayRouteLocationEngine).updateSpeed(currentMockNavigationSpeed)
            val request = buildEngineRequest()
            //navigation!!.setLocationEngineRequest(request)
            locationEngine!!.requestLocationUpdates(request, callback, Looper.getMainLooper())
            navigation!!.locationEngine = (locationEngine as ReplayRouteLocationEngine)
        }else{ // Real location engine to perform real navigation
            locationEngine = LocationEngineProvider.getBestLocationEngine(_activity.applicationContext)

            val request = buildEngineRequest()
            //navigation!!.setLocationEngineRequest(request)
            locationEngine!!.requestLocationUpdates(request, callback, Looper.getMainLooper())
        }

        //showSnackbar(SEARCHING_FOR_GPS_MESSAGE, BaseTransientBottomBar.LENGTH_LONG)
        setGPSSignal(false)
    }

    /**
     * Methods to manage view depending on GPS state
     */
    private fun setGPSSignal(found: Boolean){
        if(!found){
        }
        else{
            Handler().postDelayed({
                Handler().postDelayed({
                    showLoadingGPS(false)
                }, 3000)
            }, 3000)
        }
    }

    private fun showLoadingGPS(visible: Boolean, text: String = ""){
    }

    private fun initializeNavigation(mapboxMap: MapboxMap) {
        navigation = MapboxNavigation(_activity, accessToken)
        navigation!!.setLocationEngineRequest(buildEngineRequest())
        navigation!!.locationEngine = locationEngine!!
        navigation!!.cameraEngine = DynamicCamera(mapboxMap)
        navigation!!.addProgressChangeListener(this)
        navigation!!.addMilestoneEventListener(this)
        navigation!!.addOffRouteListener(this)
        navigationMap!!.addProgressChangeListener(navigation!!)
        navigationMap!!.updateTrafficVisibility(true)
        navigationMap!!.updateIncidentsVisibility(true)
    }

    private fun changeMockNavigationSpeed(speed : Int){
        (locationEngine as ReplayRouteLocationEngine).updateSpeed(1)
        navigation!!.locationEngine = (locationEngine as ReplayRouteLocationEngine)
    }

    private fun displayRouteOnMap(){

        var arrayFeatures = ArrayList<Feature>()
        var sortedArrayPoints: ArrayList<Point>

        if(!AppDataHolder.currentRoute?.isStartingPoints!!){
            sortedArrayPoints = MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.previewCoordinates!!)
            if(AppDataHolder.currentRoute?.isLoop!!)
                sortedArrayPoints.add(MapUtils.convertLocationToPoint(AppDataHolder.currentRoute!!.previewCoordinates!!.first())) // We add first location in order to join first and last location graphically when drawing
        }else{
            sortedArrayPoints = MapUtils.convertCoordinatesToPoints((AppDataHolder.currentRoute!!.previewCoordinates!!.subList(AppDataHolder.currentStartingPoint?.pos!!, AppDataHolder.currentRoute!!.previewCoordinates!!.size)))
        }

        arrayFeatures.add(
                Feature.fromGeometry(
                        LineString.fromLngLats(sortedArrayPoints)
                ))

        loadedStyle!!.addSource(GeoJsonSource(INVISIBLE_SOURCE))
        loadedStyle!!.addLayer(LineLayer(INVISIBLE_LAYER, INVISIBLE_SOURCE))

        // We add polyline of route on map
        // Create the LineString from the list of coordinates and then make a GeoJSON
        // FeatureCollection so we can add the line to our map as a layer.
        try{
            loadedStyle!!.addSource(GeoJsonSource("line-source", FeatureCollection.fromFeatures(arrayFeatures)))
        }catch(exception: Exception){
            Log.e("crash", exception.localizedMessage)
        }
        // The layer properties for our line. This is where we make the line dotted, set the
        // color, etc.
        try{
            loadedStyle!!.addLayerBelow(
                    LineLayer("route-line-layer", "line-source").withProperties(
                            PropertyFactory.iconAllowOverlap(true),
                            PropertyFactory.iconIgnorePlacement(true),
                            PropertyFactory.lineCap(Property.LINE_CAP_ROUND),
                            PropertyFactory.lineJoin(Property.LINE_JOIN_ROUND),
                            PropertyFactory.lineWidth(8f),
                            PropertyFactory.lineOpacity(0.5f),
                            PropertyFactory.lineColor(Color.parseColor("#57C528"))
                    ), INVISIBLE_LAYER)
        }catch(exception: Exception){
            Log.e("crash", exception.localizedMessage)
        }
    }

    private fun displayDebugRouteOnMap(arrayCoordinates : List<Location>){

        var arrayFeatures = ArrayList<Feature>()
        var sortedArrayPoints = MapUtils.convertCoordinatesToPoints(arrayCoordinates)
        sortedArrayPoints.add(MapUtils.convertLocationToPoint(arrayCoordinates.first())) // We add first location in order to join first and last location graphically when drawing
        arrayFeatures.add(
                Feature.fromGeometry(
                        LineString.fromLngLats(sortedArrayPoints)
                ))
        // We add polyline of route on map
        // Create the LineString from the list of coordinates and then make a GeoJSON
        // FeatureCollection so we can add the line to our map as a layer.
        try{
            loadedStyle!!.addSource(GeoJsonSource("line-source-debug", FeatureCollection.fromFeatures(arrayFeatures)))
        }catch(exception: Exception){
            Log.e("crash", exception.localizedMessage)
        }
        // The layer properties for our line. This is where we make the line dotted, set the
        // color, etc.
        try{
            loadedStyle!!.addLayer(
                    LineLayer("route-line-layer", "line-source-debug").withProperties(
                            PropertyFactory.iconAllowOverlap(true),
                            PropertyFactory.iconIgnorePlacement(true),
                            PropertyFactory.lineCap(Property.LINE_CAP_ROUND),
                            PropertyFactory.lineJoin(Property.LINE_JOIN_ROUND),
                            PropertyFactory.lineWidth(3f),
                            PropertyFactory.lineColor(_activity.resources.getColor(R.color.mapbox_navigation_route_alternative_congestion_red))
                    ))
        }catch(exception: Exception){
            Log.e("crash", exception.localizedMessage)
        }
    }

    private fun playAnnouncement(milestone: Milestone) {
        if (milestone is VoiceInstructionMilestone) {

            //recenter map for each instruction only if user do not change camera position manualy
            if(recenterButton!!.isOrWillBeHidden()) {
                mapboxMap.locationComponent.setCameraMode(CameraMode.TRACKING_GPS, 1000, 30.0, null, 0.0, null);
                this.recenterButton!!.hide()
            }

            //check if user not muted navigation or if is not in blocked POI area
            val announcement = SpeechAnnouncement.builder().voiceInstructionMilestone(milestone).build()
            if(!isMapboxSpeakerMuted && savedCurrentBlockedPoi.size == 0)
                AudioService.getInstance().playTTS(announcement, true)
            AudioService.getInstance().isReading

            //Reset smartRun activator for each announcement
            if(NavigationUtils.currentStopWatchTimer != null) {
                saveTimeOfLastVocalInstruction = NavigationUtils.currentStopWatchTimer!!.elapsedTime
            }

            //speechPlayer!!.play(announcement)
        }
        if(milestone is BannerInstructionMilestone && !hasRunStarted) {
            val instructions = milestone.bannerInstructions

            if(!isMapboxSpeakerMuted && instructions.primary().type() == "arrive" && !isArrivedOnSmartRun) {
                val announcement = LanguageResource.languagesResources.navigationArrivedToRoutes
                audioService.playTTS(announcement, true, true)
                isArrivedOnSmartRun = true;

                //Reset smartRun activator for each announcement
                if(NavigationUtils.currentStopWatchTimer != null) {
                    saveTimeOfLastVocalInstruction = NavigationUtils.currentStopWatchTimer!!.elapsedTime
                }


                //System.out.println("ARRIVED")
            }
        }
    }

    /**
     * CAMERA
     */
    private fun moveCameraTo(location: Location) {
        val cameraPosition = buildCameraPositionFrom(location, location.bearing.toDouble())
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun moveCameraOnPOI(location: Location){
        val cameraPosition = buildCameraPositionFrom(location, location.bearing.toDouble())
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun moveCameraToInclude(destination: Point) {
        val origin = LatLng(lastLocation!!)
        val bounds = LatLngBounds.Builder()
                .include(origin)
                .include(LatLng(destination.latitude(), destination.longitude()))
                .build()
        val resources = _activity.resources
        val routeCameraPadding = _activity.resources.getDimension(R.dimen.component_navigation_route_camera_padding).toInt()
        val padding = intArrayOf(
                routeCameraPadding,
                routeCameraPadding,
                routeCameraPadding,
                routeCameraPadding
        )
        val cameraPosition = navigationMap!!.retrieveMap().getCameraForLatLngBounds(bounds, padding)
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition!!), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun moveCameraOverhead() {
        if (lastLocation == null) {
            return
        }
        val cameraPosition = buildCameraPositionFrom(lastLocation!!, DEFAULT_BEARING)
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun cameraOverheadUpdate(): CameraUpdate? {
        if (lastLocation == null) {
            return null
        }
        val cameraPosition = buildCameraPositionFrom(lastLocation!!, DEFAULT_BEARING)
        return CameraUpdateFactory.newCameraPosition(cameraPosition)
    }

    private fun buildCameraPositionFrom(location: Location, bearing: Double): CameraPosition {
        return CameraPosition.Builder()
                .zoom(DEFAULT_ZOOM)
                .target(LatLng(location.latitude, location.longitude))
                .bearing(bearing)
                .tilt(DEFAULT_TILT)
                .build()
    }

    private fun adjustMapPaddingForNavigation() {
        val resources = _activity.resources
        val mapViewHeight = mapView!!.height
        val bottomSheetHeight =
                resources.getDimension(R.dimen.component_navigation_bottomsheet_height).toInt()
        val topPadding = mapViewHeight - bottomSheetHeight * BOTTOMSHEET_PADDING_MULTIPLIER
        navigationMap!!.retrieveMap()
                .setPadding(ZERO_PADDING, topPadding, ZERO_PADDING, ZERO_PADDING)
    }

    private fun resetMapAfterNavigation() {
        navigationMap!!.removeRoute()
        navigationMap!!.clearMarkers()
        addEventToHistoryFile("cancel_navigation")
        navigation!!.stopNavigation()
        moveCameraOverhead()
    }

    private fun removeLocationEngineListener() {
        if (locationEngine != null) {
            locationEngine!!.removeLocationUpdates(callback)
        }
    }

    @SuppressLint("MissingPermission")
    private fun addLocationEngineListener() {
        if (locationEngine != null) {
            val request = buildEngineRequest()
            locationEngine!!.requestLocationUpdates(request, callback, Looper.getMainLooper())
        }
    }


    /**private fun calculateRouteWith(destination: Point?, isOffRoute: Boolean) {
    val origin = Point.fromLngLat(lastLocation!!.longitude, lastLocation!!.latitude)
    val bearing = java.lang.Float.valueOf(lastLocation!!.bearing).toDouble()
    NavigationRoute.builder(this)
    .accessToken(Mapbox.getAccessToken()!!)
    .origin(origin, bearing, BEARING_TOLERANCE)
    .destination(destination!!)
    .build()
    .getRoute(object : Callback<DirectionsResponse> {
    override fun onResponse(
    call: Call<DirectionsResponse>,
    response: Response<DirectionsResponse>
    ) {
    handleRoute(response, isOffRoute)
    }

    override fun onFailure(call: Call<DirectionsResponse>, throwable: Throwable) {
    Timber.e(throwable)
    }
    })
    }**/

    private fun startRunNavigation(arrayCoordinates : List<Location>) {
        // Transition to navigation state
        mapState = MapState.NAVIGATION

        currentRunSession = RunSession.initSession(AppDataHolder.currentRoute!!, System.currentTimeMillis())

        Handler().postDelayed({
            hasRunStarted = true
        }, 2000)

        //cancelNavigationFab.show()

        // Show the InstructionView
        //TransitionManager.beginDelayedTransition(navigationLayout)
        instructionView.visibility = View.VISIBLE

        adjustMapPaddingForNavigation()


        // We check if we need to activate mapbox or not
        if(AppDataHolder.currentRoute!!.isActivated!! && this.locomotion != "polyline" ){
            // Updates camera with last location before starting navigating,
            // making sure the route information is updated
            // by the time the initial camera tracking animation is fired off
            // Alternatively, NavigationMapboxMap#startCamera could be used here,
            // centering the map camera to the beginning of the provided route
            navigationMap!!.resumeCamera(lastLocation!!)
            loadedStyle!!.removeSource("line-source")
            loadedStyle!!.removeLayer("route-line-layer")
            navigationMap!!.updateLocationLayerRenderMode(RenderMode.GPS)
            fetchRoute(arrayCoordinates)
            // Location updates will be received from ProgressChangeListener
            removeLocationEngineListener()
        }else{
            //delayed because finish run direclty when lead is finished
            Handler().postDelayed({
                if(!hasRunStarted) { //condition which fix crash because startManualNavigation() is called 2 times else
                    startManualNavigation()
                }
            }, 1000)

        }

        mapboxMap.locationComponent.setCameraMode(CameraMode.TRACKING_GPS, 1000, 30.0, null, 0.0, null);
        this.recenterButton!!.hide()
        //navigationMap!!.resetCameraPositionWith(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
        /*val cameraUpdate = cameraOverheadUpdate()
        if (cameraUpdate != null) {
            val navUpdate = NavigationCameraUpdate(cameraUpdate)
            navigationMap!!.retrieveCamera().update(navUpdate)
        }*/
    }

    private fun leadUserToRunRoute(){

        //mapState = MapState.INFO
        if(AppDataHolder.currentRoute?.isStartingPoints!!){
            closestRoutePoint = Point.fromLngLat(AppDataHolder.currentStartingPoint?.long!!, AppDataHolder.currentStartingPoint?.lat!!)
        }else{
            closestRoutePoint = MapUtils.getClosestPointFromUser(lastLocation!!, MapUtils.convertCoordinatesToPoints(AppDataHolder.currentRoute!!.arrayCoordinates!!))
        }

        startingLocation = lastLocation

        displayMarkersOnMap()

        //OFFLINE
        if(this.offlineDatas != null) {
            var splitReponse = this.offlineDatas!!.requestResponseJson!!.split("\"routes\":[{");
            var responseWithOptions = splitReponse[0]+"\"routes\":[{\"routeOptions\":"+this.offlineDatas!!.routeOptions+","+splitReponse[1]
            val response: DirectionsResponse = DirectionsResponse.fromJson(responseWithOptions)

            navigation!!.stopNavigation()
            val computedRoute = response.routes()?.first()
            calculatedRouteLeadUserToRun = computedRoute
            navigation!!.startNavigation(computedRoute!!)
            startNavTimestamp = System.currentTimeMillis()
            initDistanceToTravel = computedRoute.distance()!!
            hasLeadStarted = true
            if(hasLeadBeenRequested) {
                audioService.playTTS(LanguageResource.languagesResources.recalculated, true, true)
            } else {
                NavigationUtils.startTimer()

                //Manage TTS for custom begin instruction for white label
                var sentence: String = LanguageResource.languagesResources.startNavigationToRoute
                if(!whiteLabel!!.isWhiteLabel!!) {
                    sentence += LanguageResource.languagesResources.wasteToTrash;
                } else {
                    sentence = whiteLabel!!.beginInstruction + LanguageResource.languagesResources.startNavigationToRoute;
                }
                audioService.playTTS(sentence, true, true)
            }
            showStatsLayout(true)
            navigationState = NavigationState.IN_PROGRESS

            // Transition to navigation state
            mapState = MapState.NAVIGATION
            navigationMap!!.updateLocationLayerRenderMode(RenderMode.GPS)

            //cancelNavigationFab.show()

            // Show the InstructionView
            //TransitionManager.beginDelayedTransition(navigationLayout)
            showLoadingGPS(false)
            instructionView.visibility = View.VISIBLE

            Handler().postDelayed({
                recenterButton!!.hide()
                navigationMap!!.retrieveMap().locationComponent.setCameraMode(CameraMode.TRACKING_GPS, 1000, 30.0, null, 0.0, null);
                //navigationMap!!.updateCameraTrackingMode(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
            }, 3000)

            hasLeadBeenRequested = true
        } else { //ONLINE
            NavigationRoute.builder(_context)
                    .accessToken(this.accessToken)
                    .origin(MapUtils.convertLocationToPoint(lastLocation!!))
                    //.profile(this.locomotion)
                    .profile(DirectionsCriteria.PROFILE_CYCLING)
                    .destination(closestRoutePoint!!)
                    .language(ApplicationRunner.appLanguage.getLocale())
                    .voiceUnits(this.distanceUnit)
                    .build()
                    .getRoute(object : Callback<DirectionsResponse> {
                        override fun onResponse(call: Call<DirectionsResponse>, response: Response<DirectionsResponse>) {
                            if(response.isSuccessful){
                                if(response.body()!!.routes().isNotEmpty()){

                                    if(downloadForOffline) {
                                        //get url
                                        var url: String = call.request().url().toString()

                                        //get coordinate looks like "XX.XXXXX,XX.XXXX;XX.XXXXXX,XX.XXXXX" in String
                                        //And parse to [[(double)lon, (double)lat], [(double)lon, (double)lat]]
                                        var coordinateRaw = call.request().url().pathSegments().last();
                                        var listOfCoordinate = coordinateRaw.split(";")
                                        var start = listOfCoordinate[0].split(",")
                                        var end = listOfCoordinate[1].split(",")
                                        var coordinate: ArrayList<List<Double>> = ArrayList()
                                        var startPoint = listOf(start[0].toDouble(), start[1].toDouble())
                                        var endPointPoint = listOf(end[0].toDouble(), end[1].toDouble())
                                        coordinate.add(startPoint)
                                        coordinate.add(endPointPoint)

                                        val optionsMap: MutableMap<String, Any?> = java.util.HashMap()
                                        optionsMap.put("uuid", response.body()!!.uuid()!!)
                                        optionsMap.put("coordinates", coordinate)
                                        optionsMap.put("baseUrl", "https://api.mapbox.com")
                                        optionsMap.put("user", "mapbox")
                                        optionsMap.put("profile", DirectionsCriteria.PROFILE_CYCLING)
                                        for(key: String in call.request().url().queryParameterNames()) {
                                            //check boolean value and parse string to bool
                                            if(call.request().url().queryParameter(key) == "true" || call.request().url().queryParameter(key) == "false") {
                                                optionsMap.put(key, call.request().url().queryParameter(key)!!.toBoolean())
                                            } else {
                                                optionsMap.put(key, call.request().url().queryParameter(key)!!)
                                            }
                                        }
                                        var routeOptions = JSONObject(optionsMap).toString()

                                        val argument: MutableMap<String, Any> = java.util.HashMap(1)
                                        val values: MutableMap<String, Any?> = java.util.HashMap(2)
                                        values["url"] = url
                                        values["routeOptions"] = routeOptions
                                        argument["onDownloadOfflineFinished"] = values
                                        methodChannel!!.invokeMethod("onDownloadOfflineFinished", argument)
                                    } else {
                                        //run navigate to closest point
                                        navigation!!.stopNavigation()
                                        val computedRoute = response.body()?.routes()?.first()
                                        //val computedRoute = response.routes()?.first()
                                        calculatedRouteLeadUserToRun = computedRoute
                                        navigation!!.startNavigation(computedRoute!!)
                                        startNavTimestamp = System.currentTimeMillis()
                                        initDistanceToTravel = computedRoute.distance()!!
                                        hasLeadStarted = true
                                        if(hasLeadBeenRequested) {
                                            audioService.playTTS(LanguageResource.languagesResources.recalculated, true, true)
                                        } else {
                                            if(!AppDataHolder.currentRoute?.isPerf!!) {
                                                NavigationUtils.startTimer()
                                            }

                                            //Manage TTS for custom begin instruction for white label
                                            var sentence: String = LanguageResource.languagesResources.startNavigationToRoute
                                            if(whiteLabel!!.isWhiteLabel!!) {
                                                sentence = whiteLabel!!.beginInstruction + LanguageResource.languagesResources.startNavigationToRoute;
                                            } else { //add sponsor speaker in rc if have sponsor
                                                if(sponsorData.isNotEmpty()) {
                                                    sentence = LanguageResource.languagesResources.annonceSponsor + " " + sponsorData["Vocal"].toString() + " " + LanguageResource.languagesResources.startNavigationToRoute;
                                                }
                                            }
                                            audioService.playTTS(sentence, true, true)
                                        }
                                        showStatsLayout(true)
                                        navigationState = NavigationState.IN_PROGRESS

                                        // Transition to navigation state
                                        mapState = MapState.NAVIGATION
                                        navigationMap!!.updateLocationLayerRenderMode(RenderMode.GPS)

                                        //cancelNavigationFab.show()

                                        // Show the InstructionView
                                        //TransitionManager.beginDelayedTransition(navigationLayout)
                                        showLoadingGPS(false)
                                        instructionView.visibility = View.VISIBLE

                                        Handler().postDelayed({
                                            recenterButton!!.hide()
                                            navigationMap!!.retrieveMap().locationComponent.setCameraMode(CameraMode.TRACKING_GPS, 1000, 30.0, null, 0.0, null);
                                            //navigationMap!!.updateCameraTrackingMode(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
                                        }, 3000)

                                        hasLeadBeenRequested = true

                                        //adjustMapPaddingForNavigation()

                                        //navigationMap!!.resumeCamera(lastLocation!!)
                                        //navigation!!.stopNavigation()
                                    }
                                }else{
                                    Toast.makeText(_context, _activity.getString(R.string.message_route_not_found), Toast.LENGTH_LONG).show()
                                }
                            }
                        }
                        override fun onFailure(call: Call<DirectionsResponse>, t: Throwable) {
                            Log.e("onFailure", t.localizedMessage)
                            hasLeadBeenRequested = false
                        }
                    })
        }
        //navigation!!.startRunNavigation(route!!)

        // Location updates will be received from ProgressChangeListener
        removeLocationEngineListener()
    }

    private fun startManualNavigation(){
        navigationMap!!.removeRoute()
        navigationMap!!.clearMarkers()
        navigation!!.stopNavigation()
        Handler().postDelayed({
            recenterButton!!.hide()
            navigationMap!!.retrieveMap().locationComponent.setCameraMode(CameraMode.TRACKING_GPS, 1000, 30.0, null, 0.0, null);
            //navigationMap!!.updateCameraTrackingMode(NavigationCamera.NAVIGATION_TRACKING_MODE_GPS)
        }, 3000)
        hasRunStarted = true
        /*loadedStyle!!.addLayer(
                LineLayer("route-line-layer", "line-source").withProperties(
                        PropertyFactory.iconAllowOverlap(true),
                        PropertyFactory.iconIgnorePlacement(true),
                        PropertyFactory.lineCap(Property.LINE_CAP_ROUND),
                        PropertyFactory.lineJoin(Property.LINE_JOIN_ROUND),
                        PropertyFactory.lineWidth(10f),
                        PropertyFactory.lineOpacity(0.5f),
                        PropertyFactory.lineColor(_activity.resources.getColor(R.color.colorPrimary))
                ))*/
        displayMarkersOnMap()
        enableLocationComponent(loadedStyle!!)
        startNavTimestamp = System.currentTimeMillis()
        showStatsLayout(true)
        navigationState = NavigationState.IN_PROGRESS
        addLocationEngineListener()
        NavigationUtils.startTimer()
        initDistanceToTravel = AppDataHolder.currentRoute!!.length!!.toDouble()
        currentRunSession?.totalDistance = AppDataHolder.currentRoute!!.length!!.toDouble()
        startingLocation = lastLocation!!
        initInstructionView()
        audioService.playTTS(LanguageResource.languagesResources.runStarted, true, true)
        //showCustomInstructionView(true)
    }

    private fun fetchRoute(arrayCoordinates : List<Location>){

        // Algorithm to keep as much coordinates as possible from route coordinates
        // The more we keep, the better will be the computed route
        var simplifiedArrayCoordinates = arrayCoordinates
        var coordinatesSize = arrayCoordinates.size
        var dividingFactor = 1
        for(i in dividingFactor..20){
            if(coordinatesSize/i < 100){
                dividingFactor = i
                break
            }
        }
        // We check initial number of coordinates - mapbox accepts maximum 100 coordinates
        // If we have less than 100 coordinates, we keep them all
        if(arrayCoordinates.size > 100){
            simplifiedArrayCoordinates = arrayCoordinates.filterIndexed { index, _ ->
                index % dividingFactor == 0
            }
        }

        //Navigate in offline data
        if(this.offlineDatas != null) {

            //Start navigation
            val route = OfflineUtils.matchingMapboxMapFromOfflineDatas(this.offlineDatas, arrayCoordinates).toDirectionRoute()

            if (hasUserRequestedMockNavigation) {
                initializeLocationEngine(route)
            }

            hasLeadBeenRequested = false;

            calculatedRouteRun = route
            //handleRoute(route, false)
            audioService.playTTS(LanguageResource.languagesResources.runStarted, true, true)
            navigationMap!!.drawRoute(route)
            navigation!!.startNavigation(route)
            navigation!!.locationEngine = locationEngine!!
            startNavTimestamp = System.currentTimeMillis()
            currentRunSession?.totalDistance = route.distance()!!
            showStatsLayout(true)
            navigationState = NavigationState.IN_PROGRESS
            if (NavigationUtils.currentStopWatchTimer == null) {
                NavigationUtils.startTimer()
            }
        } else {
            /**
             * We start a matching request to mapbox api in order to get a route that suits perfectly existing routes
             */
            //if(AppConfiguration.Navigation.debugMode) displayDebugRouteOnMap(simplifiedArrayCoordinates)
            MapboxMapMatching.builder()
                    .accessToken(this.accessToken)
                    .coordinates(MapUtils.convertCoordinatesToPoints(simplifiedArrayCoordinates))
                    .steps(true)
                    .waypointIndices(0, MapUtils.convertCoordinatesToPoints(simplifiedArrayCoordinates).size-1)
                    .voiceInstructions(true)
                    .bannerInstructions(true)
                    .language(ApplicationRunner.appLanguage.getLocale())
                    .profile(this.locomotion)
                    .voiceUnits(this.distanceUnit)
                    .overview(DirectionsCriteria.OVERVIEW_FULL)
                    .build()
                    .enqueueCall(object : Callback<MapMatchingResponse> {
                        override fun onResponse(call: Call<MapMatchingResponse>, response: Response<MapMatchingResponse>) {
                            if (response.isSuccessful) {
                                try {
                                    //download Navigation
                                    if (downloadForOffline) {
                                        //get url
                                        var url: String = call.request().url().toString()

                                        var routeOptions = OfflineUtils.matchingMapboxMapSaveRouteOptions(call, locomotion)

                                        val argument: MutableMap<String, Any> = java.util.HashMap(1)
                                        val values: MutableMap<String, Any?> = java.util.HashMap(2)
                                        values["url"] = url
                                        values["routeOptions"] = routeOptions
                                        argument["onDownloadOfflineFinished"] = values
                                        methodChannel!!.invokeMethod("onDownloadOfflineFinished", argument)

                                    } else {
                                        //Start navigation
                                        val route = response.body()!!.matchings()!![0].toDirectionRoute()

                                        //remove instructions
                                        var legIndex = 0;
                                        route.legs()!!.forEach{ leg ->
                                            var stepIndex = 0;
                                            leg.steps()!!.forEach{ step ->
                                                var instructionIndex = 0
                                                var indexToDelete: ArrayList<Int> = ArrayList()
                                                step.voiceInstructions()!!.forEach { instruction ->
                                                    if(instruction.distanceAlongGeometry()!! > 30.0) {
                                                        indexToDelete.add(instructionIndex)
                                                    }
                                                    instructionIndex += 1;
                                                }
                                                indexToDelete.forEach{ index ->
                                                    if(route.legs()!![legIndex].steps()!![stepIndex].voiceInstructions()!!.size > index) {
                                                        route.legs()!![legIndex].steps()!![stepIndex].voiceInstructions()!!.removeAt(index)
                                                    }
                                                }
                                                stepIndex += 1;
                                            }
                                            legIndex += 1;
                                        }

                                        if (hasUserRequestedMockNavigation) {
                                            initializeLocationEngine(route)
                                        }

                                        hasLeadBeenRequested = false;

                                        calculatedRouteRun = route
                                        //handleRoute(route, false)
                                        audioService.playTTS(LanguageResource.languagesResources.runStarted, true, true)
                                        navigationMap!!.drawRoute(route)
                                        navigation!!.startNavigation(route)
                                        navigation!!.locationEngine = locationEngine!!
                                        startNavTimestamp = System.currentTimeMillis()
                                        currentRunSession?.totalDistance = route.distance()!!
                                        showStatsLayout(true)
                                        navigationState = NavigationState.IN_PROGRESS
                                        if (NavigationUtils.currentStopWatchTimer == null) {
                                            NavigationUtils.startTimer()
                                        }
                                    }
                                } catch (exception: Exception){
                                    Toast.makeText(_context, "Cannot generate route : $exception", Toast.LENGTH_SHORT).show()
                                }

                            }
                        }

                        override fun onFailure(call: Call<MapMatchingResponse>, throwable: Throwable) {

                        }
                    })
        }
    }

    private fun addEventToHistoryFile(type: String) {
        val secondsFromEpoch = Date().time / 1000.0
        navigation!!.addHistoryEvent(type, secondsFromEpoch.toString())
    }

    private fun buildEngineRequest(): LocationEngineRequest {
        return LocationEngineRequest.Builder(UPDATE_INTERVAL_IN_MILLISECONDS)
                .setPriority(LocationEngineRequest.PRIORITY_HIGH_ACCURACY)
                .setFastestInterval(FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS)
                .build()
    }

    /**
     * Calls bottom sheet to be displayed
     */
    /*private fun displayBottomSheetFor(poi: POI){
        AppDataHolder.currentPOI = poi
        bindBottomSheetWith(poi)
        recenterButton!!.hide()
        bottomSheetPoiBehavior?.state = BottomSheetBehavior.STATE_EXPANDED

        if(audioService.isPOIBeingRead(poi)){
            playPoiButton!!.visibility = View.GONE
            mp3Fillprogresslayout!!.visibility = View.VISIBLE
            stopPoiButton!!.visibility = View .VISIBLE
            Handler().postDelayed({
                mp3FillprogresslayoutStatic!!.visibility = View.GONE
            }, 750)
        }else{
            playPoiButton!!.visibility = View.VISIBLE
            stopPoiButton!!.visibility = View.GONE
            mp3Fillprogresslayout!!.visibility = View.GONE
            mp3FillprogresslayoutStatic!!.visibility = View.VISIBLE
        }

        var durationBeforeClosing = 30000L
        /*if(poi.isMP3Activated!!){
            durationBeforeClosing = 60000L
        }*/

        Handler().postDelayed({
            _activity.runOnUiThread {
                bottomSheetPoiBehavior?.state = BottomSheetBehavior.STATE_HIDDEN
            }
        }, durationBeforeClosing)
    }*/

    private fun displayBannerFor(poi: POI){
        val slide_down = AnimationUtils.loadAnimation(_activity.applicationContext, R.anim.slide_down_gooddeal)
        val display = _activity.windowManager.defaultDisplay
        val size = android.graphics.Point()
        display.getSize(size)
        val heightDisplay: Float = size.y - coordinatorBannerPoi!!.y
        _activity.runOnUiThread {
            coordinatorBannerPoi!!.visibility = View.VISIBLE
            coordinatorBannerPoi!!.alpha = 0.0f
            if(poi.imageUrl != null){
                Glide.with(_activity.applicationContext).load(poi.imageUrl).into(imageBannerPoi!!)
                (imageBannerPoi!!).visibility = View.VISIBLE
            }else{
                (imageBannerPoi!!).visibility = View.GONE
            }
            (titleBannerPoi!!).text = poi.name
            (descriptionBannerPoi!!).text = _activity.getString(R.string.text_know_more_about_poi)
            (linkGooddeal!!).setOnClickListener {
                //IntentUtils.startWebIntent(_context, poi.urlWebView, false)
                //StatisticManager.poiClicked(poi)
            }
        }
        coordinatorBannerPoi!!.animate()
                .translationY(_activity.resources.getDimension(R.dimen.instruction_layout_height))
                .alpha(1.0f)
                .setListener(null)
        timeStampStartGooddealAnimation = System.currentTimeMillis()
        isGoodDealDisplayed = true
        idGoodDealDisplayed = poi.id
        Timber.i("Showing gooddeal")
    }

    private fun checkIfBannerIsDisplayed(){
        // Checking if a gooddeal is displayed - if yes and gooddeal timer is over, we hide it
        if (System.currentTimeMillis() - timeStampStartGooddealAnimation > LENGTH_ANIMATION_GOODDEAL_MS && isGoodDealDisplayed) { // Hiding a gooddeal on screen
            _activity.runOnUiThread {
                coordinatorBannerPoi!!.animate()
                        .translationY(0f)
                        .alpha(0.0f)
                        .setListener(object : AnimatorListenerAdapter() {
                            override fun onAnimationEnd(animation: Animator) {
                                super.onAnimationEnd(animation)
                                coordinatorBannerPoi!!.setVisibility(View.INVISIBLE)
                            }
                        })
                isGoodDealDisplayed = false
                Timber.i("Hiding gooddeal")
            }
        }
    }

    /**
     * Fills bottom sheet with data from POI
     */
    /**
     * Fills bottom sheet with data from POI
     */
    /*private fun bindBottomSheetWith(poi: POI){
        /*if(poi.arrayMP3.isEmpty())
            poiPlayer!!.visibility = View.GONE
        else*/
            poiPlayer!!.visibility = View.VISIBLE
        titlePoiTextview!!.text = poi.name
        if(poi.imageUrl != null && poi.imageUrl != ""){
            imagePoiImageview!!.visibility = View.VISIBLE
            Glide.with(_context.applicationContext)
                    .load(poi.imageUrl)
                    .placeholder(R.drawable.placeholder_image_loading)
                    .into(imagePoiImageview!!)
        }else{
            imagePoiImageview!!.visibility = View.GONE
        }

        /**
         * Depending on kind of POI we have to display different content
         */
        if(poi.idType == POI.Type.CHALLENGE)
            descriptionPoiTextview!!.visibility = View.GONE
        else{
            descriptionPoiTextview!!.visibility = View.VISIBLE
            descriptionPoiTextview!!.text = poi.description
        }
    }*/

    /**
     * Methods to detect state of reading of MP3 files on POIs
     */

    override fun onListeningProgress(initialDuration: Long, currentProgress: Int) {
        Log.d("onProgress", ((currentProgress*100)/initialDuration.toInt()).toString())
        mp3Fillprogresslayout!!.setProgress((currentProgress*100)/initialDuration.toInt())
        mp3TimeTextview!!.text = MetricsUtils.displayMillisecondsToSecondsFormatted(currentProgress)
    }

    override fun initMP3Duration(duration: Long) {
        //playPoiButton!!.visibility = View.GONE
        //stopPoiButton!!.visibility = View.VISIBLE
    }

    override fun onListeningFinished() {
        //mp3Fillprogresslayout!!.setProgress(0)
        //mp3TimeTextview!!.text = MetricsUtils.displayMillisecondsToSecondsFormatted(0)
        //playPoiButton!!.visibility = View.VISIBLE
        //stopPoiButton!!.visibility = View.GONE
    }

    /*
     * LocationEngine callback
     */
    private class ComponentActivityLocationCallback internal constructor(activity: MileNavigationEngine2) : LocationEngineCallback<LocationEngineResult> {

        private val activityWeakReference: WeakReference<MileNavigationEngine2>

        init {
            this.activityWeakReference = WeakReference(activity)
        }

        override fun onSuccess(result: LocationEngineResult) {
            val activity = activityWeakReference.get()
            if (activity != null) {
                val location = result.lastLocation ?: return
                activity.checkFirstUpdate(location)
                activity.updateLocation(location)
                Log.d("location", location.toString())
            }
        }

        override fun onFailure(exception: Exception) {
            Timber.e(exception)
        }
    }

    /**
     * Method to finish current navigation properly
     */
    private fun finishNavigation(completed: Boolean){
        if(!hasRouteFinished){
            hasRouteFinished = true
            // Hide the InstructionView
            instructionView.visibility = View.INVISIBLE
            mapState = MapState.INFO

            if(hasRunStarted) {
                saveStatistics()
            }
            if((NavigationUtils.currentStopWatchTimer != null) && (NavigationUtils.currentStopWatchTimer!!.isPaused || NavigationUtils.currentStopWatchTimer!!.isStarted)) {
                NavigationUtils.stopTimer()
            }
            resetMapAfterNavigation()
            bottomSheetEndBehavior?.state = BottomSheetBehavior.STATE_EXPANDED
            bottomSheetEndBehavior?.isHideable = false


            val argument: MutableMap<String, Any> = java.util.HashMap(1)
            val values: MutableMap<String, Any?> = java.util.HashMap(4)
            values["distanceRemaining"] = this.flutterDistanceRemaining
            values["distanceTraveled"] = this.flutterDistanceTravelled
            values["distanceInitial"] = currentRunSession?.totalDistance
            values["time"] = this.totalTimeTravelled
            argument["isFinished"] = values
            methodChannel!!.invokeMethod("onNavigationFinished", argument)

            if(this.whiteLabel!!.isWhiteLabel!! && !this.whiteLabel!!.endInstruction.equals("")) {
                audioService.playTTS(this.whiteLabel!!.endInstruction, true, true)
            } else {
                if (completed) {
                    audioService.playTTS(LanguageResource.languagesResources.congratulationsFinishRoute, true, true)
                } else {
                    audioService.playTTS(LanguageResource.languagesResources.finishNavigation, true, true)
                }
            }
        }
    }

    /**
     * Binding final bottom sheet when route is finished
     */
    private fun bindResumeBottomSheet(){
        //val stars = rating.progressDrawable as LayerDrawable
        //stars.getDrawable(2).setColorFilter(resources.getColor(R.color.colorPrimary), PorterDuff.Mode.SRC_ATOP)

        /*totalDistanceTravelledTextview!!.text = MetricsUtils.displayDistanceFormatted(totalDistanceTravelled)
        totalTimeTextview!!.text = MetricsUtils.displayMillisecondsToSecondsFormatted(totalTimeTravelled)
        averageSpeedTextview!!.text = MetricsUtils.displaySpeedFormatted(averageSpeed)

        rating!!.onRatingBarChangeListener = object: RatingBar.OnRatingBarChangeListener{
            override fun onRatingChanged(p0: RatingBar?, p1: Float, p2: Boolean) {
                hasUserChangedRating = true
            }
        }*/
    }

    open fun init() {
        when (activityState!!.get()) {
            STOPPED -> {
                mapView.onCreate(null)
                mapView.onStart()
                mapView.onResume()
                mapView.onPause()
                mapView.onStop()
            }
            PAUSED -> {
                mapView.onCreate(null)
                mapView.onStart()
                mapView.onResume()
                mapView.onPause()
            }
            RESUMED -> {
                mapView.onCreate(null)
                mapView.onStart()
                mapView.onResume()
            }
            STARTED -> {
                mapView.onCreate(null)
                mapView.onStart()
            }
            CREATED -> {
                mapView.onCreate(null)
            }
            DESTROYED -> {
                mapView.onDestroy()
            }
            else -> throw IllegalArgumentException(
                    "Cannot interpret " + activityState!!.get() + " as an activity state");
        }
        mapView.getMapAsync(this)
    }

    override fun onActivityPaused(activity: Activity?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onPause()
    }

    override fun onActivityResumed(activity: Activity?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onResume()
    }

    override fun onActivityStarted(activity: Activity?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onStart()
        if (navigationMap != null) {
            navigationMap!!.onStart()
        }
    }

    override fun onActivityDestroyed(activity: Activity?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onDestroy()
        audioService.stopAudioService(false)

        // Ensure proper shutdown of the SpeechPlayer
        if (speechPlayer != null) {
            speechPlayer!!.onDestroy()
        }

        // Prevent leaks
        removeLocationEngineListener()

        (navigation!!.cameraEngine as DynamicCamera).clearMap()

        // MapboxNavigation will shutdown the LocationEngine
        navigation!!.onDestroy()
    }

    override fun onActivitySaveInstanceState(activity: Activity?, outState: Bundle?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onSaveInstanceState(outState!!)
    }

    override fun onActivityStopped(activity: Activity?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onStop()
        if (navigationMap != null) {
            navigationMap!!.onStop()
        }
    }

    override fun onActivityCreated(activity: Activity?, savedInstanceState: Bundle?) {
        if (disposed || activity.hashCode() != registrarActivityHashCode) {
            return
        }
        mapView.onCreate(savedInstanceState)
    }
}