package com.mile.v4_navigation_engine;

import android.app.Activity;
import android.content.Context;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentActivity;

import com.mapbox.mapboxsdk.camera.CameraPosition;

import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.StandardMessageCodec;
import io.flutter.plugin.platform.PlatformView;
import io.flutter.plugin.platform.PlatformViewFactory;

public class MileNavigationEngineFactory extends PlatformViewFactory {

    private final Context _context;
    private final Activity _activity;
    private final BinaryMessenger _messenger;

    private final AtomicInteger mActivityState;

    public MileNavigationEngineFactory(AtomicInteger state, Context context, Activity activity, BinaryMessenger messenger) {
        super(StandardMessageCodec.INSTANCE);
        _context = context;
        _activity = activity;
        mActivityState = state;
        _messenger = messenger;
    }

    @Override
    public PlatformView create(Context context, int id, Object args) {
        Map<String, Object> params = (Map<String, Object>) args;
        final MileNavigationEngineBuilder builder = new MileNavigationEngineBuilder();
        String route = "";
        String gpsColor = "";
        String accessToken = "";
        String mode = "";
        String sponsor = "";
        String distanceUnit = "";
        String speedUnit = "";
        String locomotion = "";
        String whiteLabel = "";
        String offlineDatas = "";
        Boolean downloadForOffline = false;

        ArrayList<String> languages = new ArrayList<String>();

        if (params.containsKey("route")) {
            route = (String) params.get("route");
        }

        if (params.containsKey("gpsColor")) {
            if(params.get("gpsColor") == null) {
                gpsColor = "";
            } else {
                gpsColor = (String)params.get("gpsColor");
            }
        }

        if (params.containsKey("accessToken")) {
            accessToken = (String) params.get("accessToken");
        }
        if (params.containsKey("mode")) {
            mode = (String) params.get("mode");
        }
        if (params.containsKey("languages")) {
            languages = (ArrayList<String>) params.get("languages");
        }
        if (params.containsKey("sponsor")) {
            sponsor = (String) params.get("sponsor");
        }
        if (params.containsKey("distanceUnit")) {
            distanceUnit = (String) params.get("distanceUnit");
        }
        if (params.containsKey("speedUnit")) {
            speedUnit = (String) params.get("speedUnit");
        }
        if (params.containsKey("locomotion")) {
            locomotion = (String) params.get("locomotion");
        }

        if (params.containsKey("whiteLabel")) {
            if(params.get("whiteLabel") == null) {
                whiteLabel = "";
            } else {
                whiteLabel = (String) params.get("whiteLabel");
            }
        }

        if (params.containsKey("offlineDatas")) {
            if(params.get("offlineDatas") == null) {
                offlineDatas = "";
            } else {
                offlineDatas = (String) params.get("offlineDatas");
            }
        }

        if (params.containsKey("downloadForOffline")) {
            if(params.get("downloadForOffline") == null) {
                downloadForOffline = false;
            } else {
                downloadForOffline = (boolean)params.get("downloadForOffline");
            }
        }

        return builder.buildNavigation(id, _context, _activity, route, gpsColor, accessToken, mode, languages, sponsor, distanceUnit, speedUnit, locomotion, whiteLabel, downloadForOffline, offlineDatas, mActivityState, _messenger);

    }
}
