package com.mile.v4_navigation_engine.model

import android.graphics.drawable.Drawable
import android.location.Location
import com.google.gson.Gson
import com.google.gson.internal.LinkedTreeMap
import com.mapbox.api.directions.v5.DirectionsCriteria
import com.mile.v4_navigation_engine.Navigation.ApplicationRunner
import com.mile.v4_navigation_engine.Navigation.model.SmartFact
import com.mile.v4_navigation_engine.R
import com.mile.v4_navigation_engine.utils.AppDataHolder
import org.json.JSONArray
import org.json.JSONObject


/**
Created by Corentin Houdayer on 2019-08-23
Dev profile @ https://github.com/houdayec
 **/

data class RouteDataClass(
        val id : String?,
        val allergensAverage : Long?,
        val ascent : Long?,
        val cities: List<String>?,
        val cityName: String?,
        val countries: List<String>?,
        var arrayDescription : Map<LanguageCode, String>?,
        val easiness: Long?,
        val gpxFileName: String?,
        val length : Long?,
        val lengthCategory: Long?,
        var arrayCoordinates: ArrayList<Location>?,
        var previewCoordinates: ArrayList<Location>?,
        var arrayName : Map<LanguageCode, String>?,
        val pois : List<String>?,
        val globalAQI : Long?,
        val arrayShareURL : Map<LanguageCode, String>?,
        val status : Long?,
        val dateEnd: String?,
        val idCategory: Long?,
        val idCity: String?,
        val imageURL : String?,
        val isActivated : Boolean?,
        val isHoraire : Boolean?,
        val isSmartFactsEnabled : Boolean?,
        val isStartingPoints: Boolean?,
        val timeStamping : Long?,
        val isLoop : Boolean?,
        val isPerf : Boolean?
) {
    var arrayPois = ArrayList<POI>()
    var arraySportSession = ArrayList<SportSession>()

    enum class Locomotion(val id : Long) {
        WALKING(1),
        RUNNING(2),
        BICYCLE(3);

        fun getDirectionCriteria() : String{
            when(this.id.toInt()){
                1 -> return DirectionsCriteria.PROFILE_CYCLING
                2 -> return DirectionsCriteria.PROFILE_CYCLING
                3 -> return DirectionsCriteria.PROFILE_CYCLING
                else -> return DirectionsCriteria.PROFILE_CYCLING
            }
        }

        fun getIcon() : Drawable? {
            when(this.id.toInt()){
                1 -> return ApplicationRunner.instance.getDrawable(R.drawable.ic_shoe_print)
                2 -> return ApplicationRunner.instance.getDrawable(R.drawable.ic_run)
                3 -> return ApplicationRunner.instance.getDrawable(R.drawable.ic_bike)
                else -> return ApplicationRunner.instance.getDrawable(R.drawable.ic_shoe_print)
            }
        }

        companion object {
            fun getLocomotion(id: Long) : Locomotion {
                when(id.toInt()){
                    1 -> return WALKING
                    2 -> return RUNNING
                    3 -> return BICYCLE
                    else -> return WALKING
                }
            }
        }

    }
    val name : String
        get() {
            return arrayName!![ApplicationRunner.appLanguage].toString()
        }

    val description : String
        get() {
            return arrayDescription!![ApplicationRunner.appLanguage].toString()
        }

    val shareURL : String
        get() {
            return arrayShareURL!![ApplicationRunner.appLanguage].toString()
        }


    companion object{
        val ID_FIELD = "id";
        val ALLERGENS_FIELD = "Allergens";
        val ASCENT_FIELD = "Ascent";
        val CITY_FIELD = "City";
        val CITY_NAME_FIELD = "CityName";
        val COUNTRY_FIELD = "Country";
        val DESCRIPTION_FIELD = "Description";
        val EASINESS_FIELD = "Easiness";
        val GPX_FILE_NAME_FIELD = "GpxFileName";
        val LENGTH_FIELD = "Length";
        val LENGTH_CATEGORY_FIELD = "LengthCategorie";
        val MAPPER_COORDINATES_FIELD = "MapperCoordinates";
        val NAME_FIELD = "Name";
        val POIS_FIELD = "POI";
        val POLLUTENTS_FIELD = "Pollutents";
        val SHARE_URL_FIELD = "ShareUrl";
        val STATUS_FIELD = "Status";
        val DATE_END_FIELD = "dateEnd";
        val ID_CATEGORY_FIELD = "IdCategory";
        val ID_CITY_FIELD = "idCity";
        val IMAGE_URL_FIELD = "imageURL";
        val IS_ACTIVATED_FIELD = "isActivated";
        val IS_HORAIRE_FIELD = "isHoraire";
        val IS_LOOP_FIELD = "isLoop";
        val SMART_FACT_ENABLED_FIELD = "isSmartFactsEnabled";
        val STARTING_POINT_STATIC_FIELD = "isStartingPointStatic";
        val TIME_STAMPING_FIELD = "timeStamping";
        val IS_PERF_FIELD = "IsPerf";

        fun getLabelForAirQuality(level: Int?) : String{
            if(level == null) return ApplicationRunner.instance.getString(R.string.label_unknown)
            if(level > 66)
                return ApplicationRunner.instance.getString(R.string.label_good)
            else if(level > 33)
                return ApplicationRunner.instance.getString(R.string.label_medium)
            else
                return ApplicationRunner.instance.getString(R.string.label_bad)
        }

        fun getLabelForAllergensQuality(level: Int?) : String{
            if(level == null) return ApplicationRunner.instance.getString(R.string.label_unknown)
            if(level > 66)
                return ApplicationRunner.instance.getString(R.string.label_bad)
            else if(level > 33)
                return ApplicationRunner.instance.getString(R.string.label_medium)
            else
                return ApplicationRunner.instance.getString(R.string.label_good)
        }

        fun fromFlutter(jsonRoute: String): RouteDataClass {

            System.out.println(jsonRoute);
            val data: Map<*, *> = Gson().fromJson(jsonRoute, MutableMap::class.java)

            val arrayNames = HashMap<LanguageCode, String>()
            // We fetch values which are in multiple language in each available language
            if(data[NAME_FIELD] != null){
                var dataName  = data[NAME_FIELD] as LinkedTreeMap<String, String>?
                dataName?.forEach { name ->
                    arrayNames[LanguageCode.getLanguageCodeFrom(name.key)] = name.value // We fetch "FR" -> "Champagne" from db, we need to convert "FR" to model LanguageCode
                }
            }

            val arrayDescription = HashMap<LanguageCode, String>()
            if(data[DESCRIPTION_FIELD] != null) {
                var dataName = data[DESCRIPTION_FIELD] as LinkedTreeMap<String, String>?
                dataName?.forEach { name ->
                    arrayDescription[LanguageCode.getLanguageCodeFrom(name.key)] = name.value
                }
            }

            val arrayShareURL = HashMap<LanguageCode, String>()
            if (data[SHARE_URL_FIELD] != null) {
                var dataName = data[SHARE_URL_FIELD] as LinkedTreeMap<String, String>?
                dataName?.forEach { name ->
                    arrayShareURL[LanguageCode.getLanguageCodeFrom(name.key)] = name.value
                }
            }

            // Building list of ids from datasnapshot
            var arrayPois = ArrayList<String>()
            //var listPoisSnapshots: HashMap<String, Boolean>?
            if(data[POIS_FIELD] != null){
                arrayPois = data[POIS_FIELD] as ArrayList<String>
                /*listPoisSnapshots?.forEach {
                    if(it.value)
                        arrayPois.add(it.key)
                }*/
            }

            var arrayCities = ArrayList<String>()
            //var listCitiesSnapshots: HashMap<String, Boolean>?
            if(data[CITY_FIELD] != null){
                arrayCities = data[CITY_FIELD] as ArrayList<String>
                /*listCitiesSnapshots?.forEach {
                    if(it.value)
                        arrayCities.add(it.key)
                }*/
            }

            var arrayCoutries = ArrayList<String>()
            //var listCoutriesSnapshots: HashMap<String, Boolean>?
            if(data[COUNTRY_FIELD] != null){
                arrayCoutries = data[COUNTRY_FIELD] as ArrayList<String>
                /*listCoutriesSnapshots?.forEach {
                    if(it.value)
                        arrayCoutries.add(it.key)
                }*/
            }

            var arrayCoordinates = ArrayList<Location>()
            var previewCoordinates = ArrayList<Location>()
            if (data[MAPPER_COORDINATES_FIELD] != null) {
                val feetCoordinates = data[MAPPER_COORDINATES_FIELD] as LinkedTreeMap<String, Any>?
                feetCoordinates?.forEach lit@{ map ->
                    if (map.key == "feet"+data[ID_FIELD]) { //Feet priority
                        val feetCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = feetCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            arrayCoordinates.add(location)
                        }
                        return@lit
                    } else if (map.key == "bike"+data[ID_FIELD]) {
                        val feetCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = feetCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            arrayCoordinates.add(location)
                        }
                        return@lit
                    } else if (map.key == "car"+data[ID_FIELD]) {
                        val feetCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = feetCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            arrayCoordinates.add(location)
                        }
                        return@lit
                    } else if (map.key == "polyline"+data[ID_FIELD]) {
                        val feetCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = feetCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            arrayCoordinates.add(location)
                        }
                        return@lit
                    }
                    if (map.key == "preview"+data[ID_FIELD]) {
                        val previewCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = previewCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            previewCoordinates.add(location)
                        }
                    }
                }
            }

            var airQuality = 0L
            val dataPollutents = data[POLLUTENTS_FIELD] as LinkedTreeMap<String, Any>?
            dataPollutents?.forEach {
                if(it.key == "globalAQI")
                    airQuality = (it.value as Double).toLong()
            }

            var allergensAverage = 0L
            val dataAllergens = data[ALLERGENS_FIELD] as LinkedTreeMap<String, Any>?
            dataAllergens?.forEach {
                if(it.key == "percentageRate")
                    allergensAverage = (it.value as Double).toLong()
            }

            var id = ""
            if(data.containsKey(ID_FIELD))
                id = data[ID_FIELD] as String

            var imageURL = ""
            if(data.containsKey(IMAGE_URL_FIELD))
                imageURL = data[IMAGE_URL_FIELD] as String

            var ascent = 0L
            if(data.containsKey(ASCENT_FIELD))
                ascent = (data[ASCENT_FIELD] as Double).toLong()

            var length = 0L
            if(data.containsKey(LENGTH_FIELD))
                 length = (data[LENGTH_FIELD] as Double).toLong()

            var status = 0L
            if(data.containsKey(STATUS_FIELD))
                 status = (data[STATUS_FIELD] as Double).toLong()

            var areSmartFactsEnabled = true
            if(data.containsKey(SMART_FACT_ENABLED_FIELD))
                 areSmartFactsEnabled = data[SMART_FACT_ENABLED_FIELD] as Boolean

            var isStartingPointStatic = false
            if(data.containsKey(STARTING_POINT_STATIC_FIELD)) {
                isStartingPointStatic = data[STARTING_POINT_STATIC_FIELD] as Boolean
                if(isStartingPointStatic) {
                    AppDataHolder.currentStartingPoint = StartingPoint("", previewCoordinates.get(0).latitude, previewCoordinates.get(0).longitude, 0, 0) //Maybe redo
                }
            }

            var idCategory = 0L
            if(data.containsKey(ID_CATEGORY_FIELD))
                idCategory = (data[ID_CATEGORY_FIELD] as Double).toLong()

            var easiness = 0L
            if(data.containsKey(EASINESS_FIELD))
                 easiness = (data[EASINESS_FIELD] as Double).toLong()

            var cityName = ""
            if(data.containsKey(CITY_NAME_FIELD))
                cityName = data[CITY_NAME_FIELD] as String

            var timeStamping = 0L
            if(data.containsKey(TIME_STAMPING_FIELD))
                 timeStamping = (data[TIME_STAMPING_FIELD] as Double).toLong()

            var lengthCategory = 0L
            if(data.containsKey(LENGTH_CATEGORY_FIELD))
                lengthCategory = (data[LENGTH_CATEGORY_FIELD] as Double).toLong()

            var gpxFileName = ""
            if(data.containsKey(GPX_FILE_NAME_FIELD))
                gpxFileName = data[GPX_FILE_NAME_FIELD] as String

            var dateEnd = ""
            if(data.containsKey(DATE_END_FIELD))
                 dateEnd = data[DATE_END_FIELD] as String

            var idCity = ""
            if(data.containsKey(ID_CITY_FIELD))
                idCity = data[ID_CITY_FIELD] as String

            var isLoop = true
            if(data.containsKey(IS_LOOP_FIELD))
                isLoop =  data[IS_LOOP_FIELD] as Boolean

            var isPerf = false
            if(data.containsKey(IS_PERF_FIELD))
                isPerf =  data[IS_PERF_FIELD] as Boolean

            val isNavigationActivated : Boolean
            if(data[IS_ACTIVATED_FIELD] != null)
                isNavigationActivated = data[IS_ACTIVATED_FIELD] as Boolean
            else
                isNavigationActivated = true // TODO CHANGE DEFAULT

            val isHoraire : Boolean
            if(data[IS_HORAIRE_FIELD] != null)
                isHoraire = data[IS_HORAIRE_FIELD] as Boolean
            else
                isHoraire = true

            val arrayPOI = ArrayList<POI>()
            val poisData = data["arrayPOI"] as ArrayList<*>?
            poisData!!.forEach {
                var poi = POI.fromFlutter(it as LinkedTreeMap<*, *>)
                arrayPOI.add(poi)
            }

            val arraySS = ArrayList<SportSession>()
            val ssData = data["arraySportsSession"] as ArrayList<*>?
            if(ssData != null) {
                ssData!!.forEach {
                    var ss = SportSession.fromFlutter(it as LinkedTreeMap<*, *>)
                    arraySS.add(ss)
                }
            }

            var route = RouteDataClass(id, allergensAverage, ascent, arrayCities, cityName, arrayCoutries, arrayDescription, easiness, gpxFileName,length, lengthCategory, arrayCoordinates, previewCoordinates, arrayNames, arrayPois, airQuality, arrayShareURL, status, dateEnd, idCategory, idCity, imageURL, isNavigationActivated, isHoraire, areSmartFactsEnabled, isStartingPointStatic, timeStamping, isLoop, isPerf)
            route.arrayPois = arrayPOI
            route.arraySportSession = arraySS
            return route
        }

        fun fromFlutterWhiteLabel(jsonRoute: String): RouteDataClass {

            System.out.println(jsonRoute);
            val data: Map<*, *> = Gson().fromJson(jsonRoute, MutableMap::class.java)

            val arrayNames = HashMap<LanguageCode, String>()
            // We fetch values which are in multiple language in each available language
            if(data[NAME_FIELD] != null){
                var dataName  = data[NAME_FIELD] as LinkedTreeMap<String, String>?
                dataName?.forEach { name ->
                    arrayNames[LanguageCode.getLanguageCodeFrom(name.key)] = name.value // We fetch "FR" -> "Champagne" from db, we need to convert "FR" to model LanguageCode
                }
            }

            val arrayDescription = HashMap<LanguageCode, String>()
            if(data[DESCRIPTION_FIELD] != null) {
                var dataName = data[DESCRIPTION_FIELD] as LinkedTreeMap<String, String>?
                dataName?.forEach { name ->
                    arrayDescription[LanguageCode.getLanguageCodeFrom(name.key)] = name.value
                }
            }

            val arrayShareURL = HashMap<LanguageCode, String>()
            if (data[SHARE_URL_FIELD] != null) {
                var dataName = data[SHARE_URL_FIELD] as LinkedTreeMap<String, String>?
                dataName?.forEach { name ->
                    arrayShareURL[LanguageCode.getLanguageCodeFrom(name.key)] = name.value
                }
            }

            // Building list of ids from datasnapshot
            var arrayPois = ArrayList<String>()
            //var listPoisSnapshots: HashMap<String, Boolean>?
            if(data[POIS_FIELD] != null){
                arrayPois = data[POIS_FIELD] as ArrayList<String>
                /*listPoisSnapshots?.forEach {
                    if(it.value)
                        arrayPois.add(it.key)
                }*/
            }

            var arrayCities = ArrayList<String>()
            //var listCitiesSnapshots: HashMap<String, Boolean>?
            if(data[CITY_FIELD] != null){
                arrayCities = data[CITY_FIELD] as ArrayList<String>
                /*listCitiesSnapshots?.forEach {
                    if(it.value)
                        arrayCities.add(it.key)
                }*/
            }

            var arrayCoutries = ArrayList<String>()
            //var listCoutriesSnapshots: HashMap<String, Boolean>?
            if(data[COUNTRY_FIELD] != null){
                arrayCoutries = data[COUNTRY_FIELD] as ArrayList<String>
                /*listCoutriesSnapshots?.forEach {
                    if(it.value)
                        arrayCoutries.add(it.key)
                }*/
            }

            var arrayCoordinates = ArrayList<Location>()
            var previewCoordinates = ArrayList<Location>()
            if(data["arrayCoordinates"] != null){
                var locations: ArrayList<Location> = ArrayList()
                var coor = data["arrayCoordinates"] as ArrayList<*>
    
                for (i in 0 until coor.size) {
                    val item = coor.get(i) as LinkedTreeMap<String, Any>?
                    var location = Location("provider")
                    location.latitude = (item!!.get("latitude") as Double?)!!
                    location.longitude = (item!!.get("longitude") as Double?)!!
                    locations.add(location)
                }
                arrayCoordinates = locations
                previewCoordinates = locations //Maybe redo
            }

            /*var arrayCoordinates = ArrayList<Location>()
            var previewCoordinates = ArrayList<Location>()
            if (data[MAPPER_COORDINATES_FIELD] != null) {
                val feetCoordinates = data[MAPPER_COORDINATES_FIELD] as LinkedTreeMap<String, Any>?
                feetCoordinates?.forEach { map ->
                    if (map.key == "feet"+data[ID_FIELD]) {
                        val feetCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = feetCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            arrayCoordinates.add(location)
                        }
                    }
                    if (map.key == "preview"+data[ID_FIELD]) {
                        val previewCoordinatesValues = map.value as LinkedTreeMap<String, ArrayList<Double>>
                        val sortedMap = previewCoordinatesValues.entries.sortedBy { it.key.toLong() }
                        sortedMap.forEach {
                            var location = Location("provider")
                            location.longitude = it.value.get(0)
                            location.latitude = it.value.get(1)
                            previewCoordinates.add(location)
                        }
                    }
                }
            }*/

            var airQuality = 0L
            val dataPollutents = data[POLLUTENTS_FIELD] as LinkedTreeMap<String, Any>?
            dataPollutents?.forEach {
                if(it.key == "globalAQI")
                    airQuality = (it.value as Double).toLong()
            }

            var allergensAverage = 0L
            val dataAllergens = data[ALLERGENS_FIELD] as LinkedTreeMap<String, Any>?
            dataAllergens?.forEach {
                if(it.key == "percentageRate")
                    allergensAverage = (it.value as Double).toLong()
            }

            var id = ""
            if(data.containsKey(ID_FIELD))
                id = data[ID_FIELD] as String

            var imageURL = ""
            if(data.containsKey(IMAGE_URL_FIELD))
                imageURL = data[IMAGE_URL_FIELD] as String

            var ascent = 0L
            if(data.containsKey(ASCENT_FIELD))
                ascent = (data[ASCENT_FIELD] as Double).toLong()

            var length = 0L
            if(data.containsKey(LENGTH_FIELD))
                length = (data[LENGTH_FIELD] as Double).toLong()

            var status = 0L
            if(data.containsKey(STATUS_FIELD))
                status = (data[STATUS_FIELD] as Double).toLong()

            var areSmartFactsEnabled = true
            if(data.containsKey(SMART_FACT_ENABLED_FIELD))
                areSmartFactsEnabled = data[SMART_FACT_ENABLED_FIELD] as Boolean

            var isStartingPointStatic = false
            if(data.containsKey(STARTING_POINT_STATIC_FIELD)) {
                isStartingPointStatic = data[STARTING_POINT_STATIC_FIELD] as Boolean
                if(isStartingPointStatic) {
                    AppDataHolder.currentStartingPoint = StartingPoint("", previewCoordinates.get(0).latitude, previewCoordinates.get(0).longitude, 0, 0) //Maybe redo
                }
            }

            var idCategory = 0L
            if(data.containsKey(ID_CATEGORY_FIELD))
                idCategory = (data[ID_CATEGORY_FIELD] as Double).toLong()

            var easiness = 0L
            if(data.containsKey(EASINESS_FIELD))
                easiness = (data[EASINESS_FIELD] as Double).toLong()

            var cityName = ""
            if(data.containsKey(CITY_NAME_FIELD))
                cityName = data[CITY_NAME_FIELD] as String

            var timeStamping = 0L
            if(data.containsKey(TIME_STAMPING_FIELD))
                timeStamping = (data[TIME_STAMPING_FIELD] as Double).toLong()

            var lengthCategory = 0L
            if(data.containsKey(LENGTH_CATEGORY_FIELD))
                lengthCategory = (data[LENGTH_CATEGORY_FIELD] as Double).toLong()

            var gpxFileName = ""
            if(data.containsKey(GPX_FILE_NAME_FIELD))
                gpxFileName = data[GPX_FILE_NAME_FIELD] as String

            var dateEnd = ""
            if(data.containsKey(DATE_END_FIELD))
                dateEnd = data[DATE_END_FIELD] as String

            var idCity = ""
            if(data.containsKey(ID_CITY_FIELD))
                idCity = data[ID_CITY_FIELD] as String

            var isLoop = true
            if(data.containsKey(IS_LOOP_FIELD))
                isLoop =  data[IS_LOOP_FIELD] as Boolean

            var isPerf = false
            if(data.containsKey(IS_PERF_FIELD))
                isPerf =  data[IS_PERF_FIELD] as Boolean

            val isNavigationActivated : Boolean
            if(data[IS_ACTIVATED_FIELD] != null)
                isNavigationActivated = data[IS_ACTIVATED_FIELD] as Boolean
            else
                isNavigationActivated = true // TODO CHANGE DEFAULT

            val isHoraire : Boolean
            if(data[IS_HORAIRE_FIELD] != null)
                isHoraire = data[IS_HORAIRE_FIELD] as Boolean
            else
                isHoraire = true

            val arrayPOI = ArrayList<POI>()
            val poisData = data["arrayPOI"] as ArrayList<*>?
            poisData!!.forEach {
                var poi = POI.fromFlutter(it as LinkedTreeMap<*, *>)
                arrayPOI.add(poi)
            }

            val arraySS = ArrayList<SportSession>()
            val ssData = data["arraySportsSession"] as ArrayList<*>?
            ssData!!.forEach {
                var ss = SportSession.fromFlutter(it as LinkedTreeMap<*, *>)
                arraySS.add(ss)
            }

            var route = RouteDataClass(id, allergensAverage, ascent, arrayCities, cityName, arrayCoutries, arrayDescription, easiness, gpxFileName,length, lengthCategory, arrayCoordinates, previewCoordinates, arrayNames, arrayPois, airQuality, arrayShareURL, status, dateEnd, idCategory, idCity, imageURL, isNavigationActivated, isHoraire, areSmartFactsEnabled, isStartingPointStatic, timeStamping, isLoop, isPerf)
            route.arrayPois = arrayPOI
            route.arraySportSession = arraySS
            return route

        }
    }
}

data class RouteCenter(
        val center: Location = Location("provider"),
        val radius: Long = 0L
){

}
