package com.mile.v4_navigation_engine.Navigation.utils

import android.location.Location
import com.google.gson.GsonBuilder
import com.mapbox.api.directions.v5.models.LegStep
import com.mapbox.api.directions.v5.models.RouteLeg
import com.mapbox.api.directions.v5.models.RouteOptions
import com.mapbox.api.matching.v5.models.MapMatchingMatching
import com.mapbox.api.matching.v5.models.MapMatchingResponse
import com.mapbox.api.matching.v5.models.MapMatchingTracepoint
import com.mapbox.geojson.Point
import com.mapbox.turf.TurfMeasurement
import com.mile.v4_navigation_engine.Navigation.model.OfflineDatas
import com.mile.v4_navigation_engine.utils.MapUtils
import com.yashovardhan99.timeit.Stopwatch
import org.json.JSONArray
import org.json.JSONObject
import retrofit2.Call

/**
Created by Corentin Houdayer on 2019-10-29
Dev profile @ https://github.com/houdayec
 **/

class OfflineUtils {

    companion object{
        
        fun matchingMapboxMapSaveRouteOptions(call: Call<MapMatchingResponse>, locomotion: String) : String {
            //get coordinate looks like "XX.XXXXX,XX.XXXX;XX.XXXXXX,XX.XXXXX" in String
            //And parse to [[(double)lon, (double)lat], [(double)lon, (double)lat]]
            var coordinateRaw = call.request().url().pathSegments().last();
            var listOfCoordinate = coordinateRaw.split(";")
            var coordinate: ArrayList<Any> = ArrayList()
            for (sCoordinate in listOfCoordinate) {
                var splitPoint = sCoordinate.split(",")
                var point = mutableMapOf("Latitude" to splitPoint[1].toDouble(), "Longitude" to splitPoint[0].toDouble())
                //var point = listOf(splitPoint[0], splitPoint[1])
                coordinate.add(point)
            }
            val optionsMap: MutableMap<String, Any?> = java.util.HashMap()
            optionsMap.put("coordinates", coordinate)
            optionsMap.put("baseUrl", "https://api.mapbox.com")
            optionsMap.put("user", "mapbox")
            optionsMap.put("profile", locomotion)
            optionsMap.put("uuid", "mapmatching")
            for (key: String in call.request().url().queryParameterNames()) {
                //check boolean value and parse string to bool
                if (call.request().url().queryParameter(key) == "true" || call.request().url().queryParameter(key) == "false") {
                    optionsMap.put(key, call.request().url().queryParameter(key)!!.toBoolean())
                } else {
                    optionsMap.put(key, call.request().url().queryParameter(key)!!)
                }
            }
            return JSONObject(optionsMap).toString()
        }
        
        fun matchingMapboxMapFromOfflineDatas(offlineDatas: OfflineDatas?, arrayCoordinates : List<Location>) : MapMatchingMatching {
            //Add RouteOption in mapbox request response
            var splitReponse = offlineDatas!!.requestResponseJson!!.split("\"matchings\":[{");
            var responseWithOptions = splitReponse[0]+"\"matchings\":[{\"routeOptions\":"+offlineDatas!!.routeOptions+","+splitReponse[1]
            val response: MapMatchingResponse = MapMatchingResponse.fromJson(responseWithOptions)

            //PROBLEM CAN'T PARSED COORDINATED FIELD SO
            //Transform [{"latitude": XX.XXXX, "longitude": X.XXXXXXXX}, {"latitude": XX.XXXX, "longitude": X.XXXXXXXX}, ...]
            //to "XX.XXXXX,XX.XXXX;XX.XXXXXX,XX.XXXXX;......"
            /*var obj: JSONObject = JSONObject(responseWithOptions)
            var matchings = obj.get("matchings") as JSONArray
            var matching = matchings[0]  as JSONObject
            var routeOptions = matching.get("routeOptions") as JSONObject*/

            var coordinates: String = formatCoordinates(arrayCoordinates)

            val modifiedMatchings: ArrayList<MapMatchingMatching> = java.util.ArrayList()
            for (matching in response.matchings()!!) {
                /*val modifiedLegs: ArrayList<RouteLeg> = java.util.ArrayList()
                for(routeLeg in matching.legs()) {
                    modifiedLegs.add(RouteLeg.builder()
                        .annotation(routeLeg.annotation())
                        .distance(routeLeg.distance())
                        .duration(routeLeg.duration())
                        .summary(routeLeg.summary())
                        .steps(modifyLegsStepFromNewClosestPoint(arrayCoordinates[0], routeLeg.steps()!!))
                        .build()
                    )
                }*/
                modifiedMatchings.add(matching.toBuilder()
                        //.legs(modifiedLegs)
                        .routeOptions(
                        RouteOptions.builder()
                                .profile(matching.routeOptions()!!.profile())
                                .coordinates(formatCoordinates(coordinates))
                                .annotations(matching.routeOptions()!!.annotations())
                                .approaches(matching.routeOptions()!!.approaches())
                                ?.language(matching.routeOptions()!!.language())
                                ?.radiuses(matching.routeOptions()!!.radiuses())
                                ?.user(matching.routeOptions()!!.user())
                                ?.voiceInstructions(matching.routeOptions()!!.voiceInstructions())
                                ?.bannerInstructions(matching.routeOptions()!!.bannerInstructions())
                                ?.roundaboutExits(matching.routeOptions()!!.roundaboutExits())
                                ?.geometries(matching.routeOptions()!!.geometries())
                                ?.overview(matching.routeOptions()!!.overview())
                                ?.steps(matching.routeOptions()!!.steps())
                                ?.voiceUnits(matching.routeOptions()!!.voiceUnits())
                                ?.requestUuid("mapmatching")
                                ?.accessToken(matching.routeOptions()!!.accessToken())
                                ?.approaches(matching.routeOptions()!!.approaches())
                                ?.waypointIndices(matching.routeOptions()!!.waypointIndices())
                                ?.waypointNames(matching.routeOptions()!!.waypointNames())
                                ?.baseUrl(matching.routeOptions()!!.baseUrl())
                                ?.build()
                ).build())
            }
            /*var newTracePoints = modifyTracepointsFromNewClosestPoint(arrayCoordinates[0], response.tracepoints()!!)
            response.tracepoints()!!.removeAll(response.tracepoints()!!)
            response.tracepoints()!!.addAll(newTracePoints)*/

            return modifiedMatchings[0]
        }

        fun formatCoordinates(coordinates: String): MutableList<Point> {
            val coordPairs = coordinates.split(";").dropLastWhile { it.isEmpty() }.toTypedArray()
            val coordinatesFormatted: MutableList<Point> = java.util.ArrayList()
            for (coordPair in coordPairs) {
                val coords = coordPair.split(",").dropLastWhile { it.isEmpty() }.toTypedArray()
                coordinatesFormatted.add(Point.fromLngLat(java.lang.Double.valueOf(coords[0]), java.lang.Double.valueOf(coords[1])))
            }
            return coordinatesFormatted
        }

        fun formatCoordinates(coordinates: List<Location>): String {
            var coor = ""
            for (location in coordinates) {
                coor += location.longitude.toString()+","+location.latitude.toString()+";"
            }
            return coor
        }

        fun modifyLegsStepFromNewClosestPoint(routeLocation : Location, arraySteps: List<LegStep>) : List<LegStep> {
            var closestPoint = getClosestLegStepFromRoute(routeLocation, arraySteps)
            var positionToCut = arraySteps.indexOf(closestPoint)
            arraySteps.forEachIndexed { index, legStep ->
                if(legStep.maneuver().location().latitude() == closestPoint.maneuver().location().latitude() && legStep.maneuver().location().longitude() == closestPoint.maneuver().location().longitude()){
                    positionToCut = index
                }
            }
            if(positionToCut != 0){
                var firstPart = arraySteps.subList(0, positionToCut-1)
                var secondPart = arraySteps.subList(positionToCut, arraySteps.size)
                val finalList = secondPart.toMutableList()
                finalList += firstPart

                var legStep = finalList.first()
                legStep.maneuver().toBuilder().type("depart").build()
                finalList.add(0,legStep)
                //finalList.first().maneuver().toBuilder().type("depart").build()
                //finalList.last().maneuver().toBuilder().type("arrive").build()
                return finalList
            }else{
                return arraySteps
            }
        }

        fun getClosestLegStepFromRoute(routeLocation: Location, arraySteps: List<LegStep>) : LegStep {
            var minDistance = 20000.0
            var routeLocationPoint = MapUtils.convertLocationToPoint(routeLocation)
            //var closestInit = arraySteps[0].maneuver().location()
            var closestLegStep = arraySteps[0]

            arraySteps.forEach { legStep ->
                var point = legStep.maneuver().location()
                var distance = TurfMeasurement.distance(routeLocationPoint, point)
                if(distance < minDistance){
                    closestLegStep = legStep
                    minDistance = distance
                }
            }
            return closestLegStep
        }

        fun modifyTracepointsFromNewClosestPoint(routeLocation : Location, arrayTracepoints: List<MapMatchingTracepoint>) : List<MapMatchingTracepoint> {
            var closestPoint = getClosestTracepointFromRoute(routeLocation, arrayTracepoints)
            var positionToCut = arrayTracepoints.indexOf(closestPoint)
            arrayTracepoints.forEachIndexed { index, tracepoint ->
                if(tracepoint != null) {
                    if(tracepoint.location()!!.latitude() == closestPoint.location()!!.latitude() && tracepoint.location()!!.longitude() == closestPoint.location()!!.longitude()){
                        positionToCut = index
                    }
                }
            }
            if(positionToCut != 0){
                var firstPart = arrayTracepoints.subList(0, positionToCut-1)
                var secondPart = arrayTracepoints.subList(positionToCut, arrayTracepoints.size)
                val finalList = secondPart.toMutableList()
                finalList += firstPart
                return finalList
            }else{
                return arrayTracepoints
            }
        }

        fun getClosestTracepointFromRoute(routeLocation: Location, arrayTracepoints: List<MapMatchingTracepoint>) : MapMatchingTracepoint {
            var minDistance = 20000.0
            var routeLocationPoint = MapUtils.convertLocationToPoint(routeLocation)
            //var closestInit = arraySteps[0].maneuver().location()
            var closestTracepoint = arrayTracepoints[0]

            arrayTracepoints.forEach { tracepoint ->
                if(tracepoint != null) {
                    var point = tracepoint.location()!!
                    var distance = TurfMeasurement.distance(routeLocationPoint, point)
                    if (distance < minDistance) {
                        closestTracepoint = tracepoint
                        minDistance = distance
                    }
                }
            }
            return closestTracepoint
        }

    }
}