//
//  MileNavigationEngineController.swift
//  android_intent
//
//  Created by Lionel Malloggi on 05/03/2020.
//

import Flutter
import UIKit
import Mapbox

class MileNavigationEngineController: NSObject, FlutterPlatformView, MGLMapViewDelegate, CLLocationManagerDelegate {
    private var navigationView: UIView?

    func view() -> UIView {
        return navigationView!
    }

    init(withFrame frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?, registrar: FlutterPluginRegistrar) {

        LocationService.shared.requestLocation()
        LocationService.shared.locationManager.requestLocation()

        MyStyle.shared.configurator()

        let arguments = args as? NSDictionary
        let oRoute = arguments?["route"] as? String ?? ""
        let oGpsColor = arguments?["gpsColor"] as? String ?? ""
        let oAccessToken = arguments?["accessToken"] as? String ?? ""
        let oMode = arguments?["mode"] as? String ?? NavigationMode.NAVIGATE_IN_ROUTE
        let oLanguages = arguments?["languages"] as? [String] ?? ["EN"]
        let oSponsor = arguments?["sponsor"] as? String ?? ""
        let oDistanceUnit = arguments?["distanceUnit"] as? String ?? ""
        let oSpeedUnit = arguments?["speedUnit"] as? String ?? ""
        let oLocomotion = arguments?["locomotion"] as? String ?? ""
        let oWhiteLabel = arguments?["whiteLabel"] as? String ?? ""
        let oOfflineDatas = arguments?["offlineDatas"] as? String ?? ""
        let oDownloadForOffline = arguments?["downloadForOffline"] as? Bool ?? false

        super.init()
        let dict = MileNavigationEngineController.convertToDictionary(text: oRoute.replacingOccurrences(of: "\n", with: "\\n"))
        let route = RouteInformation.init(snapshotValue: dict!, directionCriteria: oLocomotion)

        var pois: [PointOfInterest] = []
        if let poisData = dict!["arrayPOI"] as? [[String: Any]] {
            for poiData in poisData {
                let poi = PointOfInterest.init(snapshotValue: poiData)
                if(!poi.isDirectionnal) {
                    pois.append(poi)
                }
            }
            route.pois = pois
        }

        var directionnals: [PointOfInterest] = []
        if let poisData = dict!["arrayPOI"] as? [[String: Any]] {
            for poiData in poisData {
                let poi = PointOfInterest.init(snapshotValue: poiData)
                if(poi.isDirectionnal) {
                    directionnals.append(poi)
                }
            }
            route.directionals = directionnals
        }

        var sports: [SportSession] = []
        if let sportsData = dict!["arraySportsSession"] as? [[String: Any]] {
            for sportData in sportsData {
                let ss = SportSession.init(dict: sportData)
                sports.append(ss)
            }
            route.sessions = sports
        }

        let navigationRouter = MyNavigationRouter.init()
        let myNavigationView = navigationRouter.createRunnincityNavigationController(coordinatesRoute: route.coordinatesGPX, pois: pois, route: route, unitDistance: oDistanceUnit)
        myNavigationView.registrar = registrar //done
        myNavigationView.viewId = viewId //done
        myNavigationView.navigationMode = oMode //not use (yet)
        myNavigationView.gpsColor = oGpsColor 
        myNavigationView.languages = oLanguages //done
        //myNavigationView.distanceUnit = oDistanceUnit //init in NavigationRouter constructor
        //myNavigationView.speedUnit = oSpeedUnit //not use (yet ?), mph is set with distanceUnit for the time being
        //myNavigationView.locomotion = oLocomotion //init RouteInformation constructor
        myNavigationView.whiteLabel = WhiteLabel(jsonString: oWhiteLabel) //en cours
        myNavigationView.offlineDatas = OfflineDatas(jsonString: oOfflineDatas) //pas fait
        myNavigationView.downloadForOffline = oDownloadForOffline //pas fait
        navigationView = myNavigationView.view
    }

    static func convertToDictionary(text: String) -> [String: Any]? {
        if let data = text.data(using: .utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
            } catch {
                print(error.localizedDescription)
            }
        }
        return nil
    }
}
