//
//  RouteInformation.swift
//  NavigationUI
//
//  Created by Hassine on 17/06/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import Foundation
import CoreLocation
import UIKit


public class RouteInformation  {

    public var shareUrl: String?

    public var key: String

    public var imageURL: String?

    public var coordinatesGPX: [CLLocationCoordinate2D] = [CLLocationCoordinate2D]()

    public var coordinatesPolyline: [CLLocationCoordinate2D] = [CLLocationCoordinate2D]()

    public var isNavigationActivated: Bool

    public var cityId: String

    public var city: City?

    public var poisId: [String] = []


    public var sponsorsId: [String] = []

    public var sponsors :[Sponsor] = []

    public var pois :[PointOfInterest] = []

    public var directionals :[PointOfInterest] = []

    public var sessionsId: [String] = []

    public var sessions :[SportSession] = []

    public var averageDuration: Int?

    public var length: Int?

    public var distance: Int?

    public var ascent: Int?

    public var status: Int?

    public var isStartingPointStatic = false

    //public var profileIdentifier: Int = 0
    public var profileIdentifier: String = "walking"

    private var nameFr: String?
    public  var nameEn: String?
    private var nameEs: String?
    private var nameDe: String?
    private var nameIt: String?
    private var nameNl: String?
    private var namePt: String?

    private var descriptionFr : String?
    private var descriptionEn : String?
    private var descriptionIt : String?
    private var descriptionEs : String?
    private var descriptionNl : String?
    private var descriptionPt : String?
    private var descriptionDe : String?

    public var category : Category = .tourism


    public var description: String? {
        get {
            return MyUtils.translateToSelectedLanguage(en: descriptionEn, fr: descriptionFr, es: descriptionEs, it: descriptionIt, de: descriptionDe, nl: descriptionNl, pt: descriptionPt)
        }
    }

    public  var name: String? {
        get {
            return MyUtils.translateToSelectedLanguage(en: nameEn, fr: nameFr, es: nameEs, it: nameIt, de: nameDe, nl: nameNl, pt: namePt)
        }
    }


    init(snapshotValue: [String: Any], directionCriteria: String) {
        profileIdentifier = directionCriteria

        self.key = snapshotValue["id"] as? String ?? ""

        if let dictName = snapshotValue["Name"] as? NSDictionary
        {
            nameEn = dictName["EN"] as? String ?? nil
            nameFr = dictName["FR"] as? String ?? nil
            nameIt = dictName["IT"] as? String ?? nil
            nameEs = dictName["ES"] as? String ?? nil
            nameDe = dictName["DE"] as? String ?? nil
            nameNl = dictName["NL"] as? String ?? nil
            namePt = dictName["PT"] as? String ?? nil
        }

        if let dictDescription = snapshotValue["Description"] as? NSDictionary
        {
            descriptionEn = dictDescription["EN"] as? String ?? nil
            descriptionFr = dictDescription["FR"] as? String ?? nil
            descriptionDe = dictDescription["DE"] as? String ?? nil
            descriptionPt = dictDescription["PT"] as? String ?? nil
            descriptionEs = dictDescription["ES"] as? String ?? nil
            descriptionIt = dictDescription["IT"] as? String ?? nil
            descriptionNl = dictDescription["NL"] as? String ?? nil
        }


        /*if let sponsors = snapshotValue["Sponsors"] as? NSArray
        {
            for sponsor in sponsors
            {
                sponsorsId.append(sponsor as! String)
            }
        }*/



        if let sessions = snapshotValue["SS"] as? NSMutableDictionary
        {
            for session in sessions
            {
                sessionsId.append(session.key as! String)
            }
        }
        imageURL = snapshotValue["imageURL"] as? String
        isStartingPointStatic = snapshotValue["isStartingPointStatic"] as? Bool ?? false
        averageDuration = snapshotValue["averageDuration"] as? Int ?? nil
        length = snapshotValue["length"] as? Int ?? 0
        ascent = snapshotValue["ascent"] as? Int ?? 0
        isNavigationActivated = snapshotValue["isNavigationActivated"] as? Bool ?? true
        status = snapshotValue["Status"] as? Int ?? nil
        if let shareUrlDictionnary = snapshotValue["ShareUrl"] as? NSMutableDictionary
        {
            shareUrl =   shareUrlDictionnary["EN"] as? String
        }
        if let pois = snapshotValue["pois"] as? NSArray
        {
            for poi in pois
            {
                poisId.append(poi as! String)
            }
        }
        cityId = ""
        if let cities = snapshotValue["IdCity"] as? NSArray
        {
            cityId = cities.firstObject as! String


        }
        if let typeRoutes = snapshotValue["typeTransportation"] as? NSArray
        {
            let index = typeRoutes[0] as? Int ?? -1
            // typeRoute = TypeRoute(rawValue : index) ?? .none
        }

        if let coordinates = snapshotValue["arrayCoordinates"] as? [[String: Double]] {
            for coordinate in coordinates{
                let latitude = coordinate["latitude"] ?? 0
                let longitude = coordinate["longitude"] ?? 0
                let gpx = CLLocationCoordinate2D.init(latitude: latitude, longitude: longitude)
                coordinatesGPX.append(gpx)
            }
        }

        if let idCategory = snapshotValue["IdCategory"] as? Int
        {
            self.category = Category(rawValue: idCategory) ?? .tourism
        }
    }
}





public enum Category : Int {
    case tourism = 0
    case challenge = 1
    case sport = 2
    case theme = 3

    public var description: String {
        switch (self) {
        case .tourism: return "TOURISM_CATEGORY".localized
        case .sport:return "SPORT_CATEGORY".localized
        case .challenge:return "CHALLENGE_CATEGORY".localized
        case .theme:return "THEME_CATEGORY".localized


        }
    }


    public var color : UIColor
    {
        switch (self) {
        case .tourism:
            return UIColor(red: 0.97, green: 0.73, blue: 0.23, alpha: 1.00)
        case .challenge:
            return UIColor(red: 0.97, green: 0.23, blue: 0.41, alpha: 1.00)
        case .sport:
            return UIColor(red: 0.20, green: 0.72, blue: 0.88, alpha: 1.00)
        case .theme:
            return UIColor(red: 0.77, green: 0.25, blue: 0.85, alpha: 1.00)
        }
    }

}


/*public class RouteInformationService  {

    var db : Firestore!
    public init() {
        if FirebaseApp.app() == nil
        {
            FirebaseApp.configure()
        }
        db = Firestore.firestore()
        //      FirebaseApp.configure()
        //   Database.database().isPersistenceEnabled = true

        let settings = FirestoreSettings()
        settings.isPersistenceEnabled = true
        Firestore.firestore().settings = settings

    }




    public func getRouteFireStore(by ids:[String],_ callBack:@escaping ([RouteInformation]) -> Void) {


        let settings = FirestoreSettings()
        settings.isPersistenceEnabled = true
        var routes = [RouteInformation]()
        Firestore.firestore().settings = settings
        // [END setup]
        var db = Firestore.firestore()

        var indexRou = 0

        for id in ids
        {
            db.collection("Route").document(id).getDocument(completion: { (document, error) in

                if error != nil
                {
                    indexRou += 1
                    if ids.count == indexRou
                    {
                        callBack(routes)
                    }
                    return
                }
                if let snapshot = document?.data()
                {
                    indexRou += 1
                    let route = RouteInformation.init(snapshotValue: snapshot, key: document!.documentID)
                    print(indexRou)
                    routes.append(route)
                    if ids.count == indexRou
                    {
                        callBack(routes)
                    }
                }
                else
                {
                    indexRou += 1
                    print(indexRou)
                    if ids.count == indexRou
                    {

                        callBack(routes)
                    }
                }
            })
        }

    }

    public func getRoutesFireStore(by cityId:[String],routeIDs:[String],_ callBack:@escaping (Result<[RouteInformation]>) -> Void) {
        let start = CFAbsoluteTimeGetCurrent()
        // db.collection("RouteTest").whereField("City", in:cityId)
        var cityId = cityId

        if cityId.count > 10
        {

            while cityId.count > 10 {
                cityId.removeLast()
            }
        }
        db.collection("Route").whereField("IdCity", arrayContainsAny: cityId)

            .getDocuments() { (snapshot, err) in

                let diff = CFAbsoluteTimeGetCurrent() - start
                print("Time = Took 1 routes \(diff) seconds")
                if let err = err {
                    print("Error getting documents: \(err)")
                } else {
                    var routes : [RouteInformation] = []

                    for document in snapshot!.documents {
                        let route = RouteInformation(snapshotValue: document.data(),key:document.documentID )
                        if route.status == 4{
                            routes.append(route)
                        }



                        let diff = CFAbsoluteTimeGetCurrent() - start
                    }
                    print("Time = 2 Took routes \(diff) seconds")
                    var res = routes.filter { (r) -> Bool in
                        return routeIDs.contains(r.key)
                    }
                    callBack(Result.success(res))

                }

            }

    }




    public func getRoutesFireStore(routeIDs:[String],_ callBack:@escaping (Result<[RouteInformation]>) -> Void) {
        let start = CFAbsoluteTimeGetCurrent()
        // db.collection("RouteTest").whereField("City", in:cityId)


        var count = 0
        var routes : [RouteInformation] = []
        for id in routeIDs
        {
            db.collection("Route").document(id).getDocument { (snapshot, err) in

                if let err = err {
                    print("Error getting documents: \(err)")
                } else {

                    let route = RouteInformation(snapshotValue: snapshot!.data()!,key:snapshot!.documentID )
                    if route.status == 4
                    {
                        routes.append(route)
                    }

                }

                count += 1
                if count == routeIDs.count
                {
                    callBack(Result.success(routes))
                }

            }
        }
    }


    public func setReview(data: [String : Any]) -> Void {
        db.collection("Review").addDocument(data: data)
    }


    public func getAllRoutesFireStore(_ callBack:@escaping (Result<[RouteInformation]>) -> Void) {
        let start = CFAbsoluteTimeGetCurrent()

        db.collection("Route")
            .getDocuments() { (snapshot, err) in
                let diff = CFAbsoluteTimeGetCurrent() - start
                print("Time = Took routes all \(diff) seconds")
                if let err = err {
                    print("Error getting documents: \(err)")
                } else {
                    var routes : [RouteInformation] = []
                    DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()) {
                        for document in snapshot!.documents {
                            let route = RouteInformation(snapshotValue: document.data(),key:document.documentID )
                            if route.status == 4
                            {
                                routes.append(route)
                            }

                        }
                        callBack(Result.success(routes))

                    }

                }
            }
    }




    public func getAllFirtsRoutesFireStore(_ callBack:@escaping (Result<[RouteInformation]>) -> Void) {
        let start = CFAbsoluteTimeGetCurrent()

        db.collection("Route").limit(to: 20)
            .getDocuments() { (snapshot, err) in
                let diff = CFAbsoluteTimeGetCurrent() - start
                print("Time = Took routes all \(diff) seconds")
                if let err = err {
                    print("Error getting documents: \(err)")
                } else {
                    var routes : [RouteInformation] = []
                    DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()) {
                        for document in snapshot!.documents {
                            let route = RouteInformation(snapshotValue: document.data(),key:document.documentID )
                            if route.status == 4
                            {
                                routes.append(route)
                            }

                        }
                        callBack(Result.success(routes))

                    }

                }
            }
    }
}


*/


public enum Result<T> {
    case success(T)
    case failure(Error)
}
