//
//  Sponsor.swift
//  NavigationUI
//
//  Created by Hassine on 17/06/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import Foundation
import CoreLocation



public struct Sponsor {
    
    public var key: String
    public var urlWeb: String?
    public var coordinates: [CLLocationCoordinate2D] = []
    public var logoURL: String?
    public var imageURL: String?
    public var idAnnotation: Array<Int32>?
    
    
    public var description: String? {
        get {
            return MyUtils.translateToSelectedLanguage(en: descriptionEn, fr: descriptionFr, es: descriptionEs, it: descriptionIt, de: descriptionDe, nl: descriptionNl, pt: descriptionPt)
        }
    }
    
    public  var name: String? {
        get {
            return MyUtils.translateToSelectedLanguage(en: nameEn, fr: nameFr, es: nameEs, it: nameIt, de: nameDe, nl: nameNl, pt: namePt)
        }
    }
    
    public  var vocal: String? {
        get {
            return MyUtils.translateToSelectedLanguage(en: vocalEn, fr: vocalFr, es: vocalEs, it: vocalIt, de: vocalDe, nl: vocalNl, pt: vocalPt)
        }
    }
    
    public  var websiteURL: String? {
        get {
            return MyUtils.translateToSelectedLanguage(en: websiteURLEn, fr: websiteURLFr, es: websiteURLEs, it: websiteURLIt, de: websiteURLDe, nl: websiteURLNl, pt: websiteURLPt)
        }
    }
    
    
    private var nameFr: String?
    private var nameEn: String?
    private var nameEs: String?
    private var nameDe: String?
    private var nameIt: String?
    private var nameNl: String?
    private var namePt: String?
    
    private var websiteURLFr: String?
    private var websiteURLEn: String?
    private var websiteURLEs: String?
    private var websiteURLDe: String?
    private var websiteURLIt: String?
    private var websiteURLNl: String?
    private var websiteURLPt: String?
    
    private var descriptionFr : String?
    private var descriptionEn : String?
    private var descriptionIt : String?
    private var descriptionEs : String?
    private var descriptionNl : String?
    private var descriptionPt : String?
    private var descriptionDe : String?
    
    private var vocalFr : String?
    private var vocalEn : String?
    private var vocalIt : String?
    private var vocalEs : String?
    private var vocalNl : String?
    private var vocalPt : String?
    private var vocalDe : String?
    

    
    
    
    
    init(snapshotValue: [String:Any], key:String) {
        self.key = key
        
        if let dictName = snapshotValue["Name"] as? NSDictionary
        {
            nameEn = dictName["EN"] as? String ?? nil
            nameFr = dictName["FR"] as? String ?? nil
            nameIt = dictName["IT"] as? String ?? nil
            nameEs = dictName["ES"] as? String ?? nil
            nameDe = dictName["DE"] as? String ?? nil
            nameNl = dictName["NL"] as? String ?? nil
            namePt = dictName["PT"] as? String ?? nil
        }
        if let dictDescription = snapshotValue["Description"] as? NSDictionary
        {
            descriptionEn = dictDescription["EN"] as? String ?? nil
            descriptionFr = dictDescription["FR"] as? String ?? nil
            descriptionDe = dictDescription["DE"] as? String ?? nil
            descriptionPt = dictDescription["PT"] as? String ?? nil
            descriptionEs = dictDescription["ES"] as? String ?? nil
            descriptionIt = dictDescription["IT"] as? String ?? nil
            descriptionNl = dictDescription["NL"] as? String ?? nil
        }
        if let dictName = snapshotValue["Link"] as? NSDictionary
        {
            websiteURLEn = dictName["EN"] as? String ?? nil
            websiteURLFr = dictName["FR"] as? String ?? nil
            websiteURLIt = dictName["IT"] as? String ?? nil
            websiteURLEs = dictName["ES"] as? String ?? nil
            websiteURLDe = dictName["DE"] as? String ?? nil
            websiteURLNl = dictName["NL"] as? String ?? nil
            websiteURLPt = dictName["PT"] as? String ?? nil
        }
        
        logoURL = snapshotValue["LogoUrl"] as? String
        if let dictVocale = snapshotValue["Vocal"] as? NSDictionary
        {
            vocalEn = dictVocale["EN"] as? String ?? nil
            vocalFr = dictVocale["FR"] as? String ?? nil
            vocalIt = dictVocale["IT"] as? String ?? nil
            vocalEs = dictVocale["ES"] as? String ?? nil
            vocalDe = dictVocale["DE"] as? String ?? nil
            vocalNl = dictVocale["NL"] as? String ?? nil
            vocalPt = dictVocale["PT"] as? String ?? nil
        }
        imageURL = snapshotValue["ImageUrl"] as? String
        if let coordinates = snapshotValue["GPS"] as? NSMutableArray
        {
            
            for coordinate in coordinates
            {
                if let dict = coordinate as? NSMutableDictionary
                {
                    let latitude = dict.value(forKey: "Latitude") as? Double ?? 0
                    let longitude = dict.value(forKey: "Longitude") as? Double ?? 0
                    let gpx = CLLocationCoordinate2D.init(latitude: latitude, longitude: longitude)
                    self.coordinates.append(gpx)
                }
            }
        }
    }
}



