//
//  HealthManager.swift
//  NavigationUI
//
//  Created by hassine othmane on 5/14/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import Foundation
import HealthKit


class HealthManager {

    let healthKitStore:HKHealthStore = HKHealthStore()


func authorizeHealthKitOnStatVC(completion: ((_ success:Bool, _ error:NSError?) -> Void)!)
{

    // 2. Set the types you want to write to HK Store
    let healthKitTypesToWrite: Set<HKSampleType> = [
        HKObjectType.quantityType(forIdentifier: HKQuantityTypeIdentifier.activeEnergyBurned)!,
        HKObjectType.quantityType(forIdentifier: HKQuantityTypeIdentifier.distanceWalkingRunning)!,
        HKQuantityType.workoutType()
        ]

    // 3. If the store is not available (for instance, iPad) return an error and don't go on.
    if !HKHealthStore.isHealthDataAvailable()
    {
        let error = NSError(domain: "com.raywenderlich.tutorials.healthkit", code: 2, userInfo: [NSLocalizedDescriptionKey:"HealthKit is not available in this Device"])
        if( completion != nil )
        {
            NSLog(error.debugDescription)
        }
        return;
    }

    // 4.  Request HealthKit authorization
    healthKitStore.requestAuthorization(toShare: healthKitTypesToWrite, read: nil) { (success, error) -> Void in
        if success {
            print("success") //This means the view was presented to the user, not that he accepted all categories.
        completion(true,nil)
        } else {
            print("failure")
        completion(false,nil)
        }


        if let error = error {
            print(error.localizedDescription)
            completion(true,error as NSError)
        }
    }

}


    func authorizeHealthKitOnSignUpVC(completion: ((_ success:Bool, _ error:NSError?) -> Void)!)
    {
        // 1. Set the types you want to read from HK Store
        let healthKitTypesToRead: Set<HKObjectType> = [

            //     HKObjectType.characteristicType(forIdentifier: HKCharacteristicTypeIdentifier.dateOfBirth)!,
            HKObjectType.characteristicType(forIdentifier: HKCharacteristicTypeIdentifier.biologicalSex)!,
            HKObjectType.quantityType(forIdentifier: HKQuantityTypeIdentifier.bodyMass)!,
            HKObjectType.quantityType(forIdentifier: HKQuantityTypeIdentifier.height)!
        ]

        // 2. Set the types you want to write to HK Store
        let healthKitTypesToWrite: Set<HKSampleType> = [
            HKObjectType.quantityType(forIdentifier: HKQuantityTypeIdentifier.activeEnergyBurned)!,
            HKObjectType.quantityType(forIdentifier: HKQuantityTypeIdentifier.distanceWalkingRunning)!,
            HKQuantityType.workoutType()
        ]

        // 3. If the store is not available (for instance, iPad) return an error and don't go on.
        if !HKHealthStore.isHealthDataAvailable()
        {
            let error = NSError(domain: "com.raywenderlich.tutorials.healthkit", code: 2, userInfo: [NSLocalizedDescriptionKey:"HealthKit is not available in this Device"])
            if( completion != nil )
            {
                NSLog(error.debugDescription)
            }
            return;
        }

        // 4.  Request HealthKit authorization
        healthKitStore.requestAuthorization(toShare: healthKitTypesToWrite, read: healthKitTypesToRead) { (success, error) -> Void in
            if success {
                print("success") //This means the view was presented to the user, not that he accepted all categories.
                completion(true,nil)
            } else {
                print("failure")
                completion(false,nil)
            }


            if let error = error {
                print(error.localizedDescription)
                completion(true,error as NSError)
            }
        }

    }


    func readProfileSex() -> HKBiologicalSexObject?
    {

        var biologicalSex :HKBiologicalSexObject?


        do {
        // 2. Read biological sex
        biologicalSex = try healthKitStore.biologicalSex()
        }catch {
            print("Error reading Biological Sex: \(error)")
        }


        // 4. Return the information read in a tuple
        return biologicalSex
    }


      //MARK: Read Data

    private func readMostRecentSample(sampleType:HKSampleType , completion: ((HKSample?, Error?) -> Void)!)
    {

        // 1. Build the Predicate
        let past = Date.distantPast
        let now   = Date()
        let mostRecentPredicate = HKQuery.predicateForSamples(withStart: past, end:now, options: [])

        // 2. Build the sort descriptor to return the samples in descending order
        let sortDescriptor = NSSortDescriptor(key:HKSampleSortIdentifierStartDate, ascending: false)
        // 3. we want to limit the number of samples returned by the query to just 1 (the most recent)
        let limit = 1

        // 4. Build samples query
        let sampleQuery = HKSampleQuery(sampleType: sampleType, predicate: mostRecentPredicate, limit: limit, sortDescriptors: [sortDescriptor])
        { (sampleQuery, results, error ) -> Void in

            if error != nil {
                completion?(nil,error)
                return
            }


            // Get the first sample
            let mostRecentSample = results?.first as? HKQuantitySample

            // Execute the completion closure
            if completion != nil {
                completion(mostRecentSample,nil)
            }
        }
        // 5. Execute the Query
        self.healthKitStore.execute(sampleQuery)
    }




    func updateHeight(completionHandler: @escaping (_ heightString: String) -> Void){
        // 1. Construct an HKSampleType for Height
        let sampleType = HKSampleType.quantityType(forIdentifier: HKQuantityTypeIdentifier.height)
        var heightString = ""
        // 2. Call the method to read the most recent Height sample
        readMostRecentSample(sampleType: sampleType!, completion: { (mostRecentHeight, error) -> Void in

            if( error != nil )
            {
                print("Error reading height from HealthKit Store: \(error?.localizedDescription)")
                return;
            }


            let heightInMeter = mostRecentHeight as? HKQuantitySample;
            // 3. Format the height to display it on the screen

            if let meters = heightInMeter?.quantity.doubleValue(for: HKUnit.meterUnit(with: HKMetricPrefix.centi)) {
                heightString = String(Int(meters)) //convert m in cm
                /*let heightFormatter = NSLengthFormatter()
                heightFormatter.forPersonHeightUse = true;
                heightLocalizedString = heightFormatter.stringFromMeters(meters);*/
            }


            // 4. Update UI. HealthKit use an internal queue. We make sure that we interact with the UI in the main thread
            DispatchQueue.main.async(execute: { () -> Void in
                completionHandler((heightString))
            });
        })
    }


    func updateWeight(completionHandler: @escaping (_ WeightString: String) -> Void){
        // 1. Construct an HKSampleType for Height
        let sampleType = HKSampleType.quantityType(forIdentifier: HKQuantityTypeIdentifier.bodyMass)
        var weightString = ""
        // 2. Call the method to read the most recent Height sample
        readMostRecentSample(sampleType: sampleType!, completion: { (mostRecentWeight, error) -> Void in

            if( error != nil )
            {
                print("Error reading height from HealthKit Store: \(error?.localizedDescription)")
                return;
            }

            let weightInMeter = mostRecentWeight as? HKQuantitySample;
            // 3. Format the height to display it on the screen

            if let kilo = weightInMeter?.quantity.doubleValue(for: HKUnit.gramUnit(with: .kilo))
            {
               weightString = String(Int(kilo))
                /*let heightFormatter = NSLengthFormatter()
                 heightFormatter.forPersonHeightUse = true;
                 heightLocalizedString = heightFormatter.stringFromMeters(meters);*/
            }


            // 4. Update UI. HealthKit use an internal queue. We make sure that we interact with the UI in the main thread
            DispatchQueue.main.async(execute: { () -> Void in
                completionHandler(weightString)
            });
        })
    }



    //MARK: Write Workout

    func saveRunningWorkout(startDate:Date , endDate:Date , distanceinMeter:Double, kiloCalories:Double,
                            completion: ( (Bool, Error?) -> Void)!) {

        // 1. Create quantities for the distance and energy burned
        let distanceQuantity = HKQuantity(unit: HKUnit.meter(), doubleValue: distanceinMeter)
        let caloriesQuantity = HKQuantity(unit: HKUnit.kilocalorie(), doubleValue: kiloCalories)

        // 2. Save Running Workout
        let workout = HKWorkout(activityType: HKWorkoutActivityType.running, start: startDate, end: endDate, duration: abs(endDate.timeIntervalSince(startDate)), totalEnergyBurned: caloriesQuantity, totalDistance: distanceQuantity, metadata: nil)
        healthKitStore.save(workout, withCompletion: { (success, error) -> Void in
            if( error != nil  ) {
                // Error saving the workout
                completion(success,error)
            }
            else {
                // Workout saved
                completion(success,nil)

            }
        })
    }

}

