//
//  MyNavigationPresenter.swift
//  NavigationUI
//
//  Created Hassine on 10/03/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import UIKit
import CoreLocation
import MapboxNavigation
import MapboxCoreNavigation
import MapboxDirections



/// MyNavigation Module Presenter
class MyNavigationPresenter {
    
    weak private var _view: MyNavigationViewProtocol?
    private var interactor: MyNavigationInteractorProtocol
    private var wireframe: MyNavigationRouterProtocol
    
    init(view: MyNavigationViewProtocol) {
        self._view = view
        self.interactor = MyNavigationInteractor()
        self.wireframe = MyNavigationRouter()
    }
}

// MARK: - extending MyNavigationPresenter to implement it's protocol
extension MyNavigationPresenter: MyNavigationPresenterProtocol {
    
    
    
    
    func changeBeginningOfSmartRunWithnearestPoint( _ myTablePointGPX: [Waypoint], positionGPS: CLLocation!) -> [Waypoint]{
        guard positionGPS != nil else {
            return myTablePointGPX
        }
        let nearestPointGPX = findNearestGPXPoint(myTablePointGPX, positionGPS: positionGPS)
        var newTablePointGPX: [Waypoint] = []
        for j in nearestPointGPX ..< myTablePointGPX.count {
            newTablePointGPX.append(myTablePointGPX[j])
        }
        for k in 0 ..< nearestPointGPX {
            newTablePointGPX.append(myTablePointGPX[k])
        }
        return newTablePointGPX
    }
    
    
    func findNearestGPXPoint(_ myTablePointGPX: [Waypoint], positionGPS: CLLocation) -> Int{
        var nearestPointGPX:    Int                 = 0
        var deltaComparaison:   CLLocationDistance  = positionGPS.distance(from: CLLocation.init(latitude: myTablePointGPX[0].coordinate.latitude, longitude: myTablePointGPX[0].coordinate.longitude) )
        for i in 0 ..< myTablePointGPX.count {
            let delta: CLLocationDistance = positionGPS.distance(from:CLLocation.init(latitude: myTablePointGPX[i].coordinate.latitude, longitude: myTablePointGPX[i].coordinate.longitude))
            if delta < deltaComparaison{
                deltaComparaison    = delta
                nearestPointGPX     = i
            }
        }
        return nearestPointGPX
    }
    
    
    
    
    
    
    func calculateRouteOffline(points: [CLLocationCoordinate2D], locale: String, measurementSystem: MeasurementSystem, objectFor view: MyNavigationViewProtocol, profileIdentifier: MBDirectionsProfileIdentifier?, firstCalculation: Bool, id: String) {
        
        
        let profileIdentifier = profileIdentifier ?? MBDirectionsProfileIdentifier.walking
        var waypoints = [Waypoint]()
        for point in points
        {
            let waypoint = Waypoint.init(coordinate: point)
            waypoints.append(waypoint)
        }
        
        
        waypoints = changeBeginningOfSmartRunWithnearestPoint(waypoints, positionGPS:LocationService.shared.getCurrentLocation()!)
        //  waypoints = changeBeginningOfSmartRunWithnearestPoint


        let matchOptions = NavigationMatchOptions(waypoints: waypoints, profileIdentifier: profileIdentifier)
        matchOptions.includesSteps = true


        //matchOptions.waypointIndices = IndexSet([0, 1, waypoints.count - 1])
        let middle = waypoints.count / 2
        var index = 0
        matchOptions.waypoints.forEach { (waypoint) in

            if index  == middle
            {
                waypoint.separatesLegs = true
            }
            else
            {
                waypoint.separatesLegs = true
            }
            index += 1


        }

        //  matchOptions.distanceMeasurementSystem = MeasurementSystem.init(rawValue: MyUtils.getDistanceMeasurementSystem())!

        matchOptions.locale = Locale(identifier: "it-IT")
        //   matchOptions.locale.usesMetric = true
        matchOptions.distanceMeasurementSystem = measurementSystem
        
        
        
        
        let decoder1 = JSONDecoder()
        decoder1.userInfo = [.options: matchOptions, .credentials: Directions.shared.credentials]
        
        
        
        /*      if let path = Bundle.main.path(forResource: "test", ofType: "json") {
         do {
         let a = try Data(contentsOf: URL(fileURLWithPath: path), options: .mappedIfSafe)
         //let jsonResult = try JSONSerialization.jsonObject(with: data, options: .mutableLeaves)
         //if let jsonResult = jsonResult as? Dictionary<String, AnyObject>, let person = jsonResult["person"] as? [Any] {
         // do stuff
         // }




         var result = try decoder1.decode(MapMatchingResponse.self, from: a)

         let response = try RouteResponse(matching: result, options: matchOptions, credentials:  Directions.shared.credentials)


         let json = try JSONSerialization.jsonObject(with: a, options: []) as? [String : Any]




         // let response = try decoder1.decode(RouteResponse.self, from: a!)
         //print(response)


         guard var route = response.routes?.first else {
         return
         }



         print(response.waypoints )
         if firstCalculation
         {
         view.showOfflineRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
         }
         else
         {
         view.updateAndStartOfflineRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
         }




         } catch {
         // handle error
         }
         }


         */
        
        

        do{
            var a = UserDefaults.standard.data(forKey: id)






            let result = try decoder1.decode(MapMatchingResponse.self, from: a!)

            let response = try RouteResponse(matching: result, options: matchOptions, credentials:  Directions.shared.credentials)


            let json = try JSONSerialization.jsonObject(with: a!, options: []) as? [String : Any]

            print(json!)
            


            // let response = try decoder1.decode(RouteResponse.self, from: a!)
            //print(response)


            guard var route = response.routes?.first else {
                return
            }



            print(response.waypoints )
            if firstCalculation
            {
                if route.legs.first?.steps.last?.maneuverType == ManeuverType.arrive
                {
                    route.legs.first?.steps.removeLast()

                }
                
                view.showOfflineRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
            }
            else
            {
                if route.legs.first?.steps.last?.maneuverType == ManeuverType.arrive
                {
                    route.legs.first?.steps.removeLast()

                }

                view.updateAndStartOfflineRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
            }
        }
        catch
        {

        }
    }
    
    

    
    func projectOnNewReference(_ previousLocation: CLLocation, _ nextLocation: CLLocation, _ poiLocation: CLLocation) -> ProjectionUtils {
        //All coordinates from CLLocation are written with reference 0
        //All coordinates projected in OA vector abscisse are written with reference 1
        
        var xO0 = previousLocation.coordinate.latitude
        var yO0 = previousLocation.coordinate.longitude
        var xA0 = nextLocation.coordinate.latitude
        var yA0 = nextLocation.coordinate.longitude
        var xB0 = poiLocation.coordinate.latitude
        var yB0 = poiLocation.coordinate.longitude
        
        
        //On déplace le repère
        xA0 = xA0-xO0
        yA0 = yA0-yO0
        xB0 = xB0-xO0
        yB0 = yB0-yO0
        xO0 = 0
        yO0 = 0
        
        
        let OA = sqrt(pow(Double(xA0),2) + pow(Double(yA0),2))
        let cosAlpha  = xA0 / OA
        let sinAlpha  = yA0 / OA
        let tanAlpha    = sinAlpha/cosAlpha
        
        // let Alpha = atan(tanAlpha)
        
        //Le changement de repère donne y1= cos(Alpha)y0 - sin(Alpha)x0
        //Le changement de repère donne x1= cos(Alpha)x0 + sin(Alpha)y0
        
        let xA1 =  cosAlpha*xA0 + sinAlpha*yA0
        // let yA1 = cosAlpha*yA0 - sinAlpha*xA0
        let xB1 = cosAlpha*xB0 + sinAlpha*yB0
        let yB1 = (-sinAlpha*xB0) + (cosAlpha*yB0)
        
        var isOnPlot = false
        if abs(xB1) < abs(xA1) {
            //xB is either in OA or before O.
            
            if xB1 > xO0 {
                isOnPlot = true
            }else{
                isOnPlot = false
            }
        }
        
        let projection = ProjectionUtils.init()
        projection.isOnPlot = isOnPlot
        
        //Projection of B onto OA. In 0 reference
        let xP0 = cosAlpha*xB1 + (previousLocation.coordinate.latitude)
        let yP0 = sinAlpha*xB1 + (previousLocation.coordinate.longitude)
        
        let coordinates = CLLocation(latitude: xP0, longitude: yP0)
        let parentCoordinatesInNewReference = CLLocation(latitude: xB1, longitude: yB1)
        
        projection.coordinates = coordinates
        projection.parentCoordinatesInNewReference = parentCoordinatesInNewReference
        
        
        return projection
    }
    
    
    func calculateCoordinates(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) -> [CLLocationCoordinate2D] {
        let degreesBetweenPoints = 8.0
        //45 sides
        let numberOfPoints = floor(360.0 / degreesBetweenPoints)
        let distRadians: Double = withMeterRadius / 6371000.0
        // earth radius in meters
        let centerLatRadians: Double = coordinate.latitude * Double.pi / 180
        let centerLonRadians: Double = coordinate.longitude * Double.pi / 180
        var coordinates = [CLLocationCoordinate2D]()
        //array to hold all the points
        for index in 0 ..< Int(numberOfPoints) {
            let degrees: Double = Double(index) * Double(degreesBetweenPoints)
            let degreeRadians: Double = degrees * Double.pi / 180
            let pointLatRadians: Double = asin(sin(centerLatRadians) * cos(distRadians) + cos(centerLatRadians) * sin(distRadians) * cos(degreeRadians))
            let pointLonRadians: Double = centerLonRadians + atan2(sin(degreeRadians) * sin(distRadians) * cos(centerLatRadians), cos(distRadians) - sin(centerLatRadians) * sin(pointLatRadians))
            let pointLat: Double = pointLatRadians * 180 / Double.pi
            let pointLon: Double = pointLonRadians * 180 / Double.pi
            let point: CLLocationCoordinate2D = CLLocationCoordinate2DMake(pointLat, pointLon)
            coordinates.append(point)
        }
        return coordinates
    }
    
    func polygonCircleForCoordinate(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) -> MGLPolygon{
        var coordinates = calculateCoordinates(coordinate: coordinate, withMeterRadius: withMeterRadius) 
        return MGLPolygon(coordinates: &coordinates, count: UInt(coordinates.count))
    }
    
    

    

    
    
    
    func initLocationManager(delegate:MyNavigationView) {
        LocationService.shared.locationManager.delegate             = delegate
        LocationService.shared.locationManager.requestAlwaysAuthorization()
        LocationService.shared.locationManager.activityType         = .fitness
        LocationService.shared.locationManager.desiredAccuracy      = 10
        LocationService.shared.locationManager.distanceFilter = kCLDistanceFilterNone
        LocationService.shared.locationManager.startUpdatingLocation()
        LocationService.shared.locationManager.startUpdatingHeading()
        LocationService.shared.locationManager.allowsBackgroundLocationUpdates = true
        LocationService.shared.locationManager.startMonitoringSignificantLocationChanges()
    }
    
    
    func checkBestLanguage(pois:[PointOfInterest]?) -> (LanguageRessource,String) {
        if  let arraySaved = UserDefaults.standard.array(forKey: "languagePreferredOrderV4") as! Array<String>? {
            var poiTranslated = 0
            for language in arraySaved
            {
                if let pois = pois
                {
                    if pois.count == 0
                    {
                        return(LanguageRessource.init(languageSelected: language), MyUtils.getLanguageAndCountry(language: language))
                        
                    }
                    for poi in pois
                    {
                        if let desc = poi.getDescription(language: language)
                        {
                            poiTranslated += 1
                        }
                    }
                    let percent = (poiTranslated / pois.count * 100)
                    if percent >= 70 {
                        return(LanguageRessource.init(languageSelected: language), MyUtils.getLanguageAndCountry(language: language))
                    }
                }
                else
                {
                    return(LanguageRessource.init(languageSelected: language), MyUtils.getLanguageAndCountry(language: language))
                }
                
            }
            
            return(LanguageRessource.init(languageSelected: "EN"), MyUtils.getLanguageAndCountry(language: "EN"))
        }
        else
        {
            
            var poiTranslated = 0
            for language in [Locale.current.languageCode?.uppercased() ?? "EN","EN","FR"]
            {
                if let pois = pois
                {
                    if pois.count == 0
                    {
                        return(LanguageRessource.init(languageSelected: language), MyUtils.getLanguageAndCountry(language: language))
                        
                    }
                    for poi in pois
                    {
                        if let desc = poi.getDescription(language: language)
                        {
                            poiTranslated += 1
                        }
                    }
                    let percent = (poiTranslated / pois.count * 100)
                    if percent >= 70 {
                        
                        return(LanguageRessource.init(languageSelected: language), MyUtils.getLanguageAndCountry(language: language))
                    }
                }
                else
                {
                    return(LanguageRessource.init(languageSelected: language), MyUtils.getLanguageAndCountry(language: language))
                }
                
            }
            return(LanguageRessource.init(languageSelected: "EN"), MyUtils.getLanguageAndCountry(language: "EN"))
        }
    }
    
    /// init Measurement Unit
    func initMeasurementSystem(measurementSystem:MeasurementSystem) -> Void
    {
        if measurementSystem == .imperial
        {
            NavigationSettings.shared.distanceUnit                  = .mile
        }
        else if measurementSystem == .metric
        {
            NavigationSettings.shared.distanceUnit                  = .kilometer
        }
    }
    
    
    
    func calculateRoute(points: [CLLocationCoordinate2D], locale:String,measurementSystem: MeasurementSystem,objectFor view: MyNavigationViewProtocol, profileIdentifier:MBDirectionsProfileIdentifier?,firstCalculation:Bool) {
        print(points)
        let profileIdentifier = profileIdentifier ?? MBDirectionsProfileIdentifier.walking
        var waypoints = [Waypoint]()
        for point in points
        {
            let waypoint = Waypoint.init(coordinate: point)
            waypoints.append(waypoint)
        }
        
        let matchOptions = NavigationMatchOptions(waypoints: waypoints, profileIdentifier: profileIdentifier)
        matchOptions.includesSteps = true
        matchOptions.distanceMeasurementSystem = measurementSystem

        //matchOptions.waypointIndices = IndexSet([0, 1, waypoints.count - 1])
        let middle = waypoints.count / 2
        var index = 0
        matchOptions.waypoints.forEach { (waypoint) in
            
            if index  == middle - 1
            {
                waypoint.separatesLegs = true
            }
            else
            {
                waypoint.separatesLegs = false
            }
            index += 1
            
            
        }
        
        //  matchOptions.distanceMeasurementSystem = MeasurementSystem.init(rawValue: MyUtils.getDistanceMeasurementSystem())!
        
        matchOptions.locale = Locale(identifier: locale)
        //   matchOptions.locale.usesMetric = true
        matchOptions.distanceMeasurementSystem = measurementSystem
        
        //RouteCompletionHandler = (_ session: Session, _ result: Result<RouteResponse, DirectionsError>) -> Void
        Directions.shared.calculateRoutes(matching: matchOptions) { (session, result) in

            switch result {
            case .failure(let error):
                print(error.localizedDescription)
            case .success(let response):

                guard var route = response.routes?.first else {
                    return
                }

                if firstCalculation
                {
                    if route.legs.first?.steps.last?.maneuverType == ManeuverType.arrive
                    {
                        route.legs.first?.steps.removeLast()

                    }






                    view.showRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
                }
                else
                {
                    if route.legs.first?.steps.last?.maneuverType == ManeuverType.arrive
                    {
                        route.legs.first?.steps.removeLast()

                    }
                 
                    view.updateAndStartRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
                }

            }
        }
        
        
    }
    

    
    func calculateRoute(startPoint coordinate: CLLocationCoordinate2D,locale:String,measurementSystem: MeasurementSystem, objectFor view: MyNavigationViewProtocol){
        
        
        
        if let myLocation = LocationService.shared.getCurrentLocation()
        {
            
            let myCurrentWayPoint = Waypoint.init(location: myLocation)
            let wayPoint = Waypoint.init(coordinate: coordinate)
            
            let option = NavigationRouteOptions(waypoints: [myCurrentWayPoint,wayPoint], profileIdentifier: .walking)
            option.distanceMeasurementSystem = measurementSystem
            option.locale = Locale(identifier: locale)
            //   option.locale.usesMetric = true
            //            option.distanceMeasurementSystem = MeasurementSystem.init(rawValue: MyUtils.getDistanceMeasurementSystem())!
            option.distanceMeasurementSystem = measurementSystem
            option.waypoints = option.waypoints.map {
                $0.coordinateAccuracy = 150
                // $0.heading = 90
                return $0
            }
            
            
            /*   URLSession.shared.dataTask(with: URL(string: "https://api.mapbox.com/directions/v5/mapbox/walking/\(myCurrentWayPoint.coordinate.longitude),\(myCurrentWayPoint.coordinate.latitude);\(wayPoint.coordinate.longitude),\(wayPoint.coordinate.latitude)?access_token=pk.eyJ1IjoiaGFzc2luZW1pbGUiLCJhIjoiY2p6dG5jdnR5MDI0cTNsbDJ5bThoMzAwYSJ9.tFjeBrGWV_22rZrgmtnnRQ")!) { (data, response, error) -> Void in*/
            
            
            //            print("https://maps.googleapis.com/maps/api/directions/json?origin=\(myCurrentWayPoint.coordinate.latitude),\(myCurrentWayPoint.coordinate.longitude)&destination=\(wayPoint.coordinate.latitude),\(wayPoint.coordinate.longitude)&mode=driving&key=AIzaSyDVhvUK9CVFd-yo2_XW6AGRU2lxDLB_1vo")
            //
            //                      URLSession.shared.dataTask(with: URL(string: "https://maps.googleapis.com/maps/api/directions/json?origin=\(myCurrentWayPoint.coordinate.latitude),\(myCurrentWayPoint.coordinate.longitude)&destination=\(wayPoint.coordinate.latitude),\(wayPoint.coordinate.longitude)&mode=WALKING&key=AIzaSyDVhvUK9CVFd-yo2_XW6AGRU2lxDLB_1vo")!) { (data, response, error) -> Void in
            //                // Check if data was received successfully
            //                if error == nil && data != nil {
            //                    do {
            //                        // Convert to dictionary where keys are of type String, and values are of any type
            //                        let json = try JSONSerialization.jsonObject(with: data!, options: .mutableContainers) as! [String: Any]
            //                        // Access specific key with value of type String
            //                        var a = json["routes"] as!NSArray
            //                        var r = Route.init(json: a[0] as! [String : Any], waypoints: [myCurrentWayPoint,wayPoint], options: RouteOptions.init(waypoints: [myCurrentWayPoint,wayPoint]))
            //                         view.showRoute(route: r, waypoints: [myCurrentWayPoint,wayPoint])
            //                    } catch {
            //                        // Something went wrong
            //                    }
            //                }
            //            }.resume()
            
            
            //  }
            
            
            Directions.shared.calculate(option) { (session,result) in
                switch result {
                case .failure(let error):
                    print(error.localizedDescription)
                    view.showDirectRouteToStartpoint()
                case .success(let response):
                    guard let route = response.routes?.first else {
                        return
                    }
                    view.showRouteToStartpoint(route: route, routeOptions: option)
                }
            }
        }
    }
    
    func fetch(objectFor view: MyNavigationViewProtocol) {
        
    }
    
    func interactor(_ interactor: MyNavigationInteractorProtocol, didFetch object: MyNavigationEntity) {
        
    }
    
    func interactor(_ interactor: MyNavigationInteractorProtocol, didFailWith error: Error) {
        
    }
    
    
}



struct ResponseDisposition: Decodable {
    var code: String?
    var message: String?
    var error: String?
    
    private enum CodingKeys: CodingKey {
        case code, message, error
    }
}
