//
//  MyNavigationView+Functions.swift
//  NavigationUI
//
//  Created by Hassine on 04/06/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxCoreNavigation

import CoreLocation
import Speech
import MapboxDirections
import LiquidFloatingActionButton
import SnapKit
import MaterialComponents.MaterialButtons
import AVFoundation

import MapboxSpeech
import MapboxNavigation

extension MyNavigationView 
{

    @objc func sponsor_action(_ sender: UIButton) {
        showSpotify()
    }
    
    func initLocationManager() -> Void {
        presenter.initLocationManager(delegate: self)
    }
    
    func initMeasurementSystem() -> Void{
        presenter.initMeasurementSystem(measurementSystem:measurementSystem) // config  navigation MeasurementSystem
        updateMeasurementUI()                                               // Update UI "Bottom Bar"
    }
    
    func initAudioSpeech() -> Void {
        PlayerManager.shared.configureSession(lang:  languageAndCountry)
    }
    
    func initNavigation(userRoute:Route ,routeOptions: RouteOptions) -> Void {
        
        if navigationFinished  == false
        {

            routeOptions.distanceMeasurementSystem = .metric
            navigationService                                           = MapboxNavigationService.init(route: userRoute, routeIndex: 0, routeOptions: routeOptions)
            RouteControllerProactiveReroutingInterval = 500
            navigationService.router.reroutesProactively = false
            navigationService.delegate                                  = self
            initVoiceController()
            resumeNotifications()
            navigationService.start()

if navigationService != nil
{
            guard let firstInstruction = navigationService.routeProgress.currentLegProgress.currentStepProgress.currentVisualInstruction else {
                return

            }
            navigationService(navigationService, didPassVisualInstructionPoint: firstInstruction, routeProgress: navigationService.routeProgress)
}
        //    self.present(navigationViewController, animated: true, completion: nil)

          //  NotificationCenter.default.addObserver(self, selector: #selector(routeVoiceController.didPassSpokenInstructionPoint(notification:)), name: .routeControllerDidPassSpokenInstructionPoint, object: navigationService.router)

        }
    }
    
    
    
        func initOfflineNavigation(userRoute:Route ,routeOptions: RouteOptions) -> Void {
            
            if navigationFinished  == false
            {
                var stps = userRoute.legs.first?.steps
                
     
             
                
               
                var minimumDistance:CLLocationDistance = 99999
                var indexStep = 0
                for (index,stp) in stps!.enumerated()
                {
                    if stp.intersections!.first!.location.distance(to:LocationService.shared.getCurrentLocation()!.coordinate) < minimumDistance
                    {
                        minimumDistance = stp.intersections!.first!.location.distance(to:LocationService.shared.getCurrentLocation()!.coordinate)
                        indexStep = index
                    }
                }
                
                for n in 0 ... indexStep-1
                {
                    var e = stps?.remove(at: 0)
                    stps?.append(e!)
                }
                
                var inexArrive = 0
                for (index,stp) in stps!.enumerated()
                {
                    if stp.maneuverType == .arrive || stp.maneuverType == ManeuverType.depart
                    {
                        inexArrive = index
                    }
                }



                if inexArrive > 0
                {
                    stps?[inexArrive-1].instructionsDisplayedAlongStep?.removeAll()
                    stps?[inexArrive-1].instructionsSpokenAlongStep?.removeAll()

                    stps?.remove(at: inexArrive)

                }





                for stp in stps!
                {

                    if stp.maneuverType == ManeuverType.arrive || stp.maneuverType == ManeuverType.depart
                    {
                        stps?.removeAll(where: { (s1) -> Bool in
                            return s1 == stp
                        })
                    }


                }
                
                
    //            stp?.append(userRoute.legs.first!.steps[3])
    //            stp?.append(userRoute.legs.first!.steps[4])
    //            stp?.append(userRoute.legs.first!.steps[5])
    //            stp?.append(userRoute.legs.first!.steps[6])
    //            stp?.append(userRoute.legs.first!.steps[7])
    //            stp?.append(userRoute.legs.first!.steps.first!)
    //            stp?.append(userRoute.legs.first!.steps[1])
    //            stp?.append(userRoute.legs.first!.steps[2])

                for stp in stps!
                {
                    for te in stp.instructionsSpokenAlongStep!
                    {
                        print("stp = \(te.text) \(stp.maneuverType)")
                    }
                }

                
                userRoute.legs.first?.steps = stps!
                 

                var wayPoint = Waypoint.init(coordinate: userRoute.shape!.coordinates.first!)
                routeOptions.waypoints.append(wayPoint)
                routeOptions.distanceMeasurementSystem = .metric
                navigationService                                           = MapboxNavigationService.init(route: userRoute, routeIndex: 0, routeOptions: routeOptions)
                RouteControllerProactiveReroutingInterval = 500
                navigationService.router.reroutesProactively = false
                navigationService.delegate                                  = self
                initVoiceController()
                resumeNotifications()
                navigationService.start()
            }
        }
    
    func initVoiceController() -> Void {
        voiceController = CustomVoiceController.init(accessToken: navigationService.directions.credentials.accessToken, host: navigationService.directions.credentials.host.absoluteString)
           self.voiceController?.initNotification()

        //Added for flutter
        voiceController!.navigationView = self

        let speechSynthesizer = MultiplexedSpeechSynthesizer([voiceController] as? [SpeechSynthesizing])

        routeVoiceController = RouteVoiceController(navigationService: navigationService, speechSynthesizer: speechSynthesizer)

        //voiceController = RouteVoiceController(navigationService: navigationService,accessToken: credentials.accessToken, host: credentials.host.absoluteString)

    }
    
    func deinitNavigation()
    {
        timerCourse.invalidate()
        timerSmartFact?.invalidate()
        timerSmartFact = nil
        LocationService.shared.stopUpdatingLocation()
        if self.navigationService != nil
        {
            self.navigationService.stop()
            navigationService = nil
            
        }
        
        self.voiceController = nil
        self.suspendNotifications()
        //PlayerManager.shared.stopPlayer()
        navigationFinished  = true
        self.voiceController?.stopPlayer()
        PlayerManager.shared.stopSpeaking()
        DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+1) {
            if(self.whiteLabel?.isWhiteLabel ?? false && self.whiteLabel?.endInstruction != "") {
                PlayerManager.shared.playSynt(description: self.whiteLabel?.endInstruction ?? "")
            } else {
                PlayerManager.shared.playSynt(description: self.languageRessource.endRoute)
            }
            LocationService.shared.stopUpdatingLocation()
            LocationService.shared.locationManager.allowsBackgroundLocationUpdates = false

            
        }
    }
    
    func stopNavigation() -> Void {
        suspendNotifications()
        if navigationService != nil
        {
            navigationService.stop()
            navigationService.delegate = nil
        }
        voiceController = nil
        navigationService = nil
        PlayerManager.shared.speechSynthesizer.stopSpeaking(at: AVSpeechBoundary.immediate)
        wayToRoute = nil
    }
    

    
    func playSynthesizerDirectionnal(description:String) -> Void {
           let speechSynthesizer = SpeechSynthesizer.shared
               let options = SpeechOptions(text:  description)
               options.speechGender = .female
               options.locale = Locale.init(identifier: languageAndCountry)
               speechSynthesizer.audioData(with: options) { (data: Data?, error: Error?) in
                   guard error == nil else {
                       print("Error speechSynthesizer : \(error!)")
                       PlayerManager.shared.playSyntDirectionnal(description: description)
                   //    PlayerManager.shared.playSynt(description: speakString)
                       return
                   }
                   if let data  = data
                   {
                  self.voiceController?.playDirectionnal(instruction: SpokenInstruction.init(distanceAlongStep: 0, text: description, ssmlText: description), data: data)
                   }
               }
           }
}
