//
//  MyNavigationView+LiquidFloatingActionButtonDataSource+LiquidFloatingActionButtonDelegate.swift
//  NavigationUI
//
//  Created by Hassine on 16/06/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import Foundation
import LiquidFloatingActionButton
import AVFoundation
import MapboxCoreNavigation

import MessageUI


extension MyNavigationView: LiquidFloatingActionButtonDataSource, LiquidFloatingActionButtonDelegate
    
{
    public func numberOfCells(_ liquidFloatingActionButton: LiquidFloatingActionButton) -> Int {
        return cellsFloatingButton.count
    }

    public func cellForIndex(_ index: Int) -> LiquidFloatingCell {
        return cellsFloatingButton[index]
    }
    
    
    public func liquidFloatingActionButton(_ liquidFloatingActionButton: LiquidFloatingActionButton, didSelectItemAtIndex index: Int) {
        if index == 0
        {
            if isMuted
            {
                NavigationSettings.shared.voiceMuted = false
                activateSession()
                (cellsFloatingButton[index] as! CustomCell).label.text = "GUIDANCE_ACTIVE".localized
                PlayerManager.shared.volume = 1
            }
            else
            {
                NavigationSettings.shared.voiceMuted = true
                deactivateSession()
                PlayerManager.shared.volume = 0
                (cellsFloatingButton[index] as! CustomCell).label.text = "GUIDANCE_INACTIVE".localized
                
            }
            isMuted = !isMuted
            
        }
        else if index == 1 {
            flutterOnEmergencySmsClick()
        } else if index == 2 {
           flutterOnReportClick()
        }
    }
    
    
    func deactivateSession(){
        PlayerManager.shared.stopSpeaking()
        let when = DispatchTime.now() + 0.2 // We wait a little after cancel TTS to deativate session (if TTS is still speaking, we must no deactivate
        DispatchQueue.main.asyncAfter(deadline: when) {
            do {
                if !PlayerManager.shared.speechSynthesizer.isSpeaking {
                    if self.voiceController?.audioPlayer != nil {
                        guard !self.voiceController!.audioPlayer!.isPlaying else {
                            NSLog("Skip session deactivation because AVAudioPlayer is playing")
                            return
                        }
                    }
                    try AVAudioSession.sharedInstance().setActive(false)
                    NSLog("Session deactivated")
                }else{
                    NSLog("Will deactivate session after speech")
                }
            } catch {
                NSLog("Problem with the deactivation of the session")
            }
        }
    }
    
    func activateSession(){
        do {
            PlayerManager.shared.stopSpeaking()
            try AVAudioSession.sharedInstance().setCategory(AVAudioSession.Category.playback, options: [AVAudioSession.CategoryOptions.mixWithOthers, AVAudioSession.CategoryOptions.duckOthers])
            try AVAudioSession.sharedInstance().setActive(true)
            NSLog("Session activated")
        } catch {
            NSLog("Session deactiva")
        }
    }
    
    func showSpotify() -> Void {
        
    
        
        
        
        
        let appURL = URL(string: "spotify://")
        if UIApplication.shared.canOpenURL(appURL!) {
            if let stringUrl = routeInformation?.city?.spotifyURL {
                if let url = URL.init(string: stringUrl)
                {
                    UIApplication.shared.open(url)
                }
            }
        }
        else {
            let message = NSLocalizedString("INSTALL_SPOTIFY_ALERT", comment: "")
            let alert = UIAlertController(title: "INSTALL_SPOTIFY_ALERT_TITTLE".localized, message: "\(message) \(routeInformation?.city?.name ?? "")", preferredStyle: .alert)
            
            alert.addAction(UIAlertAction(title: "INSTALL".localized, style: .default, handler: { (_) in
                let url = "https://itunes.apple.com/app/spotify-music/id324684580?mt=8"
                if let url = URL(string: url) {
                    UIApplication.shared.open(url, options: [:], completionHandler: { (_) in
                    })
                }
            }))
            alert.addAction(UIAlertAction(title: "cancel".localized, style: .cancel, handler:{ (_) in
                
                alert.dismiss(animated: true)
            }))
            
            self.present(alert, animated: true)
        }
    }
    
    public func liquidFloatingActionButtonWillOpenDrawer(_ liquidFloatingActionButton: LiquidFloatingActionButton) {
        
        DispatchQueue.main.async {
            self.fab?.isHidden = true
            liquidFloatingActionButton.tintColor = .blue
            liquidFloatingActionButton.color = UIColor(red: 1.00, green: 1.00, blue: 1.00, alpha: 1.00)
            if #available(iOS 13.0, *) {
                var im =  UIImage(named: "moreGreen_ic")?.withTintColor(UIColor.red)
                liquidFloatingActionButton.image = im
                liquidFloatingActionButton.image!.withTintColor(UIColor.red)
            } else {
                // Fallback on earlier versions
            }
            liquidFloatingActionButton.tintColor = .blue
            let screenRect = CGRect.init(x: 0, y: -70, width: self.mapView_map.frame.width, height: self.mapView_map.frame.height)
            self.coverView = UIView(frame: screenRect)
            self.coverView.backgroundColor = UIColor(red: 0.02, green: 0.13, blue: 0.22, alpha: 1.00).withAlphaComponent(0.75)
            self.containerMap_view.insertSubview(self.coverView, aboveSubview: self.mapView_map)
            
        }
    }
    
    public func liquidFloatingActionButtonWillCloseDrawer(_ liquidFloatingActionButton: LiquidFloatingActionButton) {
        DispatchQueue.main.async {
            self.fab?.isHidden = false
            self.coverView.removeFromSuperview()
             if(self.whiteLabel?.isWhiteLabel ?? false) {
                liquidFloatingActionButton.color = UIColor(hex: self.gpsColor+"FF")!
            } else {
                liquidFloatingActionButton.color = Style.shared.primaryColor
            }
            liquidFloatingActionButton.image =  UIImage(named: "more_ic")
        }
    }
}
