//
//  MyNavigationView+MapFunctions.swift
//  NavigationUI
//
//  Created by Hassine on 21/08/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxNavigation
import MapboxDirections

import MapboxSpeech
import Speech


// MARK: - 

extension MyNavigationView {
    
    func removeLayerFromMap(withIdentifier:String) -> Void {
        DispatchQueue.main.async {
            if let layer = self.mapView_map.style!.layer(withIdentifier: withIdentifier)
            {
                self.mapView_map.style!.removeLayer(layer)
            }
        }
    }
    
    func polygonCircleForCoordinate(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) {
        let polygon = presenter.polygonCircleForCoordinate(coordinate: coordinate, withMeterRadius: withMeterRadius)
        self.mapView_map.addAnnotation(polygon)
    }

    func drawRoute(coordinates:[CLLocationCoordinate2D]) -> Void {
        DispatchQueue.main.async {
            guard coordinates.count > 0 else {return}
            var cour = coordinates
            let polyline = MGLPolylineFeature.init(coordinates: &cour, count: UInt(cour.count))
            if let source = self.mapView_map.style?.source(withIdentifier: "route-source") as? MGLShapeSource
            {
                source.shape = polyline
            }
            else
            {
                let source = MGLShapeSource.init(identifier: "route-source", features: [polyline], options: nil)
                let lineStyle = MGLLineStyleLayer.init(identifier: "route-style", source: source)
                let borderStyle = MGLLineStyleLayer.init(identifier: "route-style-border", source: source)
                var color = Style.shared.primaryColor
                if(self.whiteLabel?.isWhiteLabel ?? false) {
                   color = UIColor(hex: "#2196f3FF")! //blue universel color
               }
                lineStyle.lineColor = NSExpression(forConstantValue: color)
                lineStyle.lineWidth = NSExpression(forConstantValue: 5)
                borderStyle.lineColor = NSExpression(forConstantValue: UIColor.white)
                borderStyle.lineWidth = NSExpression(forConstantValue: 10)
                self.mapView_map.style!.addSource(source)
                self.mapView_map.style!.insertLayer(borderStyle, below: self.mapView_map.style!.layer(withIdentifier: "com.mapbox.annotations.points")!)
                self.mapView_map.style!.insertLayer(lineStyle, above: borderStyle)
            }
        }
    }
    
    func drawWayToRoute(coordinates: [CLLocationCoordinate2D]) {
        guard coordinates.count > 0 else { return }
        var routeCoordinates = coordinates
        let polyline = MGLPolylineFeature(coordinates: &routeCoordinates, count: UInt(coordinates.count))
        guard let style = self.mapView_map.style else { return }
        if let source = mapView_map.style?.source(withIdentifier: "WayToRoute-source") as? MGLShapeSource {
            polylineSource = source
            polylineSource!.shape = polyline
        } else {
            let source =  MGLShapeSource(identifier: "WayToRoute-source", shape: nil, options: nil)
            style.addSource(source)
            polylineSource = source
            let dashedLayer = MGLLineStyleLayer(identifier: "polyline-dash", source: source)
            dashedLayer.lineJoin = NSExpression(forConstantValue: "round")
            dashedLayer.lineCap = NSExpression(forConstantValue: "round")
            var color = Style.shared.primaryColor
            if(self.whiteLabel?.isWhiteLabel ?? false) {
                color = UIColor(hex: "#2196f3FF")! //blue universel color
            }
            dashedLayer.lineColor = NSExpression(forConstantValue: Style.shared.primaryColor)
            dashedLayer.lineOpacity = NSExpression(forConstantValue: 1)
            dashedLayer.lineWidth = NSExpression(format: "mgl_interpolate:withCurveType:parameters:stops:($zoomLevel, 'linear', nil, %@)",
                                                 [14: 3, 18: 20])
            dashedLayer.lineDashPattern = NSExpression(forConstantValue: [0, 1.5])
            mapView_map.style!.insertLayer(dashedLayer, below: mapView_map.style!.layer(withIdentifier: "com.mapbox.annotations.points")!)
        }
        
    }
    
    func addPois() -> Void {
        if let pois =  pois
        {
            for (index,poi) in pois.enumerated()
            {
                let annotation = MyCustomPointAnnotation()
                if let first = poi.coordinatesGPX.first
                {
                    annotation.coordinate = first
                    annotation.title = poi.key
                    annotation.subtitle = "\(index+1)"
                    annotation.category = poi.type
                    annotation.willUseImage = true
                    mappedAnnotation[poi.name] = index
                    self.mapView_map.addAnnotation(annotation)
                    if MyNavigationView.testMode == true
                    {
                        polygonCircleForCoordinate(coordinate: annotation.coordinate, withMeterRadius: Double(poi.range))
                    }
                }
            }
        }
    }

    func addSponsors() -> Void {
        /*if let route = routeInformation
        {
            for (index,sponsor) in route.sponsors.enumerated()
            {
                for coordinate in sponsor.coordinates
                {
                    let annotation = MyCustomPointAnnotation()
                    annotation.coordinate = coordinate
                    annotation.title = sponsor.key
                    //annotation.subtitle = "\(index+1)"
                    annotation.category = .sponsor
                    annotation.willUseImage = true
                    mappedSponsors[sponsor.key] = index
                    self.mapView_map.addAnnotation(annotation)
                }
            }
        }*/
    }

    func addSportSession() -> Void {
        if let sportsSession =  routeInformation?.sessions {
            for (index,sportSession) in sportsSession.enumerated() {
                let annotation = MyCustomPointAnnotation()
                if let first = sportSession.gps.first {
                    annotation.coordinate = first
                    annotation.category = .sport
                    annotation.title = sportSession.key
                    annotation.willUseImage = true
                    mappedAnnotation[sportSession.key!] = index
                    self.mapView_map.addAnnotation(annotation)
                    if MyNavigationView.testMode == true {
                        polygonCircleForCoordinate(coordinate: annotation.coordinate, withMeterRadius: Double(sportSession.radius!))
                    }
                }
            }
        }
    }
    
    
    func addDirectionals() -> Void {
        if MyNavigationView.testMode == true
        {
            for (index,poi) in directionals.enumerated()
            {
                
                if poi.isBloquant == true
                {
                    for coordinate in poi.coordinatesGPX
                    {
                        let annot = BloquantPointAnnotation()
                        annot.coordinate = coordinate
                        annot.title = poi.name
                        annot.subtitle = poi.description
                        mapView_map.addAnnotation(annot)
                        var coordinates = presenter.calculateCoordinates(coordinate: annot.coordinate, withMeterRadius: Double(poi.range))
                        let polygon = BloquantPointPolygon(coordinates: &coordinates, count: UInt(coordinates.count))
                        mappedAnnotation[poi.name] = index
                       self.mapView_map.addAnnotation(polygon)
                    }

                }
                else
                {
                    let annot = DirectionalPointAnnotation()
                    annot.coordinate = poi.coordinatesGPX.first!
                    annot.title = poi.name
                    annot.subtitle = poi.description
                    mapView_map.addAnnotation(annot)
                    var coordinates = presenter.calculateCoordinates(coordinate: annot.coordinate, withMeterRadius:  Double(poi.range))
                    let polygon = DirectionalPointPolygon(coordinates: &coordinates, count: UInt(coordinates.count))
                    mappedAnnotation[poi.name] = index
                  self.mapView_map.addAnnotation(polygon)
                }
            }
        }
    }
    
    func addFlags() -> Void {
        if let route = myRoute
        {
            let annotation = MyCustomFlagPointAnnotation()
            if let first = route.shape?.coordinates.first
            {
                annotation.coordinate = first
                annotation.willUseImage = true
                annotation.isStartFlag = true
                mapView_map.addAnnotation(annotation)
            }
            
            let lastAnnotation = MyCustomFlagPointAnnotation()
            if let last = route.shape?.coordinates.last
            {
                lastAnnotation.coordinate = last
                lastAnnotation.willUseImage = true
                lastAnnotation.isStartFlag = false
                mapView_map.addAnnotation(lastAnnotation)
            }
        }
    }
    
    func addFlagsOffline(first:CLLocationCoordinate2D,last:CLLocationCoordinate2D) -> Void {
        if let route = myRoute
        {
            let annotation = MyCustomFlagPointAnnotation()
            
                annotation.coordinate = first
                annotation.willUseImage = true
                annotation.isStartFlag = true
                mapView_map.addAnnotation(annotation)
            
            
            let lastAnnotation = MyCustomFlagPointAnnotation()
           
                lastAnnotation.coordinate = last
                lastAnnotation.willUseImage = true
                lastAnnotation.isStartFlag = false
                mapView_map.addAnnotation(lastAnnotation)
            
        }
    }
    
    func animatePolyline(allCoordinates:  [CLLocationCoordinate2D]) {
        currentIndex = 1
        timerPolyAnimation = Timer.scheduledTimer(timeInterval: 0.01, target: self, selector: #selector(tick(sender:)), userInfo:  allCoordinates, repeats: true)
    }
    
    @objc func tick(sender:  Timer) {
        if let allCoordinates = sender.userInfo as? [CLLocationCoordinate2D]
        {
            if currentIndex > allCoordinates.count {
                timerPolyAnimation?.invalidate()
                timerPolyAnimation = nil
                return
            }
            let coordinates = Array(allCoordinates[0..<currentIndex])
            updatePolylineWithCoordinates(coordinates: coordinates)
            currentIndex += 1
        }
    }
    
    func updatePolylineWithCoordinates(coordinates: [CLLocationCoordinate2D]) {
        var mutableCoordinates = coordinates
        let polyline = MGLPolylineFeature(coordinates: &mutableCoordinates, count: UInt(mutableCoordinates.count))
        polylineSource?.shape = polyline
    }
    
    func printGpxTag()
    {
        print("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n")
        MyUtils.generateGpxTags(coordinatesArray: gpxForTest!)
        print("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n")
    }
}

func checkIfInDirectionalZone(_ myDirectionPOIArray : Array<PointOfInterest>?, _ userPosition : CLLocation) -> Bool {
     guard userPosition.horizontalAccuracy < 50 else {
         return false
     }
     
     if let tempArrayPOIDirection = myDirectionPOIArray {
         for poiDirection in tempArrayPOIDirection {
             for coorGPS in poiDirection.getLocations() {
                 let delta: CLLocationDistance = userPosition.distance(from: coorGPS)
                 if delta < Double(poiDirection.range) {
                   //  NSLog("is in Direction zone")
                     return true
                 }
             }
         }
         return false
     }else{
         return false
     }
 }
