//
//  MyNavigationView+InitUIFunctions.swift
//  NavigationUI
//
//  Created by Hassine on 20/08/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxCoreNavigation

import CoreLocation
import Speech
import MapboxDirections
import LiquidFloatingActionButton
import SnapKit
import MaterialComponents.MaterialButtons
import AVFoundation



extension MyNavigationView
{
    func initUI() -> Void {
        DispatchQueue.main.async {
            //maneuverView
            self.maneuverView.primaryColor                          = UIColor.white
            self.maneuverView.secondaryColor                        = UIColor.white
            //primaryInstruction_lbl
            self.primaryInstruction_lbl.primaryInstructionStyle()
            self.primaryInstruction_lbl.numberOfLines               = 2;
            self.primaryInstruction_lbl.minimumScaleFactor          = 0.4
            self.primaryInstruction_lbl.adjustsFontSizeToFitWidth   = true;
            //distanceInstruction_lbl
            self.distanceInstruction_lbl.distanceInstructionStyle()
            //gps_lbl
            self.gpsSignl_lbl.title3Style()
            self.statusBarBanner_view.backgroundColor               = Style.shared.primaryColor
            //containerSeconcBannerView
            self.containerSecondBanner_view.roundCorners(corners: [.bottomLeft, .bottomRight], radius: 30)
            if(self.gpsColor != "") {
                self.containerBanner_view.backgroundColor = UIColor(hex: "#28353DFF")
            } else {
                self.containerBanner_view.backgroundColor = Style.shared.primaryColor
            }
            // informationsContainer_view
            self.informationsContainer_view.backgroundColor         = Style.shared.backgroundColor
            self.informationsContainer_view.layer.cornerRadius      = 30
            // pauseRun_slideBtn
            self.pauseRun_slideBtn.myView                           = self.dotSlide_view
            self.pauseRun_slideBtn.buttonUnlockedText               = "UNLOCKED"
            self.pauseRun_slideBtn.delegate                         = self
            //dotSlide_view
            self.dotSlide_view.layer.cornerRadius                   = 40
            self.dotSlide_view.layer.borderWidth                    = 15
            self.dotSlide_view.backgroundColor                      = Style.shared.slideDotColor
            self.dotSlide_view.layer.borderColor                    = Style.shared.slideDotBorderColor.cgColor//
             if(self.gpsColor != "") {
                self.dotSlide_view.backgroundColor = UIColor(hex: self.gpsColor+"FF")
                self.dotSlide_view.layer.borderColor = UIColor(hex: self.gpsColor+"FF")?.lighter(by: 30)?.cgColor
            } else {
                  self.dotSlide_view.backgroundColor = Style.shared.slideDotColor
                  self.dotSlide_view.layer.borderColor = Style.shared.slideDotBorderColor.cgColor//
            }
            self.dotSlide_view.isUserInteractionEnabled             = false
            // dotIcon_imageView
            self.dotIcon_imageView.image                            = UIImage(named: "pause_icon")
            //progressRouteIcon_imageView
            self.progressRouteIcon_imageView.image                  = UIImage(named: "thumbeilslider")
            // progressRoute_ProgressBar
            self.progressRoute_ProgressBar.trackTintColor           = Style.shared.slideBackgroundColor
            // Labels
            self.timerValue_lbl.timerCradStyle()
            if(self.gpsColor != "") {
                self.timerValue_lbl.font = Style.shared.timerCardFont
                self.timerValue_lbl.textColor = UIColor(hex: self.gpsColor+"FF")
            } else {
                 self.timerValue_lbl.timerCradStyle()
            }
            self.speedValue_lbl.valueCradStyle()
            self.distanceValue_lbl.valueCradStyle()
            self.speedTitle_lbl.titleCradStyle()
            self.distanceTitle_lbl.titleCradStyle()
            //center_btn
            self.center_btn.layer.cornerRadius                      = 20
            if(self.gpsColor != "") {
                self.center_btn.backgroundColor = UIColor(hex: self.gpsColor+"FF")
            } else {
                self.center_btn.backgroundColor = Style.shared.primaryColor
            }
            self.center_btn.setImage(UIImage(named: "ic_gps_center"), for: UIControl.State.normal)
        }
    }
    
    func initFloatingButtons() -> Void
    {
        var color = Style.shared.primaryColor
        let customCellFactory: (String,String) -> LiquidFloatingCell = { (iconName,title) in
            let cell = CustomCell(icon: UIImage(named: iconName)!, name: title)
            if(self.gpsColor != "") {
                color = UIColor(hex: self.gpsColor+"FF")!
            }

            return cell
        }
        
        cellsFloatingButton.append(customCellFactory("ic_sound_on", "AudioGuideActivated".localized))
        cellsFloatingButton.append(customCellFactory("sms_ic", "EmergencySMS".localized))
        cellsFloatingButton.append(customCellFactory("report_ic", "report".localized))
        
        let createButton: (CGRect, LiquidFloatingActionButtonAnimateStyle) -> LiquidFloatingActionButton = { (frame, style) in
            let floatingActionButton = CustomDrawingActionButton(frame: frame)
            floatingActionButton.animateStyle = style
            floatingActionButton.dataSource = self
            floatingActionButton.delegate = self
            floatingActionButton.color = color
            return floatingActionButton
        }
        
        let _: (CGRect, LiquidFloatingActionButtonAnimateStyle) -> LiquidFloatingActionButton = { (frame, style) in
            let floatingActionButton = CustomDrawingActionButton(frame: frame)
            floatingActionButton.animateStyle = style
            floatingActionButton.delegate = self
            floatingActionButton.color = UIColor.white
            /*var buttonColor = Style.shared.primaryColor
             if(self.whiteLabel?.isWhiteLabel ?? false) {
               buttonColor = UIColor(hex: self.gpsColor+"FF")!
            }*/
            return floatingActionButton
        }
        
        let floatingFrame = CGRect(x: self.containerMap_view.frame.width - 56 - 16, y: self.containerMap_view.frame.height - 56 - 35 , width: 56, height: 56)
        
        more_btn = createButton(floatingFrame, .up)
        
        let image = UIImage(named: "more_ic")
        more_btn.image = image
        self.containerMap_view.addSubview(more_btn)
        if let stringUrl = routeInformation?.city?.spotifyURL {
            if URL.init(string: stringUrl) != nil
            {
                
                DispatchQueue.main.async {
                    self.fab = MDCFloatingButton()
                    self.fab?.frame =  CGRect(x: self.containerMap_view.frame.width - 45 - 16 , y: self.containerMap_view.frame.height - 45 - 35 - 65 , width: 45, height: 45)
                    self.fab?.inkColor = Style.shared.primaryColor
                    self.fab?.backgroundColor = .white
                    self.fab?.tintColor = Style.shared.primaryColor
                    self.fab?.setImage(UIImage(named: "ic_music"), for: .normal)
                    self.fab?.addTarget(self, action: #selector(self.sponsor_action(_:)), for: .touchUpInside)
                    self.containerMap_view.addSubview(self.fab!)
                }
            }
        }
    }
    
    
    func initMap() -> Void {
        DispatchQueue.main.async {
            if let location = LocationService.shared.getCurrentLocation() {
                self.mapView_map.setCenter(location.coordinate, zoomLevel: 17, direction: 0, animated: false)
            }
            self.mapView_map.delegate                               = self
            self.mapView_map.compassView.isHidden                   = true
            self.mapView_map.logoView.isHidden                      = true
            self.mapView_map.attributionButton.isHidden             = true
            self.mapView_map.showsUserLocation                      = true
            self.mapView_map.tracksUserCourse                       = false
            self.mapView_map.userTrackingMode                       = .followWithHeading
            self.mapView_map.showsUserHeadingIndicator              = true
        }
    }
    
    func updateMeasurementUI() -> Void {
        if self.measurementSystem == .metric
        {

            DispatchQueue.main.async {
                self.speedTitle_lbl.text                            = "km/h"
                self.distanceTitle_lbl.text                         = "km"
            }
        }
        else if self.measurementSystem == .imperial
        {
            DispatchQueue.main.async {
                self.speedTitle_lbl.text                            = "mph"
                self.distanceTitle_lbl.text                         = "mi"
            }
        }
    }
    
    func initRouteInformations() -> Void {
        self.timerCourse = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(self.timerDuringSmartRun), userInfo: nil, repeats: true)
        //  self.timestamp = Int64(Date().timeIntervalSince1970)
        timerCourse.fire()
    }

    

    
    func updateGPS(position: CLLocation) -> Void {
           if (position.horizontalAccuracy < 0)
           {
               // No Signal
               gpsSignl_lbl.text = "GPS_NO_SIGNAL".localized
               gpsSignl_lbl.textColor = UIColor(red: 0.77, green: 0.25, blue: 0.85, alpha: 1.00)
               animateGpsSignlLbl()
               
           }
           else if (position.horizontalAccuracy > 70)
           {
               // Poor Signal
               gpsSignl_lbl.text = "GPS_SIGNAL_VERY_WEAK".localized
               gpsSignl_lbl.textColor = UIColor(red: 0.95, green: 0.35, blue: 0.32, alpha: 1.00)
               animateGpsSignlLbl()
           }
           else if (position.horizontalAccuracy > 15)
           {
               // Average Signal
               gpsSignl_lbl.text = "GPS_SIGNAL_WEAK".localized
               gpsSignl_lbl.textColor = UIColor(red: 0.97, green: 0.73, blue: 0.23, alpha: 1.00)
               animateGpsSignlLbl()
           }
           else
           {
               gpsSignl_lbl.text = ""
               gpsSignl_lbl.textColor = UIColor.green
               //animateGpsSignlLbl()
           }
       }
       
       func animateGpsSignlLbl(){
           gpsSignl_lbl.alpha = 0
           UIView.animate(withDuration: 0.3
               , animations: {
                   self.gpsSignl_lbl.alpha = 1
           }) { _ in
           }
       }
       
       
       func updateDistance(location:CLLocation) -> Void
       {
       
        if location.horizontalAccuracy < 25 && isPaused == false
        {
           traveledDistance += lastLocation!.distance(from: location)
         //  print("Traveled Distance:",  traveledDistance)
           var distance = distanceFormatter.string(from: traveledDistance)
           var traveledDistanceSplited = distance.split(separator: " ")
           if  traveledDistanceSplited.count <= 1
           {
               traveledDistanceSplited = distance.split(separator: " ")
           }
           if traveledDistanceSplited.count > 1
           {
               if ( String(traveledDistanceSplited[0]).count == 1)
               {
                   distanceValue_lbl.text = "\(traveledDistanceSplited[0]).0"
               }
               else
               {
                   distanceValue_lbl.text = String(traveledDistanceSplited[0])
               }
               distanceTitle_lbl.text = String(traveledDistanceSplited[1] )
           }
           else if traveledDistanceSplited.count == 1
           {
               distanceValue_lbl.text = "\(traveledDistanceSplited[0]).0"
           }
        }
       }
       
       func updateSpeed(location:CLLocation) -> Void
       {
        
        if location.horizontalAccuracy < 30 && isPaused == false
        {
           let distance =  Measurement(value: location.speed, unit: UnitSpeed.metersPerSecond)
           if distance.value > 0
           {
               var speedString : Measurement<Unit>
               if measurementSystem == .metric
               {
                   let speedString = distance.converted(to: UnitSpeed.kilometersPerHour)
                   let formatter = MeasurementFormatter()
                   formatter.unitOptions = .providedUnit
                   formatter.numberFormatter.maximumFractionDigits = 1
                   let speedSplited = formatter.string(from: speedString).split(separator: " ")
                   if speedSplited.count  > 1
                   {
                       self.speedValue_lbl.text = String(speedSplited[0])
                       self.speedTitle_lbl.text = String(speedSplited[1])
                   }
                   else
                   {
                       let speedSplited = formatter.string(from: speedString).split(separator: " ")
                       if speedSplited.count  > 1
                       {
                           self.speedValue_lbl.text = String(speedSplited[0])
                           self.speedTitle_lbl.text = String(speedSplited[1])
                       }
                   }
               }
               else if measurementSystem == .imperial
               {
                   let speedString = distance.converted(to: UnitSpeed.milesPerHour)
                   let formatter = MeasurementFormatter()
                   formatter.unitOptions = .providedUnit
                   formatter.numberFormatter.maximumFractionDigits = 1
                   let speedSplited = formatter.string(from: speedString).split(separator: " ")
                   if speedSplited.count  > 1
                   {
                       self.speedValue_lbl.text = String(speedSplited[0])
                       self.speedTitle_lbl.text = String(speedSplited[1])
                   }
                   else
                   {
                       let speedSplited = formatter.string(from: speedString).split(separator: " ")
                       if speedSplited.count  > 1
                       {
                           self.speedValue_lbl.text = String(speedSplited[0])
                           self.speedTitle_lbl.text = String(speedSplited[1])
                       }
                   }
               }
           }
           else
           {
               self.speedValue_lbl.text = "00.0"
               if measurementSystem == .metric
               {
                   self.speedTitle_lbl.text = " km/h"
               }
               else if measurementSystem == .imperial
               {
                   self.speedTitle_lbl.text = " mph"
               }
           }
       }
    }
}
