//
//  MyNavigationView.swift
//  NavigationUI
//
//  Created by hassine othmane on 3/25/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import UIKit

import MapboxNavigation

import MapboxDirections
import MapboxCoreNavigation

import Speech
import LiquidFloatingActionButton
import MaterialComponents.MaterialButtons

public class MyNavigationView: UIViewController {
    
    
    //MARK: - static
    public static var testMode = false
    public static var mutedZone : Bool = false
    static var statusAudioPlayerPoi : PoiStatuts = .notPlayed
    
    // MARK: - Strings
    let FINISH_TITLE_BTN = "FINISH_TITLE_BTN".localized(tableName: "NavigationUI", anyClass: MyNavigationView.self)
    
    // MARK: - IBOutlet
    
    // Top Banner View
    @IBOutlet weak var statusBarBanner_view         : UIView!
    @IBOutlet weak var containerBanner_view         : UIView!
    @IBOutlet weak var containerSecondBanner_view   : UIView!
    @IBOutlet weak var maneuverView                 : ManeuverView!
    @IBOutlet weak var distanceInstruction_lbl      : DistanceLabel!
    @IBOutlet weak var primaryInstruction_lbl       : UILabel!
    @IBOutlet weak var secondaryInstruction_lbl     : UILabel!
    // Map
    @IBOutlet weak var center_btn                   : UIButton!
    @IBOutlet weak var mapView_map                  : NavigationMapView!
    @IBOutlet weak var containerMap_view            : UIView!
    var more_btn                                    : LiquidFloatingActionButton!
    var spotify_btn                                 : LiquidFloatingActionButton!
    var fab                                         : MDCFloatingButton?
    // Bottom View
    @IBOutlet weak var informationsContainer_view   : UIView!
    @IBOutlet weak var timerValue_lbl               : UILabel!
    @IBOutlet weak var distanceValue_lbl            : UILabel!
    @IBOutlet weak var distanceTitle_lbl            : UILabel!
    @IBOutlet weak var speedValue_lbl               : UILabel!
    @IBOutlet weak var speedTitle_lbl               : UILabel!
    @IBOutlet weak var gpsSignl_lbl: UILabel!
    //Bottom View : progressView
    @IBOutlet weak var containerProgress_view       : UIView!
    @IBOutlet weak var containerView_view           : UIView!
    @IBOutlet weak var progressRoute_ProgressBar    : UIProgressView!
    @IBOutlet weak var progressRouteIcon_imageView  : UIImageView!
    //Bottom View : slide Button
    @IBOutlet weak var pauseRun_slideBtn            : MMSlidingButton!
    @IBOutlet weak var dotIcon_imageView            : UIImageView!
    @IBOutlet weak var dotSlide_view                : UIView!
    
    
    // MARK: - vars
    
    // Params
    var routeInformation    : RouteInformation?
    var pois                : [PointOfInterest]?
    var directionals        : [PointOfInterest] = [PointOfInterest]()
    
    var user                : MyUser?
    var measurementSystem   : MeasurementSystem!
    var routeCoordinates    : [CLLocationCoordinate2D]?
    
    // Audio
    var synthesizer         : AVSpeechSynthesizer = AVSpeechSynthesizer()
    var voiceController     : CustomVoiceController?
    var routeVoiceController     : RouteVoiceController?
    var isMuted = false
    
    
    // Stats Run informations
    var startDate           : Date!
    var traveledDistance    : Double = 0
    var runTime_seconds     : Int    = 0
    var runSmartFact_seconds: Int    = 0
    var indexSmartFact      : Int    = 0
    var smartFactsIndexes  = [Int]()
    // Locations
    var startLocation       : CLLocation?
    var lastLocation        : CLLocation?
    var previousLocation    : CLLocation?
    var distanceToStartPoint: Double? // when user leave the route before firstPoint the raiming distance is = 0 for this reason we will calculate the distance between location and start Point and we will check if the user advance or no
    
    // Map UI
    var wayToRoute          : Route?
    var myRoute             : Route?
    var myRouteOption       : RouteOptions?
    var polylineSource      : MGLShapeSource?
    var coverView           : UIView!
    
    // Navigation
    var navigationService   : NavigationService!
    var calculatingRoute    = false
    var navigationFinished  = false
    var isPaused            = false
    var isOfllineWayToRoute = false
    var isOfflinRoute       = false
    
    // Languages & Units
    var languageRessource   : LanguageRessource! //senteces of naviagtion (this depend of Language on settings not the langauge of the IPhone )
    var languageAndCountry  = "fr-FR"
    
    // Others
    var timerCourse         : Timer = Timer()
    var timerPolyAnimation  : Timer?
    var timerSmartFact      : Timer?
    var distanceWayToRoute  : Double?
    var distanceRoute       : Double?
    var distanceTravelledRoute:Double = 0
    var lastPositionDistanceRemaining: Double? // this distance represnt the rest of distance , we will use it to comapre with (navigationService.route.distance-distanceWayToRoute)
    var lastDistanceRemaining: Double?
    let distanceFormatter   : DistanceFormatter = DistanceFormatter()
    var gpxForTest          : [CLLocationCoordinate2D]?
    var currentIndex        = 1 // index of coodinate (used in animation of polyline)
    var mappedAnnotation    : [String:Int] = [String:Int]()
    var mappedDirectionals    : [String:Int] = [String:Int]()
    var mappedSponsors      : [String:Int] = [String:Int]()
    var arrayStringPOIQueue : Array<String> = []
    var presenter           : MyNavigationPresenterProtocol!
    var cellsFloatingButton : [LiquidFloatingCell] = []

    var lastCoordinateOffline : CLLocationCoordinate2D?
    var mappedSessions = [String:Int]()
    
    // MARK: - Clousres
    public var back_closure :(()->())?


    var navigationMode = ""
    var channel: FlutterMethodChannel?
    var isMapReady = false
    var mapReadyResult: FlutterResult?
    var registrar: FlutterPluginRegistrar!
    var viewId: Int64 = -1
    var whiteLabel: WhiteLabel?
    var offlineDatas: OfflineDatas?
    var gpsColor = ""
    var languages: [String]? = ["EN"]
    var distanceUnit = ""
    var speedUnit = ""
    var locomotion = ""
    var downloadForOffline = false
    
    // MARK: - viewDidLoad : in viewDidLoad we will init only the UI and important variables. (to have a good UI experienece)
    public override func viewDidLoad() {
        super.viewDidLoad()
        presenter = MyNavigationPresenter(view: self) // init the presenter
        initUI()
        initFlutterChannel()
        initMeasurementSystem() // init the unit of mapbox
        initMap()
        
    }
    
    // MARK: - viewDidAppear : in viewDidAppear we will init all others variables and after unit the UI correctly.
    public override func viewDidAppear(_ animated: Bool) {
        initFloatingButtons() // init the floating buttons : + and music button
        checkBestLanguage()
        initLocationManager()

        initAudioSpeech() // we will init the Speech Synth , the voice Controller 'Amazon' we will init it later when we have the navigation service
        
        #if DEBUG
       
        #else
        MyNavigationView.testMode = false
        #endif
        // TODO: is better to init user, smart facts, language ... before show the UIViewController : actually the app have problems with communication between diffrent modules : Combine , RXSwift can be a good solution , or continie with delagtes and closures.
    }
    
    // MARK: - IBAction
    @IBAction func mute_action(_ sender: Any) {
        if isMuted
        {
          
           
            self.voiceController?.audioPlayer?.volume = NavigationSettings.shared.voiceVolume
            PlayerManager.shared.volume = NavigationSettings.shared.voiceVolume
            
        }
        else
        {
            self.voiceController?.audioPlayer?.volume = 0
            PlayerManager.shared.volume = 0
        }
        isMuted = !isMuted
    }
    
    @IBAction func centerMap_action(_ sender: Any) {
       DispatchQueue.main.async {
           
            self.center_btn.isHidden = true
            
            self.mapView_map.setUserTrackingMode(.followWithHeading, animated: false) {
                 //self.mapView_map.zoomLevel = 18
            }
        }
//        var mode: MGLUserTrackingMode
//
//        switch (mapView_map.userTrackingMode) {
//        case .none:
//            mode = .follow
//        case .follow:
//            mode = .followWithHeading
//        case .followWithHeading:
//            mode = .followWithCourse
//        case .followWithCourse:
//            mode = .none
//        @unknown default:
//            fatalError("Unknown user tracking mode")
//        }
//
//        mapView_map.userTrackingMode = mode

    }
    
    func setRoute(routeCoordinates:[CLLocationCoordinate2D]?) -> Void {
        self.routeCoordinates = routeCoordinates
    }
    
    func set(object: MyNavigationEntity) {
    }
}
