//
//  MyNavigationContracts.swift
//  NavigationUI
//
//  Created Hassine on 10/03/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import UIKit
import CoreLocation
import MapboxDirections
//MARK: View -
/*
 Should replace "class" with "BaseViewProtocol" if available;
 & that will allow the View to act as a UIViewController;
 & Implement common view functions.
 */
/// MyNavigation Module View Protocol
protocol MyNavigationViewProtocol: class {
    // Update UI with value returned.
    /// Set the view Object of Type MyNavigationEntity
    func set(object: MyNavigationEntity)

    /// show the route to the Start Point
    func showRouteToStartpoint(route: Route,routeOptions: RouteOptions)

    /// show the route to the Start Point
    func showRoute(route:Route,routeOptions: RouteOptions, waypoints:[Waypoint]?)

    /// show the route to the Start Point
    func updateAndStartRoute(route: Route,routeOptions: RouteOptions, waypoints: [Waypoint]?)

    /// show system Message
    func showAlertSystemMessage(title:String,message:String)

}

//MARK: Interactor -
/// MyNavigation Module Interactor Protocol
protocol MyNavigationInteractorProtocol {
    // Fetch Object from Data Layer
    func fetch(objectFor presenter: MyNavigationPresenterProtocol)
}

//MARK: Presenter -
/// MyNavigation Module Presenter Protocol
protocol MyNavigationPresenterProtocol {
    /// The presenter will fetch data from the Interactor thru implementing the Interactor fetch function.
    func fetch(objectFor view: MyNavigationViewProtocol)
    /// The Interactor will inform the Presenter a successful fetch.
    func interactor(_ interactor: MyNavigationInteractorProtocol, didFetch object: MyNavigationEntity)
    /// The Interactor will inform the Presenter a failed fetch.
    func interactor(_ interactor: MyNavigationInteractorProtocol, didFailWith error: Error)

    /// calculate route from your location to a point
    func calculateRoute(startPoint coordinate:CLLocationCoordinate2D,locale:String,measurementSystem: MeasurementSystem, objectFor view: MyNavigationViewProtocol)

    /// calculate route from list of points
    func calculateRoute(points : [CLLocationCoordinate2D],locale:String,measurementSystem: MeasurementSystem, objectFor view: MyNavigationViewProtocol,profileIdentifier:MBDirectionsProfileIdentifier?,firstCalculation:Bool)
    
}

//MARK: Router (aka: Wireframe) -
/// MyNavigation Module Router Protocol
protocol MyNavigationRouterProtocol {
    // Show Details of Entity Object coming from ParentView Controller.
    // func showDetailsFor(object: MyNavigationEntity, parentViewController viewController: UIViewController)
}
