//
//  MyNavigationPresenter.swift
//  NavigationUI
//
//  Created Hassine on 10/03/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import UIKit
import CoreLocation
import MapboxCoreNavigation
import MapboxDirections

/// MyNavigation Module Presenter
class MyNavigationPresenter {
    
    weak private var _view: MyNavigationViewProtocol?
    private var interactor: MyNavigationInteractorProtocol
    private var wireframe: MyNavigationRouterProtocol
    
    init(view: MyNavigationViewProtocol) {
        self._view = view
        self.interactor = MyNavigationInteractor()
        self.wireframe = MyNavigationRouter()
    }
}

// MARK: - extending MyNavigationPresenter to implement it's protocol
extension MyNavigationPresenter: MyNavigationPresenterProtocol {

    func calculateRoute(points: [CLLocationCoordinate2D], locale: String, measurementSystem: MeasurementSystem, objectFor view: MyNavigationViewProtocol, profileIdentifier: MBDirectionsProfileIdentifier?, firstCalculation: Bool) {

        let profileIdentifier = profileIdentifier ?? MBDirectionsProfileIdentifier.walking
        var waypoints = [Waypoint]()
        for point in points {
            let waypoint = Waypoint.init(coordinate: point)
            waypoints.append(waypoint)
        }

        let matchOptions = NavigationMatchOptions(waypoints: waypoints, profileIdentifier: profileIdentifier)
        matchOptions.includesSteps = true

        //matchOptions.waypointIndices = IndexSet([0, 1, waypoints.count - 1])
        let middle = waypoints.count / 2
        var index = 0
        matchOptions.waypoints.forEach { (waypoint) in
            if index  == middle {
                waypoint.separatesLegs = true
            } else {
                waypoint.separatesLegs = false
            }
            index += 1
        }

      //  matchOptions.distanceMeasurementSystem = MeasurementSystem.init(rawValue: MyUtils.getDistanceMeasurementSystem())!
        
        matchOptions.locale = Locale(identifier: locale)
     //   matchOptions.locale.usesMetric = true
         matchOptions.distanceMeasurementSystem = measurementSystem
//        Directions.shared.calculateRoutes(matching: matchOptions) { (waypoints: [Waypoint]?, routes: [Route]?, error: Error?) in
//            if let route = routes?.first {
//                if firstCalculation {
//                     view.showRoute(route: route, waypoints: waypoints)
//                } else {
//                     view.updateAndStartRoute(route: route, waypoints: waypoints)
//                }
//            } else {
//               print("LOG : don't find way to the route")
//            }
//        }





        Directions.shared.calculateRoutes(matching: matchOptions) { (session, result) in

            switch result {
            case .failure(let error):
                print(error.localizedDescription)
            case .success(let response):

                guard var route = response.routes?.first else {
                    return
                }

                if firstCalculation
                {
                    if route.legs.first?.steps.last?.maneuverType == ManeuverType.arrive
                    {
                       // route.legs.first?.steps.removeLast()

                    }

                    view.showRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
                }
                else
                {
                    if route.legs.first?.steps.last?.maneuverType == ManeuverType.arrive
                    {
                       // route.legs.first?.steps.removeLast()

                    }
                    view.updateAndStartRoute(route: route, routeOptions: RouteOptions.init(matchOptions: matchOptions), waypoints: waypoints)
                }

            }
        }

 
        
        
 /*       URLSession.shared.dataTask(with: URL(string: "https://api.mapbox.com/directions/v5/mapbox/cycling/\(waypoints.first!.coordinate.longitude),\(waypoints.first!.coordinate.latitude);\(waypoints.last!.coordinate.longitude),\(waypoints.last!.coordinate.latitude)?access_token=pk.eyJ1IjoiaGFzc2luZW1pbGUiLCJhIjoiY2p6dG5jdnR5MDI0cTNsbDJ5bThoMzAwYSJ9.tFjeBrGWV_22rZrgmtnnRQ&geometries=polyline6&overview=full&steps=true&bearings=%3B&continue_straight=true&annotations=congestion%2Cdistance&language=nl&roundabout_exits=true&voice_instructions=true&banner_instructions=true&voice_units=imperial&enable_refresh=true")!) { (data, response, error) -> Void in
            // Check if data was received successfully
            if error == nil && data != nil {
                do {
                    // Convert to dictionary where keys are of type String, and values are of any type
                    let json = try JSONSerialization.jsonObject(with: data!, options: .mutableContainers) as! [String: Any]
                    // Access specific key with value of type String
                    var a = json["routes"] as!NSArray
                    var r = Route.init(json: a[0] as! [String : Any], waypoints: waypoints, options: RouteOptions.init(waypoints: waypoints))
                     view.showRoute(route: r, waypoints: waypoints)
                } catch {
                    // Something went wrong
                }
            }
        }.resume()
        
        */

    }

    func calculateRoute(startPoint coordinate: CLLocationCoordinate2D, locale: String, measurementSystem: MeasurementSystem, objectFor view: MyNavigationViewProtocol) {

        if let myLocation = LocationService.shared.getCurrentLocation() {

            let myCurrentWayPoint = Waypoint.init(location: myLocation)
            let wayPoint = Waypoint.init(coordinate: coordinate)

            let option = NavigationRouteOptions(waypoints: [myCurrentWayPoint,wayPoint], profileIdentifier: .walking)
         
           option.locale = Locale(identifier: locale)
         //   option.locale.usesMetric = true
//            option.distanceMeasurementSystem = MeasurementSystem.init(rawValue: MyUtils.getDistanceMeasurementSystem())!
            option.distanceMeasurementSystem = measurementSystem
            option.waypoints = option.waypoints.map {
                $0.coordinateAccuracy = 150
               // $0.heading = 90
                return $0
            }
            
         /*   URLSession.shared.dataTask(with: URL(string: "https://api.mapbox.com/directions/v5/mapbox/walking/\(myCurrentWayPoint.coordinate.longitude),\(myCurrentWayPoint.coordinate.latitude);\(wayPoint.coordinate.longitude),\(wayPoint.coordinate.latitude)?access_token=pk.eyJ1IjoiaGFzc2luZW1pbGUiLCJhIjoiY2p6dG5jdnR5MDI0cTNsbDJ5bThoMzAwYSJ9.tFjeBrGWV_22rZrgmtnnRQ")!) { (data, response, error) -> Void in*/
                
                
//            print("https://maps.googleapis.com/maps/api/directions/json?origin=\(myCurrentWayPoint.coordinate.latitude),\(myCurrentWayPoint.coordinate.longitude)&destination=\(wayPoint.coordinate.latitude),\(wayPoint.coordinate.longitude)&mode=driving&key=AIzaSyDVhvUK9CVFd-yo2_XW6AGRU2lxDLB_1vo")
//
//                      URLSession.shared.dataTask(with: URL(string: "https://maps.googleapis.com/maps/api/directions/json?origin=\(myCurrentWayPoint.coordinate.latitude),\(myCurrentWayPoint.coordinate.longitude)&destination=\(wayPoint.coordinate.latitude),\(wayPoint.coordinate.longitude)&mode=WALKING&key=AIzaSyDVhvUK9CVFd-yo2_XW6AGRU2lxDLB_1vo")!) { (data, response, error) -> Void in
//                // Check if data was received successfully
//                if error == nil && data != nil {
//                    do {
//                        // Convert to dictionary where keys are of type String, and values are of any type
//                        let json = try JSONSerialization.jsonObject(with: data!, options: .mutableContainers) as! [String: Any]
//                        // Access specific key with value of type String
//                        var a = json["routes"] as!NSArray
//                        var r = Route.init(json: a[0] as! [String : Any], waypoints: [myCurrentWayPoint,wayPoint], options: RouteOptions.init(waypoints: [myCurrentWayPoint,wayPoint]))
//                         view.showRoute(route: r, waypoints: [myCurrentWayPoint,wayPoint])
//                    } catch {
//                        // Something went wrong
//                    }
//                }
//            }.resume()
            
             
      //  }
            
//          Directions.shared.calculate(option, completionHandler: { (waypoints, routes, error)  in
//               if let route = routes?.first
//                {
//                     view.showRouteToStartpoint(route: route, waypoints: waypoints)
//                }
//                else
//                {
//                    print("LOG : don't find way to the route")
//
//                }
//            })


            Directions.shared.calculate(option) { (session,result) in
                switch result {
                case .failure(let error):
                    print(error.localizedDescription)
                   
                case .success(let response):
                    guard let route = response.routes?.first else {
                        return
                    }
                    view.showRouteToStartpoint(route: route, routeOptions: option)
                }
            }
       }



    }

    func fetch(objectFor view: MyNavigationViewProtocol) {
        
    }
    
    func interactor(_ interactor: MyNavigationInteractorProtocol, didFetch object: MyNavigationEntity) {
        
    }
    
    func interactor(_ interactor: MyNavigationInteractorProtocol, didFailWith error: Error) {
        
    }
    
    
}
