//
//  CustomUserLocationAnnotationView.swift
//  NavigationUI
//
//  Created by hassine othmane on 3/29/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import Foundation
import UIKit
import Mapbox


// Create a subclass of MGLUserLocationAnnotationView.
class CustomUserLocationAnnotationView: MGLUserLocationAnnotationView {
    let size: CGFloat = 72
    var dot: CALayer!
    var dot2: CALayer!
    //var arrow: CAShapeLayer!
    var image : UIImageView!

    // -update is a method inherited from MGLUserLocationAnnotationView. It updates the appearance of the user location annotation when needed. This can be called many times a second, so be careful to keep it lightweight.
    override func update() {
        if frame.isNull {
            frame = CGRect(x: 0, y: 0, width: size, height: size)
            return setNeedsLayout()
        }

        // Check whether we have the user’s location yet.
        if CLLocationCoordinate2DIsValid(userLocation!.coordinate) {
            setupLayers()
            updateHeading()
        }
    }

    private func updateHeading() {
        // Show the heading arrow, if the heading of the user is available.
        if let heading = userLocation!.heading?.trueHeading {
            image.isHidden = false

            // Get the difference between the map’s current direction and the user’s heading, then convert it from degrees to radians.
            let rotation: CGFloat = -MGLRadiansFromDegrees(mapView!.direction - heading)

            // If the difference would be perceptible, rotate the arrow.
            if abs(rotation) > 0.01 {
                // Disable implicit animations of this rotation, which reduces lag between changes.
                CATransaction.begin()
                CATransaction.setDisableActions(true)
                // image.setAffineTransform(CGAffineTransform.identity.rotated(by: rotation))
                CATransaction.commit()
            }
        } else {
            image.isHidden = true
        }
    }

    private func setupLayers() {


        if dot2 == nil {
            dot2 = CALayer()
            dot2.bounds = CGRect(x: 0, y: 0, width: 118, height: 118)

            // Use CALayer’s corner radius to turn this layer into a circle.
            dot2.cornerRadius = 118 / 2
            dot2.backgroundColor = UIColor(red:0.07, green:0.15, blue:0.23, alpha:0.1).cgColor
            // dot.borderWidth = 4
            //dot.borderColor = UIColor.white.cgColor
            layer.addSublayer(dot2)
        }


        // This dot forms the base of the annotation.
        if dot == nil {
            dot = CALayer()
            dot.bounds = CGRect(x: 0, y: 0, width: size, height: size)

            // Use CALayer’s corner radius to turn this layer into a circle.
            dot.cornerRadius = size / 2
            dot.backgroundColor = UIColor(red:0.07, green:0.15, blue:0.23, alpha:0.1).cgColor
            // dot.borderWidth = 4
            //dot.borderColor = UIColor.white.cgColor
            layer.addSublayer(dot)
        }




        // This arrow overlays the dot and is rotated with the user’s heading.
        if image == nil {
//            arrow = CAShapeLayer()
//            arrow.path = arrowPath()
//            arrow.frame = CGRect(x: 0, y: 0, width: size/1.7 , height: size/1.7 )
//           arrow.position = CGPoint(x: dot.frame.midX, y: dot.frame.midY)
//            arrow.fillColor = UIColor(red:0.20, green:0.72, blue:0.88, alpha:1.00).cgColor
                // arrow.backgroundColor = UIColor.red.cgColor

            image = UIImageView.init()
            image.image = UIImage(named: "puck")
            image.contentMode = .scaleAspectFill
            image.frame = CGRect(x:0, y:0, width: size/1.6, height: size/1.6)
            image.center = CGPoint.init(x: 0, y: 0)
            self.addSubview(image)


//            var v = UIView.init()
//            v.frame = CGRect(x: -size/2, y: -size/2, width: size, height: size)
//            v.backgroundColor = UIColor.red
//             self.addSubview(v)
        }
    }

    // Calculate the vector path for an arrow, for use in a shape layer.
    private func arrowPath() -> CGPath {
       let max: CGFloat = size / 1.7
        let pad: CGFloat = 3

        let top =    CGPoint(x: max * 0.5, y: 0)
        let left =   CGPoint(x: 0 + pad,   y: max - pad-5)
        let left_circle = CGPoint(x: 0 + pad-5,   y: max - pad-5)
        let right =  CGPoint(x: max - pad, y: max - pad)
        let center = CGPoint(x: max * 0.5, y: max - pad - 5)

        let bezierPath = UIBezierPath()
        bezierPath.move(to: top)
        bezierPath.addLine(to: left)

        bezierPath.addArc(withCenter: left_circle, radius: 5, startAngle:  CGFloat(M_PI), endAngle: CGFloat(3*M_PI_2), clockwise: true)


        bezierPath.addLine(to: center)
        bezierPath.addLine(to: right)
        bezierPath.addLine(to: top)
        bezierPath.close()
        return bezierPath.cgPath
    }
}

//bezierPath.addArc(withCenter: left, radius: 10, startAngle:  CGFloat(M_PI), endAngle: CGFloat(3*M_PI_2), clockwise: true)
//
//
//      //  bezierPath.addArc(withCenter: left, radius: 10, startAngle:  CGFloat(M_PI), endAngle: CGFloat(3*M_PI_2), clockwise: true)
//
//
//       bezierPath.addArc(withCenter: center, radius: 10, startAngle:  CGFloat(M_PI), endAngle: CGFloat(3*M_PI_2), clockwise: true)
//
//       bezierPath.addArc(withCenter: right, radius: 10, startAngle:  CGFloat(M_PI), endAngle: CGFloat(3*M_PI_2), clockwise: true)
//
//
//        bezierPath.addArc(withCenter: top, radius: 10, startAngle:  CGFloat(M_PI), endAngle: CGFloat(3*M_PI_2), clockwise: true)


//
//class CustomUserLocationAnnotationView: MGLUserLocationAnnotationView {
//    let size: CGFloat = 48
//    var dot: CALayer!
//    var arrow: CAShapeLayer!
//
//    // -update is a method inherited from MGLUserLocationAnnotationView. It updates the appearance of the user location annotation when needed. This can be called many times a second, so be careful to keep it lightweight.
//    override func update() {
//        if frame.isNull {
//            frame = CGRect(x: 0, y: 0, width: size, height: size)
//            return setNeedsLayout()
//        }
//
//        // Check whether we have the user’s location yet.
//        if CLLocationCoordinate2DIsValid(userLocation!.coordinate) {
//            setupLayers()
//            updateHeading()
//        }
//    }
//
//    private func updateHeading() {
//        // Show the heading arrow, if the heading of the user is available.
//        if let heading = userLocation!.heading?.trueHeading {
//            arrow.isHidden = false
//
//            // Get the difference between the map’s current direction and the user’s heading, then convert it from degrees to radians.
//            let rotation: CGFloat = -MGLRadiansFromDegrees(mapView!.direction - heading)
//
//            // If the difference would be perceptible, rotate the arrow.
//            if abs(rotation) > 0.01 {
//                // Disable implicit animations of this rotation, which reduces lag between changes.
//                CATransaction.begin()
//                CATransaction.setDisableActions(true)
//                arrow.setAffineTransform(CGAffineTransform.identity.rotated(by: rotation))
//                CATransaction.commit()
//            }
//        } else {
//            arrow.isHidden = true
//        }
//    }
//
//    private func setupLayers() {
//        // This dot forms the base of the annotation.
//        if dot == nil {
//            dot = CALayer()
//            dot.bounds = CGRect(x: 0, y: 0, width: size, height: size)
//
//            // Use CALayer’s corner radius to turn this layer into a circle.
//            dot.cornerRadius = size / 2
//            dot.backgroundColor = super.tintColor.cgColor
//            dot.borderWidth = 4
//            dot.borderColor = UIColor.white.cgColor
//            layer.addSublayer(dot)
//        }
//
//        // This arrow overlays the dot and is rotated with the user’s heading.
//        if arrow == nil {
//            arrow = CAShapeLayer()
//            arrow.path = arrowPath()
//            arrow.frame = CGRect(x: 0, y: 0, width: size / 2, height: size / 2)
//            arrow.position = CGPoint(x: dot.frame.midX, y: dot.frame.midY)
//            arrow.fillColor = dot.borderColor
//            layer.addSublayer(arrow)
//        }
//    }
//
//    // Calculate the vector path for an arrow, for use in a shape layer.
//    private func arrowPath() -> CGPath {
//        let max: CGFloat = size / 2
//        let pad: CGFloat = 3
//
//        let top =    CGPoint(x: max * 0.5, y: 0)
//        let left =   CGPoint(x: 0 + pad,   y: max - pad)
//        let right =  CGPoint(x: max - pad, y: max - pad)
//        let center = CGPoint(x: max * 0.5, y: max * 0.6)
//
//        let bezierPath = UIBezierPath()
//        bezierPath.move(to: top)
//        bezierPath.addLine(to: left)
//        bezierPath.addLine(to: center)
//        bezierPath.addLine(to: right)
//        bezierPath.addLine(to: top)
//        bezierPath.close()
//
//        return bezierPath.cgPath
//    }
//}
