//
//  PlayerManager.swift
//  MyThalassa
//
//  Created by Hassine on 09/10/2019.
//  Copyright © 2019 mile. All rights reserved.
//

import Foundation
import AVFoundation
import MapboxCoreNavigation



class PlayerManager:NSObject {
    
    var arrayStringPOIQueue:        Array<String> = []
    static let shared = PlayerManager()
    var audioPlayer: AVAudioPlayer?
    //  var audioPlayerPoiDescription: AVAudioPlayer?
    var audioPlayerPoiDescriptionQueue: AVQueuePlayer?
    var audioPlayerDirectional: AVAudioPlayer?
    
    var timer : Timer? = nil

    var aVSpeechSynthesisVoice : AVSpeechSynthesisVoice!
    var speechUtterance: AVSpeechUtterance!
    var speechSynthesizer = AVSpeechSynthesizer()
    var theSession  = AVAudioSession()
    var askToDeactivateSession = false
    
    func configureSession(lang:String) {
        arrayStringPOIQueue.removeAll()
        theSession = AVAudioSession.sharedInstance()
        speechSynthesizer.delegate = self
        aVSpeechSynthesisVoice = AVSpeechSynthesisVoice.init(language: lang)
        let voices = AVSpeechSynthesisVoice.speechVoices()
        var voiceToUse: AVSpeechSynthesisVoice?
        for voice in voices {
            print(voice)
            if voice.quality == .enhanced && voice.language == lang{
                aVSpeechSynthesisVoice = voice
            }
        }
        speechUtterance = AVSpeechUtterance.init()
        speechUtterance.rate = 0.51
        speechUtterance.voice = aVSpeechSynthesisVoice
        speechUtterance.volume = 1
        volume = 1
        NavigationSettings.shared.voiceMuted = false
                    
        
    }
    var volume:Float = 1
    
    
    func playSynt(description:String)
    {
        speechUtterance = AVSpeechUtterance(string: description)
        speechUtterance.rate = 0.51
        speechUtterance.voice = aVSpeechSynthesisVoice
       speechUtterance.volume = volume
        if speechSynthesizer.isSpeaking
        {
            arrayStringPOIQueue.append(description)
        }
        else if speechSynthesizer.isPaused
        {
            arrayStringPOIQueue.append(description)
            speechSynthesizer.continueSpeaking()
        }
        else
        {
            do {
                       try duckAudio()
                   } catch {
                      print("error")
                   }
            speechSynthesizer.speak(speechUtterance)
        }
        
    }
    
    func stopSynt() -> Void {
        
        if speechSynthesizer.isSpeaking {
            speechSynthesizer.stopSpeaking(at: .immediate)
            
        }
    }
    
   func stopSpeaking(){
        if speechSynthesizer.isSpeaking {
            speechSynthesizer.stopSpeaking(at: .immediate)
        }
    arrayStringPOIQueue.removeAll()
    }

    
    
    func startPlayerDirectional(data:Data, _ callBack:@escaping () -> Void) {
        do
        {
            if data != nil {
                self.audioPlayerDirectional = try AVAudioPlayer.init(data: data, fileTypeHint: AVFileType.mp3.rawValue)
                guard let audioPlayer = self.audioPlayerDirectional else { return }
                audioPlayer.numberOfLoops = 0
                let audioSession = AVAudioSession.sharedInstance()
                try audioSession.setCategory(AVAudioSession.Category.playback)
                try audioSession.setActive(true)
                audioPlayer.prepareToPlay()
                
                callBack()
            }
            else
            {
                print("data nil")
                return
            }
        }
        catch
        {
        }
    }

    func playPlayerDirectional() -> Void {
        if let player = audioPlayerDirectional
        {
            player.play()
        }
    }
    
    func pausePlayerDirectional() -> Void {
           if let player = audioPlayerDirectional
           {
               player.pause()
               
               if let timer = timer
               {
                   timer.invalidate()
               }
           }
       }
    
    func stopPoiDirectional() -> Void {
           if let player = audioPlayerDirectional
           {
               player.stop()
               if let timer = timer
               {
                   timer.invalidate()
               }
           }
       }
    
    
    

    
    
}

extension PlayerManager:AVSpeechSynthesizerDelegate
{
    
    
    func duckAudio() throws {
        let audioSession = AVAudioSession.sharedInstance()
        if #available(iOS 12.0, *) {
            try audioSession.setCategory(.playback, mode: .voicePrompt, options: [.duckOthers, .mixWithOthers])
        } else {
            try audioSession.setCategory(.ambient, mode: .spokenAudio, options: [.duckOthers, .mixWithOthers])
        }
        try audioSession.setActive(true)
    }
    
    func mixAudio() throws {
        let audioSession = AVAudioSession.sharedInstance()
        try audioSession.setCategory(.ambient, mode: audioSession.mode)
        try audioSession.setActive(true)
    }
    
    func unDuckAudio() throws {
        try AVAudioSession.sharedInstance().setActive(false, options: [.notifyOthersOnDeactivation])
    }

    
    
    
    public func speechSynthesizer(_ synthesizer: AVSpeechSynthesizer, didStart utterance: AVSpeechUtterance) {
        print("AVSpeechSynthesizerDelegate: didStart")
    }
    
    public func speechSynthesizer(_ synthesizer: AVSpeechSynthesizer, didFinish utterance: AVSpeechUtterance) {
        print("AVSpeechSynthesizerDelegate: didFinish")
        if arrayStringPOIQueue.count > 0
        {
            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+1) {
                self.playSynt(description: self.arrayStringPOIQueue.first!)
                self.arrayStringPOIQueue.remove(at: 0)
            }
        }
        else
        {
            do {
                      try unDuckAudio()
                  } catch {
                    print("error stop session")
                  }
        }
    }
    
    
    public func speechSynthesizer(_ synthesizer: AVSpeechSynthesizer, didPause utterance: AVSpeechUtterance) {
        print("AVSpeechSynthesizerDelegate: didPause")
    }
    
    public func speechSynthesizer(_ synthesizer: AVSpeechSynthesizer, didContinue utterance: AVSpeechUtterance) {
        print("AVSpeechSynthesizerDelegate: didContinue")
    }
    
    public func speechSynthesizer(_ synthesizer: AVSpeechSynthesizer, didCancel utterance: AVSpeechUtterance) {
        print("AVSpeechSynthesizerDelegate: didCancel")
    }
    
    public func speechSynthesizer(_ synthesizer: AVSpeechSynthesizer, willSpeakRangeOfSpeechString characterRange: NSRange, utterance: AVSpeechUtterance) {
        // This callback can be used with NSAttributedString if current text has to be highlighted.
        let subString = (utterance.speechString as NSString).substring(with: characterRange)
        print("AVSpeechSynthesizerDelegate: willSpeakRangeOfSpeechString with \(characterRange); text: \(subString)")
    }
    
}
