//
//  MyNavigationView+CLLocationManagerDelegate.swift
//  NavigationUI
//
//  Created by hassine othmane on 6/11/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import UIKit
import CoreLocation


extension MyNavigationView :CLLocationManagerDelegate
{
    
    public func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        
        if startDate == nil {
            startDate = Date()
        } else {
            print("elapsedTime:", String(format: "%.0fs", Date().timeIntervalSince(startDate)))
        }
        if startLocation == nil {
            startLocation = locations.first
        } else if let location = locations.last, lastLocation != nil  {
            updateGPS(position: location)
            
            if location.horizontalAccuracy  < 100
            {
                updateDistance(location:location)
                updateSpeed(location:location)
                
            }
        }
        if let location = locations.last ,  lastLocation == nil
        {
            if startLocation!.distance(from: location) < 50
            {
                lastLocation = locations.last
            }
        }
        else
        {
            lastLocation = locations.last
        }
        
    }
    public func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
        if (error as? CLError)?.code == .denied {
            manager.stopUpdatingLocation()
            manager.stopMonitoringSignificantLocationChanges()
        }
    }
    
}


extension MyNavigationView
{
    
    
    func updateGPS(position: CLLocation) -> Void {
        if (position.horizontalAccuracy < 0)
        {
            // No Signal
            gpsSignl_lbl.text = "GPS_NO_SIGNAL".localized
            gpsSignl_lbl.textColor = UIColor(red: 0.77, green: 0.25, blue: 0.85, alpha: 1.00)
            animateGpsSignlLbl()
            
        }
        else if (position.horizontalAccuracy > 70)
        {
            // Poor Signal
            gpsSignl_lbl.text = "GPS_SIGNAL_VERY_WEAK".localized
            gpsSignl_lbl.textColor = UIColor(red: 0.95, green: 0.35, blue: 0.32, alpha: 1.00)
            animateGpsSignlLbl()
        }
        else if (position.horizontalAccuracy > 15)
        {
            // Average Signal
            gpsSignl_lbl.text = "GPS_SIGNAL_WEAK".localized
            gpsSignl_lbl.textColor = UIColor(red: 0.97, green: 0.73, blue: 0.23, alpha: 1.00)
            animateGpsSignlLbl()
        }
        else
        {
            gpsSignl_lbl.text = ""
            gpsSignl_lbl.textColor = UIColor.green
            //animateGpsSignlLbl()
        }
    }
    
    func animateGpsSignlLbl(){
        gpsSignl_lbl.alpha = 0
        UIView.animate(withDuration: 0.3
            , animations: {
                self.gpsSignl_lbl.alpha = 1
        }) { _ in
        }
    }
    
    
    func updateDistance(location:CLLocation) -> Void
    {
        traveledDistance += lastLocation!.distance(from: location)
        print("Traveled Distance:",  traveledDistance)
        var distance = distanceFormatter.string(from: traveledDistance)
        var traveledDistanceSplited = distance.split(separator: " ")
        if  traveledDistanceSplited.count <= 1
        {
            traveledDistanceSplited = distance.split(separator: " ")
        }
        if traveledDistanceSplited.count > 1
        {
            if ( String(traveledDistanceSplited[0]).count == 1)
            {
                distanceValue_lbl.text = "\(traveledDistanceSplited[0]).0"
            }
            else
            {
                distanceValue_lbl.text = String(traveledDistanceSplited[0])
            }
            distanceTitle_lbl.text = String(traveledDistanceSplited[1] )
        }
        else if traveledDistanceSplited.count == 1
        {
            distanceValue_lbl.text = "\(traveledDistanceSplited[0]).0"
        }
    }
    
    func updateSpeed(location:CLLocation) -> Void
    {
        let distance =  Measurement(value: location.speed, unit: UnitSpeed.metersPerSecond)
        if distance.value > 0
        {
            var speedString : Measurement<Unit>
            if measurementSystem == .metric
            {
                let speedString = distance.converted(to: UnitSpeed.kilometersPerHour)
                let formatter = MeasurementFormatter()
                formatter.unitOptions = .providedUnit
                formatter.numberFormatter.maximumFractionDigits = 1
                let speedSplited = formatter.string(from: speedString).split(separator: " ")
                if speedSplited.count  > 1
                {
                    self.speedValue_lbl.text = String(speedSplited[0])
                    self.speedTitle_lbl.text = String(speedSplited[1])
                }
                else
                {
                    let speedSplited = formatter.string(from: speedString).split(separator: " ")
                    if speedSplited.count  > 1
                    {
                        self.speedValue_lbl.text = String(speedSplited[0])
                        self.speedTitle_lbl.text = String(speedSplited[1])
                    }
                }
            }
            else if measurementSystem == .imperial
            {
                let speedString = distance.converted(to: UnitSpeed.milesPerHour)
                let formatter = MeasurementFormatter()
                formatter.unitOptions = .providedUnit
                formatter.numberFormatter.maximumFractionDigits = 1
                let speedSplited = formatter.string(from: speedString).split(separator: " ")
                if speedSplited.count  > 1
                {
                    self.speedValue_lbl.text = String(speedSplited[0])
                    self.speedTitle_lbl.text = String(speedSplited[1])
                }
                else
                {
                    let speedSplited = formatter.string(from: speedString).split(separator: " ")
                    if speedSplited.count  > 1
                    {
                        self.speedValue_lbl.text = String(speedSplited[0])
                        self.speedTitle_lbl.text = String(speedSplited[1])
                    }
                }
            }
        }
        else
        {
            self.speedValue_lbl.text = "00.0"
            if measurementSystem == .metric
            {
                self.speedTitle_lbl.text = " km/h"
            }
            else if measurementSystem == .imperial
            {
                self.speedTitle_lbl.text = " mph"
            }
        }
    }
}
