//
//  MyNavigationView+MyNavigationViewProtocol.swift
//  NavigationUI
//
//  Created by hassine othmane on 3/30/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxNavigation
import MapboxDirections

import MapboxSpeech
import Speech


// MARK: - extending MyNavigationView to implement it's protocol
extension MyNavigationView: MyNavigationViewProtocol {
    
    func updateAndStartRoute(route: Route,routeOptions: RouteOptions, waypoints: [Waypoint]?) {
        
        
        if let layer = mapView_map.style!.layer(withIdentifier: "polyline-dash")
        {
            mapView_map.style!.removeLayer(layer)
        }
        
        
        
        self.calculatingRoute = false
        if let coordinates = route.shape?.coordinates
        {
            
            self.myRoute = route
            self.myRouteOption = routeOptions
            self.distanceRoute = route.distance
            DispatchQueue.main.async {
                self.drawRoute(coordinates: coordinates)
            }
            if let annotations = self.mapView_map.annotations
            {
                self.mapView_map.removeAnnotations(annotations)
            }
            self.gpxForTest?.append(contentsOf: coordinates)
            addPois()
            addSponsors()
            addSportSession()
            if MyNavigationView.testMode == true
            {
                addDirectionals()
            }
            
            printGpxTag()
            addFlags()
            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+2) {
                self.initNavigation(userRoute: self.myRoute!, routeOptions: routeOptions)
                
            }
        }
    }
    
    
    
    
    func showRoute(route:Route,routeOptions: RouteOptions, waypoints:[Waypoint]?) {
        self.calculatingRoute = false
        if let coordinates = route.shape?.coordinates
        {
            
            self.myRoute = route
            self.myRouteOption = routeOptions
            self.distanceRoute = route.distance
            DispatchQueue.main.async {
                self.drawRoute(coordinates: coordinates)
            }
            self.gpxForTest?.append(contentsOf: coordinates)
            addPois()
            addSponsors()
            addSportSession()
            if MyNavigationView.testMode == true
                       {
                           addDirectionals()
                       }
            printGpxTag()
            addFlags()
        }
    }
    
    func showAlertSystemMessage(title: String, message: String) {
        
    }
    
    func showRouteToStartpoint(route: Route,routeOptions: RouteOptions) {
        if let coordinates = route.shape?.coordinates
        {
            drawWayToRoute(coordinates: coordinates)
            self.wayToRoute = route
            self.distanceWayToRoute = route.distance
            gpxForTest = coordinates
            animatePolyline(allCoordinates: coordinates)
            initNavigation(userRoute: route, routeOptions: routeOptions)
            self.initRouteInformations()
            LocationService.shared.startUpdatelocation()
            flutterOnNavigationStarted()
            DispatchQueue.main.asyncAfter(deadline: DispatchTime.now()+3) {
                if let routeInformation = self.routeInformation
                {
                    if routeInformation.profileIdentifier == "walking"
                    {
                        self.presenter.calculateRoute(points: self.routeCoordinates!, locale: self.languageAndCountry,measurementSystem: self.measurementSystem, objectFor: self, profileIdentifier: MBDirectionsProfileIdentifier.walking, firstCalculation: true)
                    } else if routeInformation.profileIdentifier == "cycling"
                    {
                        self.presenter.calculateRoute(points: self.routeCoordinates!, locale: self.languageAndCountry,measurementSystem: self.measurementSystem, objectFor: self, profileIdentifier: MBDirectionsProfileIdentifier.cycling, firstCalculation: true)
                    }
                }
            }
        }
    }
}

extension MyNavigationView {
    
    func polygonCircleForCoordinate(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) {
        let degreesBetweenPoints = 8.0
        //45 sides
        let numberOfPoints = floor(360.0 / degreesBetweenPoints)
        let distRadians: Double = withMeterRadius / 6371000.0
        // earth radius in meters
        let centerLatRadians: Double = coordinate.latitude * Double.pi / 180
        let centerLonRadians: Double = coordinate.longitude * Double.pi / 180
        var coordinates = [CLLocationCoordinate2D]()
        //array to hold all the points
        for index in 0 ..< Int(numberOfPoints) {
            let degrees: Double = Double(index) * Double(degreesBetweenPoints)
            let degreeRadians: Double = degrees * Double.pi / 180
            let pointLatRadians: Double = asin(sin(centerLatRadians) * cos(distRadians) + cos(centerLatRadians) * sin(distRadians) * cos(degreeRadians))
            let pointLonRadians: Double = centerLonRadians + atan2(sin(degreeRadians) * sin(distRadians) * cos(centerLatRadians), cos(distRadians) - sin(centerLatRadians) * sin(pointLatRadians))
            let pointLat: Double = pointLatRadians * 180 / Double.pi
            let pointLon: Double = pointLonRadians * 180 / Double.pi
            let point: CLLocationCoordinate2D = CLLocationCoordinate2DMake(pointLat, pointLon)
            coordinates.append(point)
        }
        let polygon = MGLPolygon(coordinates: &coordinates, count: UInt(coordinates.count))
        
        self.mapView_map.addAnnotation(polygon)
    }
    
    func calculateCoordinates(coordinate: CLLocationCoordinate2D, withMeterRadius: Double) -> [CLLocationCoordinate2D] {
        let degreesBetweenPoints = 8.0
        //45 sides
        let numberOfPoints = floor(360.0 / degreesBetweenPoints)
        let distRadians: Double = withMeterRadius / 6371000.0
        // earth radius in meters
        let centerLatRadians: Double = coordinate.latitude * Double.pi / 180
        let centerLonRadians: Double = coordinate.longitude * Double.pi / 180
        var coordinates = [CLLocationCoordinate2D]()
        //array to hold all the points
        for index in 0 ..< Int(numberOfPoints) {
            let degrees: Double = Double(index) * Double(degreesBetweenPoints)
            let degreeRadians: Double = degrees * Double.pi / 180
            let pointLatRadians: Double = asin(sin(centerLatRadians) * cos(distRadians) + cos(centerLatRadians) * sin(distRadians) * cos(degreeRadians))
            let pointLonRadians: Double = centerLonRadians + atan2(sin(degreeRadians) * sin(distRadians) * cos(centerLatRadians), cos(distRadians) - sin(centerLatRadians) * sin(pointLatRadians))
            let pointLat: Double = pointLatRadians * 180 / Double.pi
            let pointLon: Double = pointLonRadians * 180 / Double.pi
            let point: CLLocationCoordinate2D = CLLocationCoordinate2DMake(pointLat, pointLon)
            coordinates.append(point)
        }
        return coordinates
    }
    
    func drawRoute(coordinates:[CLLocationCoordinate2D]) -> Void {
        DispatchQueue.main.async {
            guard coordinates.count > 0 else {return}
            var cour = coordinates
            let polyline = MGLPolylineFeature.init(coordinates: &cour, count: UInt(cour.count))
            if let source = self.mapView_map.style?.source(withIdentifier: "route-source") as? MGLShapeSource {
                source.shape = polyline
            } else {
                let source = MGLShapeSource.init(identifier: "route-source", features: [polyline], options: nil)
                let lineStyle = MGLLineStyleLayer.init(identifier: "route-style", source: source)
                let borderStyle = MGLLineStyleLayer.init(identifier: "route-style-border", source: source)
                var color = Style.shared.primaryColor
               if(self.whiteLabel?.isWhiteLabel ?? false) {
                   color = UIColor(hex: "#2196f3FF")! //blue universel color
               }
                lineStyle.lineColor = NSExpression(forConstantValue: color)
                lineStyle.lineWidth = NSExpression(forConstantValue: 5)
                borderStyle.lineColor = NSExpression(forConstantValue: UIColor.white)
                borderStyle.lineWidth = NSExpression(forConstantValue: 10)
                self.mapView_map.style!.addSource(source)
                self.mapView_map.style!.insertLayer(borderStyle, below: self.mapView_map.style!.layer(withIdentifier: "com.mapbox.annotations.points")!)
                self.mapView_map.style!.insertLayer(lineStyle, above: borderStyle)
            }
        }
    }
    
    func drawWayToRoute(coordinates: [CLLocationCoordinate2D]) {
        guard coordinates.count > 0 else { return }
        var routeCoordinates = coordinates
        let polyline = MGLPolylineFeature(coordinates: &routeCoordinates, count: UInt(coordinates.count))
        guard let style = self.mapView_map.style else { return }
        if let source = mapView_map.style?.source(withIdentifier: "WayToRoute-source") as? MGLShapeSource {
            polylineSource = source
            polylineSource!.shape = polyline
        } else {
            let source =  MGLShapeSource(identifier: "WayToRoute-source", shape: nil, options: nil)
            style.addSource(source)
            polylineSource = source
            let dashedLayer = MGLLineStyleLayer(identifier: "polyline-dash", source: source)
            dashedLayer.lineJoin = NSExpression(forConstantValue: "round")
            dashedLayer.lineCap = NSExpression(forConstantValue: "round")
            var color = Style.shared.primaryColor
            if(self.whiteLabel?.isWhiteLabel ?? false) {
                color = UIColor(hex: "#2196f3FF")! //blue universel color
            }
            dashedLayer.lineColor = NSExpression(forConstantValue: color)
            dashedLayer.lineOpacity = NSExpression(forConstantValue: 1)
            dashedLayer.lineWidth = NSExpression(format: "mgl_interpolate:withCurveType:parameters:stops:($zoomLevel, 'linear', nil, %@)",
                                                 [14: 3, 18: 20])
            dashedLayer.lineDashPattern = NSExpression(forConstantValue: [0, 1.5])
            mapView_map.style!.insertLayer(dashedLayer, below: mapView_map.style!.layer(withIdentifier: "com.mapbox.annotations.points")!)
        }
        
    }
    
    func addPois() -> Void {
        if let pois =  pois
        {
            for (index,poi) in pois.enumerated()
            {
                let annotation = MyCustomPointAnnotation()
                if let first = poi.coordinatesGPX.first
                {
                    annotation.coordinate = first
                    annotation.title = poi.key
                    annotation.subtitle = "\(index+1)"
                    annotation.category = poi.type
                    annotation.willUseImage = true
                    mappedAnnotation[poi.name] = index
                    self.mapView_map.addAnnotation(annotation)
                    if MyNavigationView.testMode == true
                    {
                        polygonCircleForCoordinate(coordinate: annotation.coordinate, withMeterRadius: Double(poi.range))
                    }
                    
                }
                
            }
        }
    }
    
    func addSponsors() -> Void {
        if let route = routeInformation
        {
            for (index,sponsor) in route.sponsors.enumerated()
            {
                for coordinate in sponsor.coordinates
                {
                    let annotation = MyCustomPointAnnotation()
                    annotation.coordinate = coordinate
                    annotation.title = sponsor.key
                    //annotation.subtitle = "\(index+1)"
                    annotation.category = .sponsor
                    annotation.willUseImage = true
                    mappedSponsors[sponsor.key] = index
                    self.mapView_map.addAnnotation(annotation)
                }
            }
        }
    }

    func addSportSession() -> Void {
        if let sportsSession =  routeInformation?.sportSession {
            for (index,sportSession) in sportsSession.enumerated() {
                let annotation = MyCustomPointAnnotation()
                if let first = sportSession.gps.first {
                    annotation.coordinate = first
                    annotation.category = .sport
                    annotation.title = sportSession.key
                    annotation.willUseImage = true
                    mappedAnnotation[sportSession.key!] = index
                    self.mapView_map.addAnnotation(annotation)
                    if MyNavigationView.testMode == true {
                        polygonCircleForCoordinate(coordinate: annotation.coordinate, withMeterRadius: Double(sportSession.radius!))
                    }
                }
            }
        }
    }
    
    func addDirectionals() -> Void {
        for (index,poi) in directionals.enumerated()
        {
            
            if poi.isBloquant == true
            {
                let annot = BloquantPointAnnotation()
                annot.coordinate = poi.coordinatesGPX.first!
                annot.title = poi.name
                annot.subtitle = poi.description
                mapView_map.addAnnotation(annot)
                var coordinates = calculateCoordinates(coordinate: annot.coordinate, withMeterRadius: Double(poi.range))
                let polygon = BloquantPointPolygon(coordinates: &coordinates, count: UInt(coordinates.count))
                mappedAnnotation[poi.name] = index
                self.mapView_map.addAnnotation(polygon)
                
            }
            else
            {
                let annot = DirectionalPointAnnotation()
                annot.coordinate = poi.coordinatesGPX.first!
                annot.title = poi.name
                annot.subtitle = poi.description
                mapView_map.addAnnotation(annot)
                var coordinates = calculateCoordinates(coordinate: annot.coordinate, withMeterRadius:  Double(poi.range))
                let polygon = DirectionalPointPolygon(coordinates: &coordinates, count: UInt(coordinates.count))
                mappedAnnotation[poi.name] = index
                self.mapView_map.addAnnotation(polygon)
            }
        }
    }
    
    func addFlags() -> Void {
        if let route = myRoute
        {
            let annotation = MyCustomFlagPointAnnotation()
            if let first = route.shape?.coordinates.first
            {
                annotation.coordinate = first
                annotation.willUseImage = true
                annotation.isStartFlag = true
                mapView_map.addAnnotation(annotation)
            }
            
            let lastAnnotation = MyCustomFlagPointAnnotation()
            if let last = route.shape?.coordinates.last
            {
                lastAnnotation.coordinate = last
                lastAnnotation.willUseImage = true
                lastAnnotation.isStartFlag = false
                mapView_map.addAnnotation(lastAnnotation)
            }
        }
    }
    
    func animatePolyline(allCoordinates:  [CLLocationCoordinate2D]) {
        currentIndex = 1
        timerPolyAnimation = Timer.scheduledTimer(timeInterval: 0.01, target: self, selector: #selector(tick(sender:)), userInfo:  allCoordinates, repeats: true)
    }
    
    @objc func tick(sender:  Timer) {
        if let allCoordinates = sender.userInfo as? [CLLocationCoordinate2D]
        {
            if currentIndex > allCoordinates.count {
                timerPolyAnimation?.invalidate()
                timerPolyAnimation = nil
                return
            }
            let coordinates = Array(allCoordinates[0..<currentIndex])
            updatePolylineWithCoordinates(coordinates: coordinates)
            currentIndex += 1
        }
    }
    
    func updatePolylineWithCoordinates(coordinates: [CLLocationCoordinate2D]) {
        var mutableCoordinates = coordinates
        let polyline = MGLPolylineFeature(coordinates: &mutableCoordinates, count: UInt(mutableCoordinates.count))
        polylineSource?.shape = polyline
    }
    
    func printGpxTag()
    {
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        MyUtils.generateGpxTags(coordinatesArray: gpxForTest!)
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
        print("        ")
    }
}
