//
//  MyNavigationView+ProgressRouteNotifications.swift
//  NavigationUI
//
//  Created by Hassine on 04/06/2020.
//  Copyright © 2020 com.mile. All rights reserved.
//

import MapboxCoreNavigation
import MapboxDirections

extension MyNavigationView
{
    // MARK: - Notifications
    func resumeNotifications() {
        NotificationCenter.default.addObserver(self, selector: #selector(progressDidChange(_ :)), name: .routeControllerProgressDidChange, object: nil)
       NotificationCenter.default.addObserver(self, selector: #selector(rerouted(_:)), name: .routeControllerDidReroute, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(updateInstructionsBanner(notification:)), name: .routeControllerDidPassVisualInstructionPoint, object: navigationService.router)
    }
    
    func suspendNotifications() {
        NotificationCenter.default.removeObserver(self, name: .routeControllerProgressDidChange, object: nil)
        NotificationCenter.default.removeObserver(self, name: .routeControllerWillReroute, object: nil)
        NotificationCenter.default.removeObserver(self, name: .routeControllerDidPassVisualInstructionPoint, object: nil)
    }
    
    // Notifications sent on all location updates
    @objc func progressDidChange(_ notification: NSNotification) {
        
        let routeProgress = notification.userInfo![RouteController.NotificationUserInfoKey.routeProgressKey] as! RouteProgress
        _ = notification.userInfo![RouteController.NotificationUserInfoKey.locationKey] as! CLLocation
        
        // Add maneuver arrow
        if routeProgress.currentLegProgress.followOnStep != nil {
            mapView_map.addArrow(route: routeProgress.route, legIndex: routeProgress.legIndex, stepIndex: routeProgress.currentLegProgress.stepIndex + 1)
        } else {
            mapView_map.removeArrow()
        }
        
        // Update the top banner with progress updates
        updateDistance(for: routeProgress.currentLegProgress.currentStepProgress)
        
        // update Polyline
        if routeProgress.currentLegProgress.currentStepProgress.distanceRemaining < 10
            
        {
            var coordinates = [CLLocationCoordinate2D]()
            if let myCoordinate = navigationService.locationManager.location?.coordinate
            {
             //   coordinates.append(myCoordinate)
            }
            
            for step in  navigationService!.routeProgress.remainingSteps
            {
                for coordinate in step.shape!.coordinates
                {
                    coordinates.append(coordinate)
                }
            }
            if (wayToRoute != nil)
            {
                drawWayToRoute(coordinates: coordinates)
            }
            else
            {
                drawRoute(coordinates: coordinates)
            }
        }
    }
    
    public func updateDistance(for currentStepProgress: RouteStepProgress) {
        let distanceRemaining = currentStepProgress.distanceRemaining
        DispatchQueue.main.async {
            self.distanceInstruction_lbl.text =  self.distanceFormatter.string(from: distanceRemaining)
        }
    }
    
    
    @objc func updateInstructionsBanner(notification: NSNotification) {
        guard let routeProgress = notification.userInfo?[RouteController.NotificationUserInfoKey.routeProgressKey] as? RouteProgress else { return }
        update(for: routeProgress.currentLegProgress.currentStepProgress.currentVisualInstruction)
    }
    
    
    public func update(for instruction: VisualInstructionBanner?) {
        let secondaryInstruction = instruction?.secondaryInstruction
        DispatchQueue.main.async {
            self.primaryInstruction_lbl.numberOfLines = secondaryInstruction == nil ? 2 : 1
            self.primaryInstruction_lbl.text = instruction?.primaryInstruction.text
            self.maneuverView.visualInstruction = instruction?.primaryInstruction
            self.maneuverView.drivingSide = instruction?.drivingSide ?? .right
            self.secondaryInstruction_lbl.text = instruction?.secondaryInstruction?.text
        }
        // secondaryLabel.instruction = secondaryInstruction
    }
    
    // Fired when the user is no longer on the route.
    // Update the route on the map.
    @objc func rerouted(_ notification: NSNotification) {
        // secondaryInstruction_lbl.text = "rerouted"

        if wayToRoute != nil
        {



            drawWayToRoute(coordinates: navigationService!.route.shape!.coordinates)
            self.distanceWayToRoute = navigationService.route.distance
            wayToRoute = navigationService!.route
        }
        else
        {
            drawRoute(coordinates: navigationService!.route.shape!.coordinates)
            self.distanceRoute = navigationService.route.distance
        }
    }
}

