//
//  MyNavigationView.swift
//  NavigationUI
//
//  Created by hassine othmane on 3/25/20.
//  Copyright © 2020 com.mile. All rights reserved.
//

import UIKit

import MapboxNavigation

import MapboxDirections
import MapboxCoreNavigation

import Speech
import LiquidFloatingActionButton
import MaterialComponents.MaterialButtons

public class MyNavigationView: UIViewController {
    
    //
    public static var testMode = false
    public static var mutedZone : Bool = false
    // MARK: - Strings
    let FINISH_TITLE_BTN = "FINISH_TITLE_BTN".localized(tableName: "NavigationUI", anyClass: MyNavigationView.self)
    
    
    // MARK: - IBOutlet
    
    // Top Banner View
    @IBOutlet weak var statusBarBanner_view         : UIView!
    @IBOutlet weak var containerBanner_view         : UIView!
    @IBOutlet weak var containerSecondBanner_view   : UIView!
    @IBOutlet weak var maneuverView                 : ManeuverView!
    @IBOutlet weak var distanceInstruction_lbl      : DistanceLabel!
    @IBOutlet weak var primaryInstruction_lbl       : UILabel!
    @IBOutlet weak var secondaryInstruction_lbl     : UILabel!
    // Map
    @IBOutlet weak var center_btn                   : UIButton!
    @IBOutlet weak var mapView_map                  : NavigationMapView!
    @IBOutlet weak var containerMap_view            : UIView!
    var more_btn                                    : LiquidFloatingActionButton!
    var spotify_btn                                 : LiquidFloatingActionButton!
    var fab                                         : MDCFloatingButton?
    // Bottom View
    @IBOutlet weak var informationsContainer_view   : UIView!
    @IBOutlet weak var timerValue_lbl               : UILabel!
    @IBOutlet weak var distanceValue_lbl            : UILabel!
    @IBOutlet weak var distanceTitle_lbl            : UILabel!
    @IBOutlet weak var speedValue_lbl               : UILabel!
    @IBOutlet weak var speedTitle_lbl               : UILabel!
    @IBOutlet weak var gpsSignl_lbl: UILabel!
    //Bottom View : progressView
    @IBOutlet weak var containerProgress_view       : UIView!
    @IBOutlet weak var containerView_view           : UIView!
    @IBOutlet weak var progressRoute_ProgressBar    : UIProgressView!
    @IBOutlet weak var progressRouteIcon_imageView  : UIImageView!
    //Bottom View : slide Button
    @IBOutlet weak var pauseRun_slideBtn            : MMSlidingButton!
    @IBOutlet weak var dotIcon_imageView            : UIImageView!
    @IBOutlet weak var dotSlide_view                : UIView!
    
    
    // MARK: - vars
    
    // Params
    var routeInformation    : RouteInformation?
    var pois                : [PointOfInterest]?
    var directionals        : [PointOfInterest] = [PointOfInterest]()
    var user                : MyUser?
    var measurementSystem   : MeasurementSystem!
    var routeCoordinates    : [CLLocationCoordinate2D]?
    
    // Audio
    var synthesizer         : AVSpeechSynthesizer = AVSpeechSynthesizer()
    var voiceController     : CustomVoiceController?
    var isMuted = false
   
    
    // Stats Run informations
    var startDate           : Date!
    var traveledDistance    : Double = 0
    var runTime_seconds     : Int    = 0
    
    // Locations
    var startLocation       : CLLocation?
    var lastLocation        : CLLocation?
    var previousLocation    : CLLocation?
    
    // Map UI
    var wayToRoute          : Route?
    var myRoute             : Route?
    var myRouteOption       : RouteOptions?
    var polylineSource      : MGLShapeSource?
    var coverView           : UIView!
    
    // Navigation
    var navigationService   : NavigationService!
    var calculatingRoute    = false
    var navigationFinished  = false
    
    // Languages & Units
    var languageRessource   : LanguageRessource! //senteces of naviagtion (this depend of Language on settings not the langauge of the IPhone )
    var languageAndCountry  = "en-US"
    
    // Others
    var timerCourse         : Timer = Timer()
    var timerPolyAnimation  : Timer?
    var distanceWayToRoute  : Double? //in meter
    var distanceRoute       : Double? //in meter
    var lastDistanceRemaining: Double? // this distance represnt the rest of distance , we will use it to comapre with (navigationService.route.distance-distanceWayToRoute)
    let distanceFormatter   : DistanceFormatter = DistanceFormatter()
    var gpxForTest          : [CLLocationCoordinate2D]?
    var currentIndex        = 1 // index of coodinate (used in animation of polyline)
    var mappedAnnotation    : [String:Int] = [String:Int]()
    var mappedSponsors      : [String:Int] = [String:Int]()
    var arrayStringPOIQueue : Array<String> = []
    var presenter           : MyNavigationPresenterProtocol!
    var cellsFloatingButton : [LiquidFloatingCell] = []

    // MARK: - Clousres
    public var back_closure :(()->())?
    
    //Flutter
    var navigationMode = ""
    var channel: FlutterMethodChannel?
    var isMapReady = false
    var mapReadyResult: FlutterResult?
    var registrar: FlutterPluginRegistrar!
    var viewId: Int64 = -1
    var whiteLabel: WhiteLabel?
    var offlineDatas: OfflineDatas?
    var gpsColor = ""
    var languages: [String]? = ["EN"]
    var distanceUnit = ""
    var speedUnit = ""
    var locomotion = ""
    var downloadForOffline = false

    public override func viewDidLoad() {
        super.viewDidLoad()
        initFlutterChannel()
        initMeasurementSystem()
        checkBestLanguage()
        initLanguageRessources() // get ressoureces ( depend Languages settings)
        initUI()
        initLocationManager()
        initMap()
        initAudioSpeech() // we will init the Speech Synth , the voice Controller 'Amazon' we will init it later when we have the navigation service
        presenter = MyNavigationPresenter(view: self)
    }
    

     // MARK: - IBAction
    @IBAction func mute_action(_ sender: Any) {
        
        if isMuted
        {


            self.voiceController?.audioPlayer?.volume = NavigationSettings.shared.voiceVolume
            PlayerManager.shared.volume = NavigationSettings.shared.voiceVolume

        }
        else
        {
            self.voiceController?.audioPlayer?.volume = 0
            PlayerManager.shared.volume = 0
        }
        isMuted = !isMuted
    }
    
    @IBAction func centerMap_action(_ sender: Any) {
        DispatchQueue.main.async {
          
            self.mapView_map.zoomLevel = 18
              self.center_btn.isHidden = true
            self.mapView_map.setUserTrackingMode(.followWithHeading, animated: true) {
               
            }
           
        }
        
    }

    func setRoute(routeCoordinates:[CLLocationCoordinate2D]?) -> Void {
        self.routeCoordinates = routeCoordinates
    }
    
    func set(object: MyNavigationEntity) {
        
    }
    
}
