part of v4_navigation_lib;

typedef void OnNavigationFinished(RouteFinished,);
typedef void OnActivePOI(ActivePOI);
typedef void OnNavigationStarted(bool);
typedef void OnEmergencySmsClick();
typedef void OnReportClick();
typedef void OnMusicClick();
typedef void OnSponsorClick(String);
typedef void OnOfflineDownloadFinished(OfflineDatas);
typedef void OnActiveSmartFact();
typedef void OnActiveSportSession(String);
typedef void OnMapBoxSpeak(bool);

class MileNavigationController extends ChangeNotifier {

  final MethodChannel _channel;

  final OnNavigationFinished onNavigationFinished;
  final OnActivePOI onActivePOI;
  final OnNavigationStarted onNavigationStarted;
  final OnEmergencySmsClick onEmergencySmsClick;
  final OnReportClick onReportClick;
  final OnMusicClick onMusicClick;
  final OnSponsorClick onSponsorClick;
  final OnOfflineDownloadFinished onDownloadOfflineFinished;
  final OnActiveSmartFact onActiveSmartFact;
  final OnActiveSportSession onActiveSportSession;
  final OnMapBoxSpeak onMapBoxSpeak;

  final int _id;

  static Future<MileNavigationController> init(
      int id,
      OnNavigationFinished onNavigationFinished,
      OnActivePOI onActivePOI,
      OnNavigationStarted onNavigationStarted,
      OnEmergencySmsClick onEmergencySmsClick,
      OnReportClick onReportClick,
      OnMusicClick onMusicClick,
      OnSponsorClick onSponsorClick,
      OnOfflineDownloadFinished onOfflineDownloadFinished,
      OnActiveSmartFact onActiveSmartFact,
      OnActiveSportSession onActiveSportSession,
      OnMapBoxSpeak onMapBoxSpeak,
      ) async {

    assert(id != null);
    final MethodChannel methodChannel = MethodChannel('flutter_mapbox_navigation_$id');
    await methodChannel.invokeMethod('map#waitForMap');
    return MileNavigationController._(
        id,
        methodChannel,
        onNavigationFinished,
        onActivePOI,
        onNavigationStarted,
        onEmergencySmsClick,
        onReportClick,
        onMusicClick,
        onSponsorClick,
        onOfflineDownloadFinished,
        onActiveSmartFact,
        onActiveSportSession,
        onMapBoxSpeak,
    );
  }

  MileNavigationController._(this._id, MethodChannel channel,  this.onNavigationFinished, this.onActivePOI, this.onNavigationStarted, this.onEmergencySmsClick, this.onReportClick, this.onMusicClick, this.onSponsorClick, this.onDownloadOfflineFinished, this.onActiveSmartFact, this.onActiveSportSession, this.onMapBoxSpeak) :
        assert(_id != null),
        assert(channel != null), _channel = channel {

    _channel.setMethodCallHandler(_handleMethodCall);
  }

  Future<dynamic> _handleMethodCall(MethodCall call) async {
    switch (call.method) {
      case 'onNavigationFinished':
        RouteFinished routeFinished = RouteFinished.fromDynamicData(call.arguments['isFinished']);
        if (onNavigationFinished != null) {
          onNavigationFinished(routeFinished);
        }
        break;
      case 'onActivePOI':
        ActivePOI poi = ActivePOI.fromDynamicData(call.arguments['activePOI']);
        if (onActivePOI != null) {
          onActivePOI(poi);
        }
        break;
      case 'onNavigationStarted':
        final bool isHidden = call.arguments['isStarted'];
        if (onNavigationStarted != null) {
          onNavigationStarted(isHidden);
        }
        break;
      case 'onDownloadOfflineFinished':
        OfflineDatas offlineDatas = OfflineDatas.fromDynamicData(call.arguments['onDownloadOfflineFinished']);
        if (onDownloadOfflineFinished != null) {
          onDownloadOfflineFinished(offlineDatas);
        }
        break;
      case 'onEmergencySmsClick':
        if (onEmergencySmsClick != null) {
          onEmergencySmsClick();
        }
        break;
      case 'onReportClick':
        if (onReportClick != null) {
          onReportClick();
        }
        break;
      case 'onMusicClick':
        if (onMusicClick != null) {
          onMusicClick();
        }
        break;
      case 'onActiveSmartFact':
        if (onActiveSmartFact != null) {
          onActiveSmartFact();
        }
        break;
      case 'onActiveSportSession':
        if (onActiveSportSession != null) {
          final String ssId = call.arguments['activeSportSession'];
          onActiveSportSession(ssId);
        }
        break;
      case 'onMapBoxSpeak':
        if (onMapBoxSpeak != null) {
          final bool speaking = call.arguments['speaking'];
          onMapBoxSpeak(speaking);
        }
        break;
      case 'onSponsorClick':
        if (onSponsorClick != null) {
          final String sponsorId = call.arguments['sponsorId'];
          onSponsorClick(sponsorId);
        }
        break;
      default:
        throw MissingPluginException();
    }
  }

  Future<void> playTTS(String description) async {
    return await _channel.invokeMethod('playTTS', <String, dynamic>{
      'description': description,
    });
  }

  Future<void> playInstructionTTS(String description) async {
    return await _channel.invokeMethod('playInstructionTTS', <String, dynamic>{
      'description': description,
    });
  }

  Future<void> stopTTS() async {
    return await _channel.invokeMethod('stopTTS', <String, dynamic>{});
  }
}
